package lib;

import core.deTool;
import core.TraceData;
import static java.lang.Math.*;
import java.util.logging.Level;
import java.util.logging.Logger;
import math.Complex;
import math.MathEval;
import umontreal.iro.lecuyer.probdist.NormalDist;

public final class Profiles { 

    
    // GetData0 : parse int type variable from array and return the value
    public int GetData0(int pIndex, int vIndex, int iIndex, int dIndex) { 
        String data = deTool.ProfileArray.DataArray[pIndex].rawData[vIndex][iIndex];
        int pointer = data.indexOf(";");
        String thisVar;
        int index = 0;
        while (pointer > 0) {
        thisVar = data.substring(0, pointer);
            if(index == dIndex) {
                return Integer.parseInt(thisVar);
            } else {
                if (data.length() > pointer) {
                    data = data.substring(pointer + 1);
                    pointer = data.indexOf(";");
                } else {
                    pointer = -1;
                }
            index++;
            }
        }
        return -1;
    }
    
    // GetData1 : parse double type variable from array and return the value
    public double GetData1(int pIndex, int vIndex, int iIndex, int dIndex) { 
        String data = deTool.ProfileArray.DataArray[pIndex].rawData[vIndex][iIndex];
        int pointer = data.indexOf(";");
        String thisVar;
        int index = 0;
        while (pointer > 0) {
        thisVar = data.substring(0, pointer);
            if(index == dIndex) {
                return Double.parseDouble(thisVar);
            } else {
                if (data.length() > pointer) {
                    data = data.substring(pointer + 1);
                    pointer = data.indexOf(";");
                } else {
                    pointer = -1;
                }
            index++;
            }
        }
        return -1.0;
    }
    
    // GetData2 : parse Complex type variable from array and return the value
    public Complex GetData2(int pIndex, int vIndex, int iIndex, int dIndex) { 
        String data = deTool.ProfileArray.DataArray[pIndex].rawData[vIndex][iIndex];
        int pointer = data.indexOf(";");
        String thisVar;
        int index = 0;
        while (pointer > 0) {
        thisVar = data.substring(0, pointer);
            if(index == dIndex) {
                return new Complex(thisVar);
            } else {
                if (data.length() > pointer) {
                    data = data.substring(pointer + 1);
                    pointer = data.indexOf(";");
                } else {
                    pointer = -1;
                }
            index++;
            }
        }
        return new Complex();
    }

    public int mandel_complex(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 0;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;
            int basinindex;
            if (deTool.ImageArray.ParentFrameIndex[TargetFrameIndex] > -1) {
                basinindex = deTool.ImageArray.ParentFrameIndex[TargetFrameIndex];
            } else {
                basinindex = TargetFrameIndex;
            }
            int[] detectx0 = new int[MaxIter];
            int[] detecty0 = new int[MaxIter];
            double[] detectxval0 = new double[MaxIter];
            double[] detectyval0 = new double[MaxIter];

            // initialize parameters
            double n;
            double big;
            if (InheritParameters) {
                n=myMath.getVariable("n");
                big=myMath.getVariable("big");
            } else {
                n=2;
                big=BIG;
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);
            Complex p1=new Complex(0,-1);

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex Z=new Complex(xval,yval);
                    Complex C=Z;
                    double test=0;

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'ExitIf' Directive: exitif:SqNorm(Z)>big;palette:0;split:GetIm(Z);palette:1,2
                            if (dummyC.SqNorm(Z)>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                SplitResult[0]=dummyC.GetIm(Z);
                                break Block0;
                            }

                            // 'Set' Directive: set:Z=Z^n+C
                            Z=Z.Pow(n).Add(C);

                            // 'Detect' Directive: detect:xval:GetRe(Z);yval:GetIm(Z);palette:3
                            if (DCD) {
                                detectxval0[iter] = dummyC.GetRe(Z);
                                detectyval0[iter] = dummyC.GetIm(Z);
                                detectx0[iter] = deTool.GetXFromVal(basinindex, detectxval0[iter]);
                                detecty0[iter] = deTool.GetYFromVal(basinindex, detectyval0[iter]);
                            }

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                            // cycle and basin detection code for block 0
                            if (!BlockExit[0]) {
                                switch (CDM) {
                                    case 0: { // pixel level precision
                                        if (deTool.DetectCB(TargetFrameIndex, 0, x, y, basinindex, BDM, detectx0, detecty0, iter)) {
                                            BlockExit[0] = true;
                                            Palette[0]=3;
                                            SplitResult[0]=dummyC.GetIm(Z);
                                        }
                                    }
                                    break;
                                    case 1: { // Double value level precision
                                        if (deTool.DetectCB(TargetFrameIndex, 0, x, y, basinindex, BDM, detectxval0, detectyval0, iter)) {
                                            BlockExit[0] = true;
                                            Palette[0]=3;
                                            SplitResult[0]=dummyC.GetIm(Z);
                                        }
                                    }
                                    break;
                                }
                            }
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[1]=dummyC.GetRe(p1);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[1]=dummyC.GetIm(p1);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData mandel_complex_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 0;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double n;
      double big;
      if (InheritParameters) {
          n=myMath.getVariable("n");
          big=myMath.getVariable("big");
      } else {
          n=2;
          big=BIG;
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex Z=new Complex(xval,yval);
      Complex C=Z;
      double test=0;
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(Z), dummyC.GetIm(Z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'ExitIf' Directive: exitif:SqNorm(Z)>big;palette:0;split:GetIm(Z);palette:1,2
              if (dummyC.SqNorm(Z)>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:Z=Z^n+C
              Z=Z.Pow(n).Add(C);

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int mandel_complex_julia(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 1;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;
            int basinindex;
            if (deTool.ImageArray.ParentFrameIndex[TargetFrameIndex] > -1) {
                basinindex = deTool.ImageArray.ParentFrameIndex[TargetFrameIndex];
            } else {
                basinindex = TargetFrameIndex;
            }
            int[] detectx0 = new int[MaxIter];
            int[] detecty0 = new int[MaxIter];
            double[] detectxval0 = new double[MaxIter];
            double[] detectyval0 = new double[MaxIter];

            // initialize parameters
            double n;
            double big;
            Complex C;
            if (InheritParameters) {
                n=myMath.getVariable("n");
                big=myMath.getVariable("big");
                x_value=myMath.getVariable("C_x");
                y_value=myMath.getVariable("C_y");
                C=new Complex(x_value, y_value);
            } else {
                n=2;
                big=100;
                C=new Complex(cX,cY);
            }

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex Z=new Complex(xval,yval);

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Detect' Directive: detect:xval:GetRe(Z);yval:GetIm(Z);palette:3
                            if (DCD) {
                                detectxval0[iter] = dummyC.GetRe(Z);
                                detectyval0[iter] = dummyC.GetIm(Z);
                                detectx0[iter] = deTool.GetXFromVal(basinindex, detectxval0[iter]);
                                detecty0[iter] = deTool.GetYFromVal(basinindex, detectyval0[iter]);
                            }

                            // 'ExitIf' Directive: exitif:SqNorm(Z)>big;palette:0;split:GetIm(Z);palette:1,2
                            if (dummyC.SqNorm(Z)>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                SplitResult[0]=dummyC.GetIm(Z);
                                break Block0;
                            }

                            // 'Set' Directive: set:Z=(Z^n)+C
                            Z=(Z.Pow(n)).Add(C);

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                            // cycle and basin detection code for block 0
                            if (!BlockExit[0]) {
                                switch (CDM) {
                                    case 0: { // pixel level precision
                                        if (deTool.DetectCB(TargetFrameIndex, 0, x, y, basinindex, BDM, detectx0, detecty0, iter)) {
                                            BlockExit[0] = true;
                                            Palette[0]=3;
                                            SplitResult[0]=dummyC.GetIm(Z);
                                        }
                                    }
                                    break;
                                    case 1: { // Double value level precision
                                        if (deTool.DetectCB(TargetFrameIndex, 0, x, y, basinindex, BDM, detectxval0, detectyval0, iter)) {
                                            BlockExit[0] = true;
                                            Palette[0]=3;
                                            SplitResult[0]=dummyC.GetIm(Z);
                                        }
                                    }
                                    break;
                                }
                            }
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData mandel_complex_julia_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 1;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double n;
      double big;
      Complex C;
      if (InheritParameters) {
          n=myMath.getVariable("n");
          big=myMath.getVariable("big");
          x_value=myMath.getVariable("C_x");
          y_value=myMath.getVariable("C_y");
          C=new Complex(x_value, y_value);
      } else {
          n=2;
          big=100;
          C=new Complex(cX,cY);
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex Z=new Complex(xval,yval);
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(Z), dummyC.GetIm(Z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'ExitIf' Directive: exitif:SqNorm(Z)>big;palette:0;split:GetIm(Z);palette:1,2
              if (dummyC.SqNorm(Z)>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:Z=(Z^n)+C
              Z=(Z.Pow(n)).Add(C);

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int cubic_cplane(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 2;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            Complex a;
            double big;
            if (InheritParameters) {
                x_value=myMath.getVariable("a_x");
                y_value=myMath.getVariable("a_y");
                a=new Complex(x_value, y_value);
                big=myMath.getVariable("big");
            } else {
                a=new Complex(-0.5,0.1);
                big=10;
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex c=new Complex(xval,yval);
                    Complex z=dummyC.Sqrt((a.Mult(-1)).Div(3));
                    Complex w=(z.Mult(-1));

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'ExitIf' Directive: exitif:SqNorm(z)>big;palette:2;split:GetIm(z);palette:2,1
                            if (dummyC.SqNorm(z)>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=2;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                SplitResult[0]=dummyC.GetIm(z);
                                break Block0;
                            }

                            // 'Set' Directive: set:z=c+((a*z)+(z^3))
                            z=c.Add((a.Mult(z)).Add(z.Pow(3)));

                        } // End of Block 0

                        Block1: // Start of Block 1
                        if (!BlockExit[1]) {

                            // 'ExitIf' Directive: exitif:SqNorm(w)>big;palette:3;split:GetIm(w);palette:3,0
                            if (dummyC.SqNorm(w)>big) {
                                DidExit++;
                                BlockExit[1] = true;
                                Palette[1]=3;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[1][x][y] = iter;
                                SplitResult[1]=dummyC.GetIm(w);
                                break Block1;
                            }

                            // 'Set' Directive: set:w=c+((a*w)+(w^3))
                            w=c.Add((a.Mult(w)).Add(w.Pow(3)));

                        } // End of Block 1

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

    public int cubic_aplane(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 3;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            double n;
            Complex c;
            double big;
            if (InheritParameters) {
                n=myMath.getVariable("n");
                x_value=myMath.getVariable("c_x");
                y_value=myMath.getVariable("c_y");
                c=new Complex(x_value, y_value);
                big=myMath.getVariable("big");
            } else {
                n=3;
                c=new Complex(cX,cY);
                big=10;
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex a=new Complex(xval,yval);
                    Complex z=dummyC.Sqrt((a.Mult(-1)).Div(3));
                    Complex w=(z.Mult(-1));

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'ExitIf' Directive: exitif:SqNorm(z)>big;palette:2;split:GetIm(z);palette:2,1
                            if (dummyC.SqNorm(z)>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=2;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                SplitResult[0]=dummyC.GetIm(z);
                                break Block0;
                            }

                            // 'Set' Directive: set:z=c+((a*z)+(z^3))
                            z=c.Add((a.Mult(z)).Add(z.Pow(3)));

                        } // End of Block 0

                        Block1: // Start of Block 1
                        if (!BlockExit[1]) {

                            // 'ExitIf' Directive: exitif:SqNorm(w)>big;palette:3;split:GetIm(w);palette:3,0
                            if (dummyC.SqNorm(w)>big) {
                                DidExit++;
                                BlockExit[1] = true;
                                Palette[1]=3;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[1][x][y] = iter;
                                SplitResult[1]=dummyC.GetIm(w);
                                break Block1;
                            }

                            // 'Set' Directive: set:w=c+((a*w)+(w^3))
                            w=c.Add((a.Mult(w)).Add(w.Pow(3)));

                        } // End of Block 1

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

    public int cubic_julia(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 4;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            Complex a;
            Complex c;
            if (InheritParameters) {
                x_value=myMath.getVariable("a_x");
                y_value=myMath.getVariable("a_y");
                a=new Complex(x_value, y_value);
                x_value=myMath.getVariable("c_x");
                y_value=myMath.getVariable("c_y");
                c=new Complex(x_value, y_value);
            } else {
                a=new Complex(cX,cY);
                c=new Complex(-0.5,0.1);
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);
            Complex p1=dummyC.Sqrt((a.Mult(-1)).Div(3));
            Complex p2=(p1.Mult(-1));

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex z=new Complex(xval,yval);

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'ExitIf' Directive: exitif:SqNorm(z)>10;palette:0;split:GetIm(z);palette:1,2
                            if (dummyC.SqNorm(z)>10) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                SplitResult[0]=dummyC.GetIm(z);
                                break Block0;
                            }

                            // 'Set' Directive: set:z=c+((a*z)+(z^3))
                            z=c.Add((a.Mult(z)).Add(z.Pow(3)));

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[1]=dummyC.GetRe(p1);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[1]=dummyC.GetIm(p1);
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[2]=dummyC.GetRe(p2);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[2]=dummyC.GetIm(p2);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData cubic_julia_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 4;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      Complex a;
      Complex c;
      if (InheritParameters) {
          x_value=myMath.getVariable("a_x");
          y_value=myMath.getVariable("a_y");
          a=new Complex(x_value, y_value);
          x_value=myMath.getVariable("c_x");
          y_value=myMath.getVariable("c_y");
          c=new Complex(x_value, y_value);
      } else {
          a=new Complex(cX,cY);
          c=new Complex(-0.5,0.1);
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex z=new Complex(xval,yval);
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(z), dummyC.GetIm(z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'ExitIf' Directive: exitif:SqNorm(z)>10;palette:0;split:GetIm(z);palette:1,2
              if (dummyC.SqNorm(z)>10) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:z=c+((a*z)+(z^3))
              z=c.Add((a.Mult(z)).Add(z.Pow(3)));

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int skew_base(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 5;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;
            int basinindex;
            if (deTool.ImageArray.ParentFrameIndex[TargetFrameIndex] > -1) {
                basinindex = deTool.ImageArray.ParentFrameIndex[TargetFrameIndex];
            } else {
                basinindex = TargetFrameIndex;
            }
            int[] detectx0 = new int[MaxIter];
            int[] detecty0 = new int[MaxIter];
            double[] detectxval0 = new double[MaxIter];
            double[] detectyval0 = new double[MaxIter];

            // initialize parameters
            Complex C;
            double n;
            double big;
            if (InheritParameters) {
                x_value=myMath.getVariable("C_x");
                y_value=myMath.getVariable("C_y");
                C=new Complex(x_value, y_value);
                n=myMath.getVariable("n");
                big=myMath.getVariable("big");
            } else {
                C=new Complex(cX,cY);
                n=2;
                big=100;
            }

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex Z=new Complex(xval,yval);

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Detect' Directive: detect:xval:GetRe(Z);yval:GetIm(Z);palette:3
                            if (DCD) {
                                detectxval0[iter] = dummyC.GetRe(Z);
                                detectyval0[iter] = dummyC.GetIm(Z);
                                detectx0[iter] = deTool.GetXFromVal(basinindex, detectxval0[iter]);
                                detecty0[iter] = deTool.GetYFromVal(basinindex, detectyval0[iter]);
                            }

                            // 'ExitIf' Directive: exitif:SqNorm(Z)>big;palette:0;split:GetIm(Z);palette:1,2
                            if (dummyC.SqNorm(Z)>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                SplitResult[0]=dummyC.GetIm(Z);
                                break Block0;
                            }

                            // 'Set' Directive: set:Z=(Z^n)+C
                            Z=(Z.Pow(n)).Add(C);

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                            // cycle and basin detection code for block 0
                            if (!BlockExit[0]) {
                                switch (CDM) {
                                    case 0: { // pixel level precision
                                        if (deTool.DetectCB(TargetFrameIndex, 0, x, y, basinindex, BDM, detectx0, detecty0, iter)) {
                                            BlockExit[0] = true;
                                            Palette[0]=3;
                                            SplitResult[0]=dummyC.GetIm(Z);
                                        }
                                    }
                                    break;
                                    case 1: { // Double value level precision
                                        if (deTool.DetectCB(TargetFrameIndex, 0, x, y, basinindex, BDM, detectxval0, detectyval0, iter)) {
                                            BlockExit[0] = true;
                                            Palette[0]=3;
                                            SplitResult[0]=dummyC.GetIm(Z);
                                        }
                                    }
                                    break;
                                }
                            }
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData skew_base_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 5;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      Complex C;
      double n;
      double big;
      if (InheritParameters) {
          x_value=myMath.getVariable("C_x");
          y_value=myMath.getVariable("C_y");
          C=new Complex(x_value, y_value);
          n=myMath.getVariable("n");
          big=myMath.getVariable("big");
      } else {
          C=new Complex(cX,cY);
          n=2;
          big=100;
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex Z=new Complex(xval,yval);
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(Z), dummyC.GetIm(Z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'ExitIf' Directive: exitif:SqNorm(Z)>big;palette:0;split:GetIm(Z);palette:1,2
              if (dummyC.SqNorm(Z)>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:Z=(Z^n)+C
              Z=(Z.Pow(n)).Add(C);

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int skew_fiber(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 6;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;
            int basinindex;
            if (deTool.ImageArray.ParentFrameIndex[TargetFrameIndex] > -1) {
                basinindex = deTool.ImageArray.ParentFrameIndex[TargetFrameIndex];
            } else {
                basinindex = TargetFrameIndex;
            }
            int[] detectx0 = new int[MaxIter];
            int[] detecty0 = new int[MaxIter];
            double[] detectxval0 = new double[MaxIter];
            double[] detectyval0 = new double[MaxIter];

            // initialize parameters
            Complex C;
            Complex b;
            double n;
            double big;
            if (InheritParameters) {
                x_value=myMath.getVariable("C_x");
                y_value=myMath.getVariable("C_y");
                C=new Complex(x_value, y_value);
                x_value=myMath.getVariable("b_x");
                y_value=myMath.getVariable("b_y");
                b=new Complex(x_value, y_value);
                n=myMath.getVariable("n");
                big=myMath.getVariable("big");
            } else {
                C=new Complex(0,0);
                b=new Complex(-1,0);
                n=2;
                big=100;
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex z=new Complex(cX,cY);
                    Complex w=new Complex(xval,yval);

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Detect' Directive: detect:xval:GetRe(w);yval:GetIm(w);palette:3
                            if (DCD) {
                                detectxval0[iter] = dummyC.GetRe(w);
                                detectyval0[iter] = dummyC.GetIm(w);
                                detectx0[iter] = deTool.GetXFromVal(basinindex, detectxval0[iter]);
                                detecty0[iter] = deTool.GetYFromVal(basinindex, detectyval0[iter]);
                            }

                            // 'Set' Directive: set:w=(w^n)+(b*z)
                            w=(w.Pow(n)).Add(b.Mult(z));

                            // 'ExitIf' Directive: exitif:SqNorm(w)>big;palette:0;split:GetIm(w);palette:1,2
                            if (dummyC.SqNorm(w)>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                SplitResult[0]=dummyC.GetIm(w);
                                break Block0;
                            }

                            // 'Set' Directive: set:z=(z^n)+C
                            z=(z.Pow(n)).Add(C);

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                            // cycle and basin detection code for block 0
                            if (!BlockExit[0]) {
                                switch (CDM) {
                                    case 0: { // pixel level precision
                                        if (deTool.DetectCB(TargetFrameIndex, 0, x, y, basinindex, BDM, detectx0, detecty0, iter)) {
                                            BlockExit[0] = true;
                                            Palette[0]=3;
                                            SplitResult[0]=dummyC.GetIm(w);
                                        }
                                    }
                                    break;
                                    case 1: { // Double value level precision
                                        if (deTool.DetectCB(TargetFrameIndex, 0, x, y, basinindex, BDM, detectxval0, detectyval0, iter)) {
                                            BlockExit[0] = true;
                                            Palette[0]=3;
                                            SplitResult[0]=dummyC.GetIm(w);
                                        }
                                    }
                                    break;
                                }
                            }
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData skew_fiber_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 6;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      Complex C;
      Complex b;
      double n;
      double big;
      if (InheritParameters) {
          x_value=myMath.getVariable("C_x");
          y_value=myMath.getVariable("C_y");
          C=new Complex(x_value, y_value);
          x_value=myMath.getVariable("b_x");
          y_value=myMath.getVariable("b_y");
          b=new Complex(x_value, y_value);
          n=myMath.getVariable("n");
          big=myMath.getVariable("big");
      } else {
          C=new Complex(0,0);
          b=new Complex(-1,0);
          n=2;
          big=100;
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex z=new Complex(cX,cY);
      Complex w=new Complex(xval,yval);
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(w), dummyC.GetIm(w));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'Set' Directive: set:w=(w^n)+(b*z)
              w=(w.Pow(n)).Add(b.Mult(z));

              // 'ExitIf' Directive: exitif:SqNorm(w)>big;palette:0;split:GetIm(w);palette:1,2
              if (dummyC.SqNorm(w)>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:z=(z^n)+C
              z=(z.Pow(n)).Add(C);

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int newton_cubic(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 7;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;
            int basinindex;
            if (deTool.ImageArray.ParentFrameIndex[TargetFrameIndex] > -1) {
                basinindex = deTool.ImageArray.ParentFrameIndex[TargetFrameIndex];
            } else {
                basinindex = TargetFrameIndex;
            }
            int[] detectx0 = new int[MaxIter];
            int[] detecty0 = new int[MaxIter];
            double[] detectxval0 = new double[MaxIter];
            double[] detectyval0 = new double[MaxIter];

            // initialize parameters
            double tiny;
            if (InheritParameters) {
                tiny=myMath.getVariable("tiny");
            } else {
                tiny=0.001;
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex a=new Complex(xval,yval);
                    Complex z=new Complex(0,0);
                    Complex L1=new Complex(0,0);
                    Complex L2=new Complex(0,0);
                    Complex L3=new Complex(0,0);

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Detect' Directive: detect:xval:GetRe(z);yval:GetIm(z)
                            if (DCD) {
                                detectxval0[iter] = dummyC.GetRe(z);
                                detectyval0[iter] = dummyC.GetIm(z);
                                detectx0[iter] = deTool.GetXFromVal(basinindex, detectxval0[iter]);
                                detecty0[iter] = deTool.GetYFromVal(basinindex, detectyval0[iter]);
                            }

                            // 'Set' Directive: set:L1=z-1
                            L1=dummyC.Sub(z,1);

                            // 'ExitIf' Directive: exitif:SqNorm(L1)<tiny;palette:0
                            if (dummyC.SqNorm(L1)<tiny) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Set' Directive: set:L2=z+(0.5-a)
                            L2=z.Add(dummyC.Sub(0.5,a));

                            // 'ExitIf' Directive: exitif:SqNorm(L2)<tiny;palette:1
                            if (dummyC.SqNorm(L2)<tiny) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=1;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Set' Directive: set:L3=z+(0.5+a)
                            L3=z.Add(a.Add(0.5));

                            // 'ExitIf' Directive: exitif:SqNorm(L3)<tiny;palette:2
                            if (dummyC.SqNorm(L3)<tiny) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=2;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Set' Directive: set:z=z-((L1*L2*L3)/((L1*L2)+(L1*L3)+(L2*L3)))
                            z=dummyC.Sub(z,((L1.Mult(L2).Mult(L3)).Div((L1.Mult(L2)).Add(L1.Mult(L3)).Add(L2.Mult(L3)))));

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData newton_cubic_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 7;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double tiny;
      if (InheritParameters) {
          tiny=myMath.getVariable("tiny");
      } else {
          tiny=0.001;
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex a=new Complex(xval,yval);
      Complex z=new Complex(0,0);
      Complex L1=new Complex(0,0);
      Complex L2=new Complex(0,0);
      Complex L3=new Complex(0,0);
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(z), dummyC.GetIm(z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'Set' Directive: set:L1=z-1
              L1=dummyC.Sub(z,1);

              // 'ExitIf' Directive: exitif:SqNorm(L1)<tiny;palette:0
              if (dummyC.SqNorm(L1)<tiny) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:L2=z+(0.5-a)
              L2=z.Add(dummyC.Sub(0.5,a));

              // 'ExitIf' Directive: exitif:SqNorm(L2)<tiny;palette:1
              if (dummyC.SqNorm(L2)<tiny) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:L3=z+(0.5+a)
              L3=z.Add(a.Add(0.5));

              // 'ExitIf' Directive: exitif:SqNorm(L3)<tiny;palette:2
              if (dummyC.SqNorm(L3)<tiny) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:z=z-((L1*L2*L3)/((L1*L2)+(L1*L3)+(L2*L3)))
              z=dummyC.Sub(z,((L1.Mult(L2).Mult(L3)).Div((L1.Mult(L2)).Add(L1.Mult(L3)).Add(L2.Mult(L3)))));

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int newton_cubic_julia(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 8;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;
            int basinindex;
            if (deTool.ImageArray.ParentFrameIndex[TargetFrameIndex] > -1) {
                basinindex = deTool.ImageArray.ParentFrameIndex[TargetFrameIndex];
            } else {
                basinindex = TargetFrameIndex;
            }
            int[] detectx0 = new int[MaxIter];
            int[] detecty0 = new int[MaxIter];
            double[] detectxval0 = new double[MaxIter];
            double[] detectyval0 = new double[MaxIter];

            // initialize parameters
            double tiny;
            Complex a;
            if (InheritParameters) {
                tiny=myMath.getVariable("tiny");
                x_value=myMath.getVariable("a_x");
                y_value=myMath.getVariable("a_y");
                a=new Complex(x_value, y_value);
            } else {
                tiny=0.01;
                a=new Complex(cX,cY);
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);
            Complex p1=new Complex(1,0);
            Complex p2=dummyC.Add(0.5,a);
            Complex p3=dummyC.Sub(0.5,a);

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex z=new Complex(xval, yval);
                    Complex L1=new Complex(0,0);
                    Complex L2=new Complex(0,0);
                    Complex L3=new Complex(0,0);

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Detect' Directive: detect:xval:GetRe(z);yval:GetIm(z)
                            if (DCD) {
                                detectxval0[iter] = dummyC.GetRe(z);
                                detectyval0[iter] = dummyC.GetIm(z);
                                detectx0[iter] = deTool.GetXFromVal(basinindex, detectxval0[iter]);
                                detecty0[iter] = deTool.GetYFromVal(basinindex, detectyval0[iter]);
                            }

                            // 'Set' Directive: set:L1=(z-1)
                            L1=(dummyC.Sub(z,1));

                            // 'ExitIf' Directive: exitif:SqNorm(L1)<tiny;palette:0
                            if (dummyC.SqNorm(L1)<tiny) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Set' Directive: set:L2=z+(0.5-a)
                            L2=z.Add(dummyC.Sub(0.5,a));

                            // 'ExitIf' Directive: exitif:SqNorm(L2)<tiny;palette:1
                            if (dummyC.SqNorm(L2)<tiny) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=1;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Set' Directive: set:L3=z+(0.5+a)
                            L3=z.Add(a.Add(0.5));

                            // 'ExitIf' Directive: exitif:SqNorm(L3)<tiny;palette:2
                            if (dummyC.SqNorm(L3)<tiny) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=2;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Set' Directive: set:z=z-((L1*L2*L3)/((L1*L2)+(L1*L3)+(L2*L3)))
                            z=dummyC.Sub(z,((L1.Mult(L2).Mult(L3)).Div((L1.Mult(L2)).Add(L1.Mult(L3)).Add(L2.Mult(L3)))));

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[1]=dummyC.GetRe(p1);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[1]=dummyC.GetIm(p1);
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[2]=dummyC.GetRe(p2);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[2]=dummyC.GetIm(p2);
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[3]=dummyC.GetRe(p3);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[3]=dummyC.GetIm(p3);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData newton_cubic_julia_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 8;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double tiny;
      Complex a;
      if (InheritParameters) {
          tiny=myMath.getVariable("tiny");
          x_value=myMath.getVariable("a_x");
          y_value=myMath.getVariable("a_y");
          a=new Complex(x_value, y_value);
      } else {
          tiny=0.01;
          a=new Complex(cX,cY);
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex z=new Complex(xval, yval);
      Complex L1=new Complex(0,0);
      Complex L2=new Complex(0,0);
      Complex L3=new Complex(0,0);
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(z), dummyC.GetIm(z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'Set' Directive: set:L1=(z-1)
              L1=(dummyC.Sub(z,1));

              // 'ExitIf' Directive: exitif:SqNorm(L1)<tiny;palette:0
              if (dummyC.SqNorm(L1)<tiny) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:L2=z+(0.5-a)
              L2=z.Add(dummyC.Sub(0.5,a));

              // 'ExitIf' Directive: exitif:SqNorm(L2)<tiny;palette:1
              if (dummyC.SqNorm(L2)<tiny) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:L3=z+(0.5+a)
              L3=z.Add(a.Add(0.5));

              // 'ExitIf' Directive: exitif:SqNorm(L3)<tiny;palette:2
              if (dummyC.SqNorm(L3)<tiny) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:z=z-((L1*L2*L3)/((L1*L2)+(L1*L3)+(L2*L3)))
              z=dummyC.Sub(z,((L1.Mult(L2).Mult(L3)).Div((L1.Mult(L2)).Add(L1.Mult(L3)).Add(L2.Mult(L3)))));

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int PerturbedMandelBoth(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 9;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;
            int basinindex;
            if (deTool.ImageArray.ParentFrameIndex[TargetFrameIndex] > -1) {
                basinindex = deTool.ImageArray.ParentFrameIndex[TargetFrameIndex];
            } else {
                basinindex = TargetFrameIndex;
            }
            int[] detectx0 = new int[MaxIter];
            int[] detecty0 = new int[MaxIter];
            double[] detectxval0 = new double[MaxIter];
            double[] detectyval0 = new double[MaxIter];
            int[] detectx1 = new int[MaxIter];
            int[] detecty1 = new int[MaxIter];
            double[] detectxval1 = new double[MaxIter];
            double[] detectyval1 = new double[MaxIter];

            // initialize parameters
            double n;
            double tiny;
            double big;
            Complex c;
            if (InheritParameters) {
                n=myMath.getVariable("n");
                tiny=myMath.getVariable("tiny");
                big=myMath.getVariable("big");
                x_value=myMath.getVariable("c_x");
                y_value=myMath.getVariable("c_y");
                c=new Complex(x_value, y_value);
            } else {
                n=2;
                tiny=0.000001;
                big=250;
                c=new Complex(cX,cY);
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex a=new Complex(xval,yval);
                    Complex zn=new Complex(0,0);
                    Complex wn=new Complex(0,0);
                    Complex v=(dummyC.Sqrt(a).Mult(2));
                    Complex z=(c.Add(v));
                    Complex w=(dummyC.Sub(c,v));
                    double r=0;
                    double s=0;

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Detect' Directive: detect:xval:GetRe(z);yval:GetIm(z);palette:4
                            if (DCD) {
                                detectxval0[iter] = dummyC.GetRe(z);
                                detectyval0[iter] = dummyC.GetIm(z);
                                detectx0[iter] = deTool.GetXFromVal(basinindex, detectxval0[iter]);
                                detecty0[iter] = deTool.GetYFromVal(basinindex, detectyval0[iter]);
                            }

                            // 'Set' Directive: set:r=SqNorm(z)
                            r=dummyC.SqNorm(z);

                            // 'ExitIf' Directive: exitif:r<tiny;palette:0;split:GetIm(z);palette:0,1
                            if (r<tiny) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                SplitResult[0]=dummyC.GetIm(z);
                                break Block0;
                            }

                            // 'ExitIf' Directive: exitif:r>big;palette:0;split:GetIm(z);palette:0,1
                            if (r>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                SplitResult[0]=dummyC.GetIm(z);
                                break Block0;
                            }

                            // 'Set' Directive: set:zn=z^n
                            zn=z.Pow(n);

                            // 'Set' Directive: set:z=c+(zn+a/zn)
                            z=c.Add(zn.Add(a.Div(zn)));

                        } // End of Block 0

                        Block1: // Start of Block 1
                        if (!BlockExit[1]) {

                            // 'Detect' Directive: detect:xval:GetRe(w);yval:GetIm(w);palette:5
                            if (DCD) {
                                detectxval1[iter] = dummyC.GetRe(w);
                                detectyval1[iter] = dummyC.GetIm(w);
                                detectx1[iter] = deTool.GetXFromVal(basinindex, detectxval1[iter]);
                                detecty1[iter] = deTool.GetYFromVal(basinindex, detectyval1[iter]);
                            }

                            // 'Set' Directive: set:s=SqNorm(w)
                            s=dummyC.SqNorm(w);

                            // 'ExitIf' Directive: exitif:s<tiny;palette:3;split:GetIm(w);palette:3,2
                            if (s<tiny) {
                                DidExit++;
                                BlockExit[1] = true;
                                Palette[1]=3;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[1][x][y] = iter;
                                SplitResult[1]=dummyC.GetIm(w);
                                break Block1;
                            }

                            // 'ExitIf' Directive: exitif:s>big;palette:3;split:GetIm(w);palette:3,2
                            if (s>big) {
                                DidExit++;
                                BlockExit[1] = true;
                                Palette[1]=3;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[1][x][y] = iter;
                                SplitResult[1]=dummyC.GetIm(w);
                                break Block1;
                            }

                            // 'Set' Directive: set:wn=w^n
                            wn=w.Pow(n);

                            // 'Set' Directive: set:w=c+(wn+a/wn)
                            w=c.Add(wn.Add(a.Div(wn)));

                        } // End of Block 1

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                            // cycle and basin detection code for block 0
                            if (!BlockExit[0]) {
                                switch (CDM) {
                                    case 0: { // pixel level precision
                                        if (deTool.DetectCB(TargetFrameIndex, 0, x, y, basinindex, BDM, detectx0, detecty0, iter)) {
                                            BlockExit[0] = true;
                                            Palette[0]=4;
                                            SplitResult[0]=dummyC.GetIm(z);
                                        }
                                    }
                                    break;
                                    case 1: { // Double value level precision
                                        if (deTool.DetectCB(TargetFrameIndex, 0, x, y, basinindex, BDM, detectxval0, detectyval0, iter)) {
                                            BlockExit[0] = true;
                                            Palette[0]=4;
                                            SplitResult[0]=dummyC.GetIm(z);
                                        }
                                    }
                                    break;
                                }
                            }
                            // cycle and basin detection code for block 1
                            if (!BlockExit[1]) {
                                switch (CDM) {
                                    case 0: { // pixel level precision
                                        if (deTool.DetectCB(TargetFrameIndex, 1, x, y, basinindex, BDM, detectx1, detecty1, iter)) {
                                            BlockExit[1] = true;
                                            Palette[1]=5;
                                            SplitResult[1]=dummyC.GetIm(w);
                                        }
                                    }
                                    break;
                                    case 1: { // Double value level precision
                                        if (deTool.DetectCB(TargetFrameIndex, 1, x, y, basinindex, BDM, detectxval1, detectyval1, iter)) {
                                            BlockExit[1] = true;
                                            Palette[1]=5;
                                            SplitResult[1]=dummyC.GetIm(w);
                                        }
                                    }
                                    break;
                                }
                            }
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData PerturbedMandelBoth_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 9;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double n;
      double tiny;
      double big;
      Complex c;
      if (InheritParameters) {
          n=myMath.getVariable("n");
          tiny=myMath.getVariable("tiny");
          big=myMath.getVariable("big");
          x_value=myMath.getVariable("c_x");
          y_value=myMath.getVariable("c_y");
          c=new Complex(x_value, y_value);
      } else {
          n=2;
          tiny=0.000001;
          big=250;
          c=new Complex(cX,cY);
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex a=new Complex(xval,yval);
      Complex zn=new Complex(0,0);
      Complex wn=new Complex(0,0);
      Complex v=(dummyC.Sqrt(a).Mult(2));
      Complex z=(c.Add(v));
      Complex w=(dummyC.Sub(c,v));
      double r=0;
      double s=0;
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(z), dummyC.GetIm(z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'Set' Directive: set:r=SqNorm(z)
              r=dummyC.SqNorm(z);

              // 'ExitIf' Directive: exitif:r<tiny;palette:0;split:GetIm(z);palette:0,1
              if (r<tiny) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'ExitIf' Directive: exitif:r>big;palette:0;split:GetIm(z);palette:0,1
              if (r>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:zn=z^n
              zn=z.Pow(n);

              // 'Set' Directive: set:z=c+(zn+a/zn)
              z=c.Add(zn.Add(a.Div(zn)));

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

  public TraceData PerturbedMandelBoth_1_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 9;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double n;
      double tiny;
      double big;
      Complex c;
      if (InheritParameters) {
          n=myMath.getVariable("n");
          tiny=myMath.getVariable("tiny");
          big=myMath.getVariable("big");
          x_value=myMath.getVariable("c_x");
          y_value=myMath.getVariable("c_y");
          c=new Complex(x_value, y_value);
      } else {
          n=2;
          tiny=0.000001;
          big=250;
          c=new Complex(cX,cY);
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex a=new Complex(xval,yval);
      Complex zn=new Complex(0,0);
      Complex wn=new Complex(0,0);
      Complex v=(dummyC.Sqrt(a).Mult(2));
      Complex z=(c.Add(v));
      Complex w=(dummyC.Sub(c,v));
      double r=0;
      double s=0;
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(w), dummyC.GetIm(w));
          // User-defined directives are executed for each iteration in inner loop: 

          Block1: // Start of Block 1
          if (!BlockExit[1]) {

              // 'Set' Directive: set:s=SqNorm(w)
              s=dummyC.SqNorm(w);

              // 'ExitIf' Directive: exitif:s<tiny;palette:3;split:GetIm(w);palette:3,2
              if (s<tiny) {
                  DidExit++;
                  BlockExit[1] = true;
                  break Block1;
              }

              // 'ExitIf' Directive: exitif:s>big;palette:3;split:GetIm(w);palette:3,2
              if (s>big) {
                  DidExit++;
                  BlockExit[1] = true;
                  break Block1;
              }

              // 'Set' Directive: set:wn=w^n
              wn=w.Pow(n);

              // 'Set' Directive: set:w=c+(wn+a/wn)
              w=c.Add(wn.Add(a.Div(wn)));

          } // End of Block 1
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int PerturbedMandelvp(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 10;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            double n;
            double tiny;
            double big;
            Complex c;
            if (InheritParameters) {
                n=myMath.getVariable("n");
                tiny=myMath.getVariable("tiny");
                big=myMath.getVariable("big");
                x_value=myMath.getVariable("c_x");
                y_value=myMath.getVariable("c_y");
                c=new Complex(x_value, y_value);
            } else {
                n=2;
                tiny=0.0001;
                big=10;
                c=new Complex(cX,cY);
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex a=new Complex(xval,yval);
                    Complex zn=new Complex(0,0);
                    Complex v=(dummyC.Sqrt(a).Mult(2));
                    Complex z=(c.Add(v));
                    double r=0;

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Set' Directive: set:r=SqNorm(z)
                            r=dummyC.SqNorm(z);

                            // 'ExitIf' Directive: exitif:r<tiny;palette:0;split:GetIm(z);palette:0,1
                            if (r<tiny) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                SplitResult[0]=dummyC.GetIm(z);
                                break Block0;
                            }

                            // 'ExitIf' Directive: exitif:r>big;palette:0;split:GetIm(z);palette:0,1
                            if (r>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                SplitResult[0]=dummyC.GetIm(z);
                                break Block0;
                            }

                            // 'Set' Directive: set:zn=(z^n)
                            zn=(z.Pow(n));

                            // 'Set' Directive: set:z=(c+(zn+(a/zn)))
                            z=(c.Add(zn.Add(a.Div(zn))));

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData PerturbedMandelvp_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 10;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double n;
      double tiny;
      double big;
      Complex c;
      if (InheritParameters) {
          n=myMath.getVariable("n");
          tiny=myMath.getVariable("tiny");
          big=myMath.getVariable("big");
          x_value=myMath.getVariable("c_x");
          y_value=myMath.getVariable("c_y");
          c=new Complex(x_value, y_value);
      } else {
          n=2;
          tiny=0.0001;
          big=10;
          c=new Complex(cX,cY);
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex a=new Complex(xval,yval);
      Complex zn=new Complex(0,0);
      Complex v=(dummyC.Sqrt(a).Mult(2));
      Complex z=(c.Add(v));
      double r=0;
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(z), dummyC.GetIm(z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'Set' Directive: set:r=SqNorm(z)
              r=dummyC.SqNorm(z);

              // 'ExitIf' Directive: exitif:r<tiny;palette:0;split:GetIm(z);palette:0,1
              if (r<tiny) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'ExitIf' Directive: exitif:r>big;palette:0;split:GetIm(z);palette:0,1
              if (r>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:zn=(z^n)
              zn=(z.Pow(n));

              // 'Set' Directive: set:z=(c+(zn+(a/zn)))
              z=(c.Add(zn.Add(a.Div(zn))));

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int PerturbedMandelvm(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 11;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            double n;
            double tiny;
            double big;
            Complex c;
            if (InheritParameters) {
                n=myMath.getVariable("n");
                tiny=myMath.getVariable("tiny");
                big=myMath.getVariable("big");
                x_value=myMath.getVariable("c_x");
                y_value=myMath.getVariable("c_y");
                c=new Complex(x_value, y_value);
            } else {
                n=2;
                tiny=0.0001;
                big=10;
                c=new Complex(cX,cY);
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex a=new Complex(xval,yval);
                    Complex wn=new Complex(0,0);
                    Complex v=(dummyC.Sqrt(a).Mult(2));
                    Complex w=(dummyC.Sub(c,v));
                    double s=0;

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Set' Directive: set:s=SqNorm(w)
                            s=dummyC.SqNorm(w);

                            // 'ExitIf' Directive: exitif:s<tiny;palette:0;split:GetIm(w);palette:0,1
                            if (s<tiny) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                SplitResult[0]=dummyC.GetIm(w);
                                break Block0;
                            }

                            // 'ExitIf' Directive: exitif:s>big;palette:0;split:GetIm(w);palette:0,1
                            if (s>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                SplitResult[0]=dummyC.GetIm(w);
                                break Block0;
                            }

                            // 'Set' Directive: set:wn=(w^n)
                            wn=(w.Pow(n));

                            // 'Set' Directive: set:w=(c+(wn+(a/wn)))
                            w=(c.Add(wn.Add(a.Div(wn))));

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData PerturbedMandelvm_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 11;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double n;
      double tiny;
      double big;
      Complex c;
      if (InheritParameters) {
          n=myMath.getVariable("n");
          tiny=myMath.getVariable("tiny");
          big=myMath.getVariable("big");
          x_value=myMath.getVariable("c_x");
          y_value=myMath.getVariable("c_y");
          c=new Complex(x_value, y_value);
      } else {
          n=2;
          tiny=0.0001;
          big=10;
          c=new Complex(cX,cY);
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex a=new Complex(xval,yval);
      Complex wn=new Complex(0,0);
      Complex v=(dummyC.Sqrt(a).Mult(2));
      Complex w=(dummyC.Sub(c,v));
      double s=0;
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(w), dummyC.GetIm(w));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'Set' Directive: set:s=SqNorm(w)
              s=dummyC.SqNorm(w);

              // 'ExitIf' Directive: exitif:s<tiny;palette:0;split:GetIm(w);palette:0,1
              if (s<tiny) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'ExitIf' Directive: exitif:s>big;palette:0;split:GetIm(w);palette:0,1
              if (s>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:wn=(w^n)
              wn=(w.Pow(n));

              // 'Set' Directive: set:w=(c+(wn+(a/wn)))
              w=(c.Add(wn.Add(a.Div(wn))));

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int perturbed_julia(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 12;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;
            int basinindex;
            if (deTool.ImageArray.ParentFrameIndex[TargetFrameIndex] > -1) {
                basinindex = deTool.ImageArray.ParentFrameIndex[TargetFrameIndex];
            } else {
                basinindex = TargetFrameIndex;
            }
            int[] detectx0 = new int[MaxIter];
            int[] detecty0 = new int[MaxIter];
            double[] detectxval0 = new double[MaxIter];
            double[] detectyval0 = new double[MaxIter];

            // initialize parameters
            Complex a;
            Complex c;
            double tiny;
            double n;
            if (InheritParameters) {
                x_value=myMath.getVariable("a_x");
                y_value=myMath.getVariable("a_y");
                a=new Complex(x_value, y_value);
                x_value=myMath.getVariable("c_x");
                y_value=myMath.getVariable("c_y");
                c=new Complex(x_value, y_value);
                tiny=myMath.getVariable("tiny");
                n=myMath.getVariable("n");
            } else {
                a=new Complex(cX,cY);
                c=new Complex(0,0);
                tiny=0.0001;
                n=2;
            }

            // User-defined distinguished points are set before loop: 
            Complex v=(dummyC.Sqrt(a).Mult(2));
            Complex p0=new Complex(0,0);
            Complex p1=(c.Add(v));
            Complex p2=(dummyC.Sub(c,v));

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex z=new Complex(xval,yval);
                    Complex zn=new Complex(0,0);
                    double r=0;

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Detect' Directive: detect:xval:GetRe(z);yval:GetIm(z);palette:2
                            if (DCD) {
                                detectxval0[iter] = dummyC.GetRe(z);
                                detectyval0[iter] = dummyC.GetIm(z);
                                detectx0[iter] = deTool.GetXFromVal(basinindex, detectxval0[iter]);
                                detecty0[iter] = deTool.GetYFromVal(basinindex, detectyval0[iter]);
                            }

                            // 'Set' Directive: set:r=SqNorm(z)
                            r=dummyC.SqNorm(z);

                            // 'ExitIf' Directive: exitif:r<tiny;palette:0;split:GetIm(z);palette:0,1
                            if (r<tiny) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                SplitResult[0]=dummyC.GetIm(z);
                                break Block0;
                            }

                            // 'ExitIf' Directive: exitif:r>10;palette:0;split:GetIm(z);palette:0,1
                            if (r>10) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                SplitResult[0]=dummyC.GetIm(z);
                                break Block0;
                            }

                            // 'Set' Directive: set:zn=(z^n)
                            zn=(z.Pow(n));

                            // 'Set' Directive: set:z=(c+(zn+(a/zn)))
                            z=(c.Add(zn.Add(a.Div(zn))));

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                            // cycle and basin detection code for block 0
                            if (!BlockExit[0]) {
                                switch (CDM) {
                                    case 0: { // pixel level precision
                                        if (deTool.DetectCB(TargetFrameIndex, 0, x, y, basinindex, BDM, detectx0, detecty0, iter)) {
                                            BlockExit[0] = true;
                                            Palette[0]=2;
                                            SplitResult[0]=dummyC.GetIm(z);
                                        }
                                    }
                                    break;
                                    case 1: { // Double value level precision
                                        if (deTool.DetectCB(TargetFrameIndex, 0, x, y, basinindex, BDM, detectxval0, detectyval0, iter)) {
                                            BlockExit[0] = true;
                                            Palette[0]=2;
                                            SplitResult[0]=dummyC.GetIm(z);
                                        }
                                    }
                                    break;
                                }
                            }
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[1]=dummyC.GetRe(p1);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[1]=dummyC.GetIm(p1);
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[2]=dummyC.GetRe(p2);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[2]=dummyC.GetIm(p2);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData perturbed_julia_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 12;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      Complex a;
      Complex c;
      double tiny;
      double n;
      if (InheritParameters) {
          x_value=myMath.getVariable("a_x");
          y_value=myMath.getVariable("a_y");
          a=new Complex(x_value, y_value);
          x_value=myMath.getVariable("c_x");
          y_value=myMath.getVariable("c_y");
          c=new Complex(x_value, y_value);
          tiny=myMath.getVariable("tiny");
          n=myMath.getVariable("n");
      } else {
          a=new Complex(cX,cY);
          c=new Complex(0,0);
          tiny=0.0001;
          n=2;
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex z=new Complex(xval,yval);
      Complex zn=new Complex(0,0);
      double r=0;
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(z), dummyC.GetIm(z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'Set' Directive: set:r=SqNorm(z)
              r=dummyC.SqNorm(z);

              // 'ExitIf' Directive: exitif:r<tiny;palette:0;split:GetIm(z);palette:0,1
              if (r<tiny) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'ExitIf' Directive: exitif:r>10;palette:0;split:GetIm(z);palette:0,1
              if (r>10) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:zn=(z^n)
              zn=(z.Pow(n));

              // 'Set' Directive: set:z=(c+(zn+(a/zn)))
              z=(c.Add(zn.Add(a.Div(zn))));

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int complex_exponential(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 13;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;
            int basinindex;
            if (deTool.ImageArray.ParentFrameIndex[TargetFrameIndex] > -1) {
                basinindex = deTool.ImageArray.ParentFrameIndex[TargetFrameIndex];
            } else {
                basinindex = TargetFrameIndex;
            }
            int[] detectx0 = new int[MaxIter];
            int[] detecty0 = new int[MaxIter];
            double[] detectxval0 = new double[MaxIter];
            double[] detectyval0 = new double[MaxIter];

            // initialize parameters
            double big;
            if (InheritParameters) {
                big=myMath.getVariable("big");
            } else {
                big=50;
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    double r=0;
                    Complex lambda=new Complex(xval,yval);
                    Complex Z=new Complex(0,0);

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Set' Directive: set:r=sqrt(GetRe(Z)*GetRe(Z))
                            r=Math.sqrt(dummyC.GetRe(Z)*dummyC.GetRe(Z));

                            // 'ExitIf' Directive: exitif:r>big;palette:0
                            if (r>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Set' Directive: set:Z=lambda*Exp(Z)
                            Z=lambda.Mult(dummyC.Exp(Z));

                            // 'Detect' Directive: detect:xval:GetRe(Z);yval:GetIm(Z)
                            if (DCD) {
                                detectxval0[iter] = dummyC.GetRe(Z);
                                detectyval0[iter] = dummyC.GetIm(Z);
                                detectx0[iter] = deTool.GetXFromVal(basinindex, detectxval0[iter]);
                                detecty0[iter] = deTool.GetYFromVal(basinindex, detectyval0[iter]);
                            }

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData complex_exponential_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 13;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double big;
      if (InheritParameters) {
          big=myMath.getVariable("big");
      } else {
          big=50;
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      double r=0;
      Complex lambda=new Complex(xval,yval);
      Complex Z=new Complex(0,0);
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(Z), dummyC.GetIm(Z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'Set' Directive: set:r=sqrt(GetRe(Z)*GetRe(Z))
              r=Math.sqrt(dummyC.GetRe(Z)*dummyC.GetRe(Z));

              // 'ExitIf' Directive: exitif:r>big;palette:0
              if (r>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:Z=lambda*Exp(Z)
              Z=lambda.Mult(dummyC.Exp(Z));

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int complex_exp_julia(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 14;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;
            int basinindex;
            if (deTool.ImageArray.ParentFrameIndex[TargetFrameIndex] > -1) {
                basinindex = deTool.ImageArray.ParentFrameIndex[TargetFrameIndex];
            } else {
                basinindex = TargetFrameIndex;
            }
            int[] detectx0 = new int[MaxIter];
            int[] detecty0 = new int[MaxIter];
            double[] detectxval0 = new double[MaxIter];
            double[] detectyval0 = new double[MaxIter];

            // initialize parameters
            double big;
            Complex lambda;
            if (InheritParameters) {
                big=myMath.getVariable("big");
                x_value=myMath.getVariable("lambda_x");
                y_value=myMath.getVariable("lambda_y");
                lambda=new Complex(x_value, y_value);
            } else {
                big=50;
                lambda=new Complex(cX,cY);
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    double r=0;
                    Complex Z=new Complex(xval,yval);

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Set' Directive: set:r=sqrt(GetRe(Z)*GetRe(Z))
                            r=Math.sqrt(dummyC.GetRe(Z)*dummyC.GetRe(Z));

                            // 'ExitIf' Directive: exitif:r>big;palette:0
                            if (r>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Set' Directive: set:Z=lambda*Exp(Z)
                            Z=lambda.Mult(dummyC.Exp(Z));

                            // 'Detect' Directive: detect:xval:GetRe(Z);yval:GetIm(Z)
                            if (DCD) {
                                detectxval0[iter] = dummyC.GetRe(Z);
                                detectyval0[iter] = dummyC.GetIm(Z);
                                detectx0[iter] = deTool.GetXFromVal(basinindex, detectxval0[iter]);
                                detecty0[iter] = deTool.GetYFromVal(basinindex, detectyval0[iter]);
                            }

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData complex_exp_julia_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 14;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double big;
      Complex lambda;
      if (InheritParameters) {
          big=myMath.getVariable("big");
          x_value=myMath.getVariable("lambda_x");
          y_value=myMath.getVariable("lambda_y");
          lambda=new Complex(x_value, y_value);
      } else {
          big=50;
          lambda=new Complex(cX,cY);
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      double r=0;
      Complex Z=new Complex(xval,yval);
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(Z), dummyC.GetIm(Z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'Set' Directive: set:r=sqrt(GetRe(Z)*GetRe(Z))
              r=Math.sqrt(dummyC.GetRe(Z)*dummyC.GetRe(Z));

              // 'ExitIf' Directive: exitif:r>big;palette:0
              if (r>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:Z=lambda*Exp(Z)
              Z=lambda.Mult(dummyC.Exp(Z));

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int complex_sine(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 15;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            double big;
            if (InheritParameters) {
                big=myMath.getVariable("big");
            } else {
                big=50;
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex lambda=new Complex(xval,yval);
                    Complex Z=new Complex(1.5708,0);
                    double r=0;

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Set' Directive: set:r=sqrt((GetIm(Z))*(GetIm(Z)))
                            r=Math.sqrt((dummyC.GetIm(Z))*(dummyC.GetIm(Z)));

                            // 'ExitIf' Directive: exitif:r>big;palette:0
                            if (r>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Set' Directive: set:Z=lambda*Sin(Z)
                            Z=lambda.Mult(dummyC.Sin(Z));

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData complex_sine_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 15;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double big;
      if (InheritParameters) {
          big=myMath.getVariable("big");
      } else {
          big=50;
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex lambda=new Complex(xval,yval);
      Complex Z=new Complex(1.5708,0);
      double r=0;
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(Z), dummyC.GetIm(Z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'Set' Directive: set:r=sqrt((GetIm(Z))*(GetIm(Z)))
              r=Math.sqrt((dummyC.GetIm(Z))*(dummyC.GetIm(Z)));

              // 'ExitIf' Directive: exitif:r>big;palette:0
              if (r>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:Z=lambda*Sin(Z)
              Z=lambda.Mult(dummyC.Sin(Z));

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int complex_sin_julia(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 16;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            double big;
            Complex lambda;
            if (InheritParameters) {
                big=myMath.getVariable("big");
                x_value=myMath.getVariable("lambda_x");
                y_value=myMath.getVariable("lambda_y");
                lambda=new Complex(x_value, y_value);
            } else {
                big=50;
                lambda=new Complex(cX,cY);
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex Z=new Complex(xval,yval);
                    double r=0;

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Set' Directive: set:r=sqrt((GetIm(Z))*(GetIm(Z)))
                            r=Math.sqrt((dummyC.GetIm(Z))*(dummyC.GetIm(Z)));

                            // 'ExitIf' Directive: exitif:r>big;palette:0
                            if (r>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Set' Directive: set:Z=lambda*Sin(Z)
                            Z=lambda.Mult(dummyC.Sin(Z));

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData complex_sin_julia_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 16;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double big;
      Complex lambda;
      if (InheritParameters) {
          big=myMath.getVariable("big");
          x_value=myMath.getVariable("lambda_x");
          y_value=myMath.getVariable("lambda_y");
          lambda=new Complex(x_value, y_value);
      } else {
          big=50;
          lambda=new Complex(cX,cY);
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex Z=new Complex(xval,yval);
      double r=0;
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(Z), dummyC.GetIm(Z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'Set' Directive: set:r=sqrt((GetIm(Z))*(GetIm(Z)))
              r=Math.sqrt((dummyC.GetIm(Z))*(dummyC.GetIm(Z)));

              // 'ExitIf' Directive: exitif:r>big;palette:0
              if (r>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:Z=lambda*Sin(Z)
              Z=lambda.Mult(dummyC.Sin(Z));

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int complex_cosine(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 17;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            double big;
            if (InheritParameters) {
                big=myMath.getVariable("big");
            } else {
                big=50;
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex lambda=new Complex(xval,yval);
                    Complex Z=new Complex(0,0);
                    double r=0;

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Set' Directive: set:r=sqrt((GetIm(Z))*(GetIm(Z)))
                            r=Math.sqrt((dummyC.GetIm(Z))*(dummyC.GetIm(Z)));

                            // 'ExitIf' Directive: exitif:r>big;palette:0
                            if (r>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Set' Directive: set:Z=lambda*Cos(Z)
                            Z=lambda.Mult(dummyC.Cos(Z));

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData complex_cosine_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 17;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double big;
      if (InheritParameters) {
          big=myMath.getVariable("big");
      } else {
          big=50;
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex lambda=new Complex(xval,yval);
      Complex Z=new Complex(0,0);
      double r=0;
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(Z), dummyC.GetIm(Z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'Set' Directive: set:r=sqrt((GetIm(Z))*(GetIm(Z)))
              r=Math.sqrt((dummyC.GetIm(Z))*(dummyC.GetIm(Z)));

              // 'ExitIf' Directive: exitif:r>big;palette:0
              if (r>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:Z=lambda*Cos(Z)
              Z=lambda.Mult(dummyC.Cos(Z));

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int complex_cos_julia(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 18;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            double big;
            Complex lambda;
            if (InheritParameters) {
                big=myMath.getVariable("big");
                x_value=myMath.getVariable("lambda_x");
                y_value=myMath.getVariable("lambda_y");
                lambda=new Complex(x_value, y_value);
            } else {
                big=50;
                lambda=new Complex(cX,cY);
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);
            Complex p1=new Complex(1.5708,0);

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex Z=new Complex(xval,yval);
                    double r=0;

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Set' Directive: set:r=sqrt((GetIm(Z))*(GetIm(Z)))
                            r=Math.sqrt((dummyC.GetIm(Z))*(dummyC.GetIm(Z)));

                            // 'ExitIf' Directive: exitif:r>big;palette:0
                            if (r>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Set' Directive: set:Z=lambda*Cos(Z)
                            Z=lambda.Mult(dummyC.Cos(Z));

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[1]=dummyC.GetRe(p1);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[1]=dummyC.GetIm(p1);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData complex_cos_julia_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 18;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double big;
      Complex lambda;
      if (InheritParameters) {
          big=myMath.getVariable("big");
          x_value=myMath.getVariable("lambda_x");
          y_value=myMath.getVariable("lambda_y");
          lambda=new Complex(x_value, y_value);
      } else {
          big=50;
          lambda=new Complex(cX,cY);
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex Z=new Complex(xval,yval);
      double r=0;
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(Z), dummyC.GetIm(Z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'Set' Directive: set:r=sqrt((GetIm(Z))*(GetIm(Z)))
              r=Math.sqrt((dummyC.GetIm(Z))*(dummyC.GetIm(Z)));

              // 'ExitIf' Directive: exitif:r>big;palette:0
              if (r>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:Z=lambda*Cos(Z)
              Z=lambda.Mult(dummyC.Cos(Z));

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int mandel_power_param(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 19;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            if (InheritParameters) {
            } else {
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex n=new Complex(xval,yval);

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'ExitIf' Directive: exitif:1>0;palette:0
                            if (1>0) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

    public int mandel_power_complex(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 20;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            Complex n;
            if (InheritParameters) {
                x_value=myMath.getVariable("n_x");
                y_value=myMath.getVariable("n_y");
                n=new Complex(x_value, y_value);
            } else {
                n=new Complex(cX,cY);
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);
            Complex p1=new Complex(0,-1);

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex Z=new Complex(xval,yval);
                    Complex C=Z;

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'ExitIf' Directive: exitif:SqNorm(Z)>10;palette:0;split:GetIm(Z);palette:1,2
                            if (dummyC.SqNorm(Z)>10) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                SplitResult[0]=dummyC.GetIm(Z);
                                break Block0;
                            }

                            // 'Set' Directive: set:Z=(Z^n)+C
                            Z=(Z.Pow(n)).Add(C);

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[1]=dummyC.GetRe(p1);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[1]=dummyC.GetIm(p1);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData mandel_power_complex_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 20;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      Complex n;
      if (InheritParameters) {
          x_value=myMath.getVariable("n_x");
          y_value=myMath.getVariable("n_y");
          n=new Complex(x_value, y_value);
      } else {
          n=new Complex(cX,cY);
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex Z=new Complex(xval,yval);
      Complex C=Z;
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(Z), dummyC.GetIm(Z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'ExitIf' Directive: exitif:SqNorm(Z)>10;palette:0;split:GetIm(Z);palette:1,2
              if (dummyC.SqNorm(Z)>10) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:Z=(Z^n)+C
              Z=(Z.Pow(n)).Add(C);

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int mandel_power_complex_julia(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 21;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            Complex n;
            double big;
            Complex C;
            if (InheritParameters) {
                x_value=myMath.getVariable("n_x");
                y_value=myMath.getVariable("n_y");
                n=new Complex(x_value, y_value);
                big=myMath.getVariable("big");
                x_value=myMath.getVariable("C_x");
                y_value=myMath.getVariable("C_y");
                C=new Complex(x_value, y_value);
            } else {
                n=new Complex(2,0);
                big=10;
                C=new Complex(cX,cY);
            }

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex Z=new Complex(xval,yval);

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'ExitIf' Directive: exitif:SqNorm(Z)>big;palette:0;split:GetIm(Z);palette:1,2
                            if (dummyC.SqNorm(Z)>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                SplitResult[0]=dummyC.GetIm(Z);
                                break Block0;
                            }

                            // 'Set' Directive: set:Z=(Z^n)+C
                            Z=(Z.Pow(n)).Add(C);

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData mandel_power_complex_julia_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 21;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      Complex n;
      double big;
      Complex C;
      if (InheritParameters) {
          x_value=myMath.getVariable("n_x");
          y_value=myMath.getVariable("n_y");
          n=new Complex(x_value, y_value);
          big=myMath.getVariable("big");
          x_value=myMath.getVariable("C_x");
          y_value=myMath.getVariable("C_y");
          C=new Complex(x_value, y_value);
      } else {
          n=new Complex(2,0);
          big=10;
          C=new Complex(cX,cY);
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex Z=new Complex(xval,yval);
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(Z), dummyC.GetIm(Z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'ExitIf' Directive: exitif:SqNorm(Z)>big;palette:0;split:GetIm(Z);palette:1,2
              if (dummyC.SqNorm(Z)>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:Z=(Z^n)+C
              Z=(Z.Pow(n)).Add(C);

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int random_mandel_complex(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 22;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            double n;
            double errorbound;
            double big;
            if (InheritParameters) {
                n=myMath.getVariable("n");
                errorbound=myMath.getVariable("errorbound");
                big=myMath.getVariable("big");
            } else {
                n=2;
                errorbound=0.01;
                big=50;
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);
            Complex p1=new Complex(-1,0);

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex Z=new Complex(xval,yval);
                    Complex C=Z;
                    Complex error=new Complex(0,0);

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'ExitIf' Directive: exitif:SqNorm(Z)>big;palette:0;split:GetIm(Z);palette:1,2
                            if (dummyC.SqNorm(Z)>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                SplitResult[0]=dummyC.GetIm(Z);
                                break Block0;
                            }

                            // 'Set' Directive: set:error=error.SetVals((2*random()-1)*errorbound,(2*random()-1)*errorbound)
                            error=error.SetVals((2*Math.random()-1)*errorbound,(2*Math.random()-1)*errorbound);

                            // 'Set' Directive: set:Z=(Z^n)+(C+error)
                            Z=(Z.Pow(n)).Add(C.Add(error));

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[1]=dummyC.GetRe(p1);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[1]=dummyC.GetIm(p1);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData random_mandel_complex_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 22;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double n;
      double errorbound;
      double big;
      if (InheritParameters) {
          n=myMath.getVariable("n");
          errorbound=myMath.getVariable("errorbound");
          big=myMath.getVariable("big");
      } else {
          n=2;
          errorbound=0.01;
          big=50;
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex Z=new Complex(xval,yval);
      Complex C=Z;
      Complex error=new Complex(0,0);
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(Z), dummyC.GetIm(Z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'ExitIf' Directive: exitif:SqNorm(Z)>big;palette:0;split:GetIm(Z);palette:1,2
              if (dummyC.SqNorm(Z)>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:error=error.SetVals((2*random()-1)*errorbound,(2*random()-1)*errorbound)
              error=error.SetVals((2*Math.random()-1)*errorbound,(2*Math.random()-1)*errorbound);

              // 'Set' Directive: set:Z=(Z^n)+(C+error)
              Z=(Z.Pow(n)).Add(C.Add(error));

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int random_complex_julia(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 23;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            double n;
            double big;
            double errorbound;
            Complex C;
            if (InheritParameters) {
                n=myMath.getVariable("n");
                big=myMath.getVariable("big");
                errorbound=myMath.getVariable("errorbound");
                x_value=myMath.getVariable("C_x");
                y_value=myMath.getVariable("C_y");
                C=new Complex(x_value, y_value);
            } else {
                n=2;
                big=50;
                errorbound=0.01;
                C=new Complex(cX,cY);
            }

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex Z=new Complex(xval,yval);
                    Complex error=new Complex(0,0);
                    double r=0;

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Set' Directive: set:r=SqNorm(Z)
                            r=dummyC.SqNorm(Z);

                            // 'ExitIf' Directive: exitif:r>big;palette:0
                            if (r>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Set' Directive: set:error=error.SetVals((2*random()-1)*errorbound,(2*random()-1)*errorbound)
                            error=error.SetVals((2*Math.random()-1)*errorbound,(2*Math.random()-1)*errorbound);

                            // 'Set' Directive: set:Z=(Z^n)+(C+error)
                            Z=(Z.Pow(n)).Add(C.Add(error));

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData random_complex_julia_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 23;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double n;
      double big;
      double errorbound;
      Complex C;
      if (InheritParameters) {
          n=myMath.getVariable("n");
          big=myMath.getVariable("big");
          errorbound=myMath.getVariable("errorbound");
          x_value=myMath.getVariable("C_x");
          y_value=myMath.getVariable("C_y");
          C=new Complex(x_value, y_value);
      } else {
          n=2;
          big=50;
          errorbound=0.01;
          C=new Complex(cX,cY);
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex Z=new Complex(xval,yval);
      Complex error=new Complex(0,0);
      double r=0;
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(Z), dummyC.GetIm(Z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'Set' Directive: set:r=SqNorm(Z)
              r=dummyC.SqNorm(Z);

              // 'ExitIf' Directive: exitif:r>big;palette:0
              if (r>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:error=error.SetVals((2*random()-1)*errorbound,(2*random()-1)*errorbound)
              error=error.SetVals((2*Math.random()-1)*errorbound,(2*Math.random()-1)*errorbound);

              // 'Set' Directive: set:Z=(Z^n)+(C+error)
              Z=(Z.Pow(n)).Add(C.Add(error));

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int ifs_affine(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 24;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            double big;
            int number;
            double angle1;
            double xscale1;
            double yscale1;
            double xpreshift1;
            double ypreshift1;
            double xpostshift1;
            double ypostshift1;
            double angle2;
            double xscale2;
            double yscale2;
            double xpreshift2;
            double ypreshift2;
            double xpostshift2;
            double ypostshift2;
            double angle3;
            double xscale3;
            double yscale3;
            double xpreshift3;
            double ypreshift3;
            double xpostshift3;
            double ypostshift3;
            double angle4;
            double xscale4;
            double yscale4;
            double xpreshift4;
            double ypreshift4;
            double xpostshift4;
            double ypostshift4;
            double angle5;
            double xscale5;
            double yscale5;
            double xpreshift5;
            double ypreshift5;
            double xpostshift5;
            double ypostshift5;
            double angle6;
            double xscale6;
            double yscale6;
            double xpreshift6;
            double ypreshift6;
            double xpostshift6;
            double ypostshift6;
            double angle7;
            double xscale7;
            double yscale7;
            double xpreshift7;
            double ypreshift7;
            double xpostshift7;
            double ypostshift7;
            double angle8;
            double xscale8;
            double yscale8;
            double xpreshift8;
            double ypreshift8;
            double xpostshift8;
            double ypostshift8;
            double angle9;
            double xscale9;
            double yscale9;
            double xpreshift9;
            double ypreshift9;
            double xpostshift9;
            double ypostshift9;
            double angleA;
            double xscaleA;
            double yscaleA;
            double xpreshiftA;
            double ypreshiftA;
            double xpostshiftA;
            double ypostshiftA;
            if (InheritParameters) {
                big=myMath.getVariable("big");
                number=(int) myMath.getVariable("number");
                angle1=myMath.getVariable("angle1");
                xscale1=myMath.getVariable("xscale1");
                yscale1=myMath.getVariable("yscale1");
                xpreshift1=myMath.getVariable("xpreshift1");
                ypreshift1=myMath.getVariable("ypreshift1");
                xpostshift1=myMath.getVariable("xpostshift1");
                ypostshift1=myMath.getVariable("ypostshift1");
                angle2=myMath.getVariable("angle2");
                xscale2=myMath.getVariable("xscale2");
                yscale2=myMath.getVariable("yscale2");
                xpreshift2=myMath.getVariable("xpreshift2");
                ypreshift2=myMath.getVariable("ypreshift2");
                xpostshift2=myMath.getVariable("xpostshift2");
                ypostshift2=myMath.getVariable("ypostshift2");
                angle3=myMath.getVariable("angle3");
                xscale3=myMath.getVariable("xscale3");
                yscale3=myMath.getVariable("yscale3");
                xpreshift3=myMath.getVariable("xpreshift3");
                ypreshift3=myMath.getVariable("ypreshift3");
                xpostshift3=myMath.getVariable("xpostshift3");
                ypostshift3=myMath.getVariable("ypostshift3");
                angle4=myMath.getVariable("angle4");
                xscale4=myMath.getVariable("xscale4");
                yscale4=myMath.getVariable("yscale4");
                xpreshift4=myMath.getVariable("xpreshift4");
                ypreshift4=myMath.getVariable("ypreshift4");
                xpostshift4=myMath.getVariable("xpostshift4");
                ypostshift4=myMath.getVariable("ypostshift4");
                angle5=myMath.getVariable("angle5");
                xscale5=myMath.getVariable("xscale5");
                yscale5=myMath.getVariable("yscale5");
                xpreshift5=myMath.getVariable("xpreshift5");
                ypreshift5=myMath.getVariable("ypreshift5");
                xpostshift5=myMath.getVariable("xpostshift5");
                ypostshift5=myMath.getVariable("ypostshift5");
                angle6=myMath.getVariable("angle6");
                xscale6=myMath.getVariable("xscale6");
                yscale6=myMath.getVariable("yscale6");
                xpreshift6=myMath.getVariable("xpreshift6");
                ypreshift6=myMath.getVariable("ypreshift6");
                xpostshift6=myMath.getVariable("xpostshift6");
                ypostshift6=myMath.getVariable("ypostshift6");
                angle7=myMath.getVariable("angle7");
                xscale7=myMath.getVariable("xscale7");
                yscale7=myMath.getVariable("yscale7");
                xpreshift7=myMath.getVariable("xpreshift7");
                ypreshift7=myMath.getVariable("ypreshift7");
                xpostshift7=myMath.getVariable("xpostshift7");
                ypostshift7=myMath.getVariable("ypostshift7");
                angle8=myMath.getVariable("angle8");
                xscale8=myMath.getVariable("xscale8");
                yscale8=myMath.getVariable("yscale8");
                xpreshift8=myMath.getVariable("xpreshift8");
                ypreshift8=myMath.getVariable("ypreshift8");
                xpostshift8=myMath.getVariable("xpostshift8");
                ypostshift8=myMath.getVariable("ypostshift8");
                angle9=myMath.getVariable("angle9");
                xscale9=myMath.getVariable("xscale9");
                yscale9=myMath.getVariable("yscale9");
                xpreshift9=myMath.getVariable("xpreshift9");
                ypreshift9=myMath.getVariable("ypreshift9");
                xpostshift9=myMath.getVariable("xpostshift9");
                ypostshift9=myMath.getVariable("ypostshift9");
                angleA=myMath.getVariable("angleA");
                xscaleA=myMath.getVariable("xscaleA");
                yscaleA=myMath.getVariable("yscaleA");
                xpreshiftA=myMath.getVariable("xpreshiftA");
                ypreshiftA=myMath.getVariable("ypreshiftA");
                xpostshiftA=myMath.getVariable("xpostshiftA");
                ypostshiftA=myMath.getVariable("ypostshiftA");
            } else {
                big=10000;
                number=3;
                angle1=0;
                xscale1=0.5;
                yscale1=0.5;
                xpreshift1=-0.5;
                ypreshift1=-0.5;
                xpostshift1=0;
                ypostshift1=0;
                angle2=0;
                xscale2=0.5;
                yscale2=0.5;
                xpreshift2=0.5;
                ypreshift2=-0.5;
                xpostshift2=0;
                ypostshift2=0;
                angle3=0;
                xscale3=0.5;
                yscale3=0.5;
                xpreshift3=0;
                ypreshift3=0.5;
                xpostshift3=0;
                ypostshift3=0;
                angle4=0;
                xscale4=0;
                yscale4=0;
                xpreshift4=0;
                ypreshift4=0;
                xpostshift4=0;
                ypostshift4=0;
                angle5=0;
                xscale5=0;
                yscale5=0;
                xpreshift5=0;
                ypreshift5=0;
                xpostshift5=0;
                ypostshift5=0;
                angle6=0;
                xscale6=0;
                yscale6=0;
                xpreshift6=0;
                ypreshift6=0;
                xpostshift6=0;
                ypostshift6=0;
                angle7=0;
                xscale7=0;
                yscale7=0;
                xpreshift7=0;
                ypreshift7=0;
                xpostshift7=0;
                ypostshift7=0;
                angle8=0;
                xscale8=0;
                yscale8=0;
                xpreshift8=0;
                ypreshift8=0;
                xpostshift8=0;
                ypostshift8=0;
                angle9=0;
                xscale9=0;
                yscale9=0;
                xpreshift9=0;
                ypreshift9=0;
                xpostshift9=0;
                ypostshift9=0;
                angleA=0;
                xscaleA=0;
                yscaleA=0;
                xpreshiftA=0;
                ypreshiftA=0;
                xpostshiftA=0;
                ypostshiftA=0;
            }

            // User-defined distinguished points are set before loop: 
            double px=0;
            double py=0;

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    double ifs_x=xval;
                    double ifs_y=yval;
                    double r=0;
                    int choice=0;
                    int weight1=0;
                    int weight2=0;
                    int weight3=0;
                    int weight4=0;
                    int weight5=0;
                    int weight6=0;
                    int weight7=0;
                    int weight8=0;
                    int weight9=0;
                    int weightA=0;
                    angle1=angle1*Math.PI/180;
                    angle2=angle2*Math.PI/180;
                    angle3=angle3*Math.PI/180;
                    angle4=angle4*Math.PI/180;
                    angle5=angle5*Math.PI/180;
                    angle6=angle6*Math.PI/180;
                    angle7=angle7*Math.PI/180;
                    angle8=angle8*Math.PI/180;
                    angle9=angle9*Math.PI/180;
                    angleA=angleA*Math.PI/180;
                    double oldifs_x=0;
                    double oldifs_y=0;

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Set' Directive: set:r=(ifs_x*ifs_x+ifs_y*ifs_y)
                            r=(ifs_x*ifs_x+ifs_y*ifs_y);

                            // 'ExitIf' Directive: exitif:r>big;palette:0
                            if (r>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Set' Directive: set:choice=RndInt(number)
                            choice=math.Misc.RndInt(number);

                            // 'Set' Directive: set:weight1=Delta(1,choice)
                            weight1=math.Misc.Delta(1,choice);

                            // 'Set' Directive: set:weight2=Delta(2,choice)
                            weight2=math.Misc.Delta(2,choice);

                            // 'Set' Directive: set:weight3=Delta(3,choice)
                            weight3=math.Misc.Delta(3,choice);

                            // 'Set' Directive: set:weight4=Delta(4,choice)
                            weight4=math.Misc.Delta(4,choice);

                            // 'Set' Directive: set:weight5=Delta(5,choice)
                            weight5=math.Misc.Delta(5,choice);

                            // 'Set' Directive: set:weight6=Delta(6,choice)
                            weight6=math.Misc.Delta(6,choice);

                            // 'Set' Directive: set:weight7=Delta(7,choice)
                            weight7=math.Misc.Delta(7,choice);

                            // 'Set' Directive: set:weight8=Delta(8,choice)
                            weight8=math.Misc.Delta(8,choice);

                            // 'Set' Directive: set:weight9=Delta(9,choice)
                            weight9=math.Misc.Delta(9,choice);

                            // 'Set' Directive: set:weightA=Delta(10,choice)
                            weightA=math.Misc.Delta(10,choice);

                            // 'Set' Directive: set:oldifs_x=ifs_x
                            oldifs_x=ifs_x;

                            // 'Set' Directive: set:oldifs_y=ifs_y
                            oldifs_y=ifs_y;

                            // 'Set' Directive: set:ifs_x=weight1*((oldifs_x*xscale1+xpreshift1)*cos(angle1)-sin(angle1)*(oldifs_y*yscale1+ypreshift1)+xpostshift1)+weight2*((oldifs_x*xscale2+xpreshift2)*cos(angle2)-sin(angle2)*(oldifs_y*yscale2+ypreshift2)+xpostshift2)+weight3*((oldifs_x*xscale3+xpreshift3)*cos(angle3)-sin(angle3)*(oldifs_y*yscale3+ypreshift3)+xpostshift3)+weight4*((oldifs_x*xscale4+xpreshift4)*cos(angle4)-sin(angle4)*(oldifs_y*yscale4+ypreshift4)+xpostshift4)+weight5*((oldifs_x*xscale5+xpreshift5)*cos(angle5)-sin(angle5)*(oldifs_y*yscale5+ypreshift5)+xpostshift5)+weight6*((oldifs_x*xscale6+xpreshift6)*cos(angle6)-sin(angle6)*(oldifs_y*yscale6+ypreshift6)+xpostshift6)+weight7*((oldifs_x*xscale7+xpreshift7)*cos(angle7)-sin(angle7)*(oldifs_y*yscale7+ypreshift7)+xpostshift7)+weight8*((oldifs_x*xscale8+xpreshift8)*cos(angle8)-sin(angle8)*(oldifs_y*yscale8+ypreshift8)+xpostshift8)+weight9*((oldifs_x*xscale9+xpreshift9)*cos(angle9)-sin(angle9)*(oldifs_y*yscale9+ypreshift9)+xpostshift9)+weightA*((oldifs_x*xscaleA+xpreshiftA)*cos(angleA)-sin(angleA)*(oldifs_y*yscaleA+ypreshiftA)+xpostshiftA)
                            ifs_x=weight1*((oldifs_x*xscale1+xpreshift1)*Math.cos(angle1)-Math.sin(angle1)*(oldifs_y*yscale1+ypreshift1)+xpostshift1)+weight2*((oldifs_x*xscale2+xpreshift2)*Math.cos(angle2)-Math.sin(angle2)*(oldifs_y*yscale2+ypreshift2)+xpostshift2)+weight3*((oldifs_x*xscale3+xpreshift3)*Math.cos(angle3)-Math.sin(angle3)*(oldifs_y*yscale3+ypreshift3)+xpostshift3)+weight4*((oldifs_x*xscale4+xpreshift4)*Math.cos(angle4)-Math.sin(angle4)*(oldifs_y*yscale4+ypreshift4)+xpostshift4)+weight5*((oldifs_x*xscale5+xpreshift5)*Math.cos(angle5)-Math.sin(angle5)*(oldifs_y*yscale5+ypreshift5)+xpostshift5)+weight6*((oldifs_x*xscale6+xpreshift6)*Math.cos(angle6)-Math.sin(angle6)*(oldifs_y*yscale6+ypreshift6)+xpostshift6)+weight7*((oldifs_x*xscale7+xpreshift7)*Math.cos(angle7)-Math.sin(angle7)*(oldifs_y*yscale7+ypreshift7)+xpostshift7)+weight8*((oldifs_x*xscale8+xpreshift8)*Math.cos(angle8)-Math.sin(angle8)*(oldifs_y*yscale8+ypreshift8)+xpostshift8)+weight9*((oldifs_x*xscale9+xpreshift9)*Math.cos(angle9)-Math.sin(angle9)*(oldifs_y*yscale9+ypreshift9)+xpostshift9)+weightA*((oldifs_x*xscaleA+xpreshiftA)*Math.cos(angleA)-Math.sin(angleA)*(oldifs_y*yscaleA+ypreshiftA)+xpostshiftA);

                            // 'Set' Directive: set:ifs_y=weight1*((oldifs_x*xscale1+xpreshift1)*sin(angle1)+cos(angle1)*(oldifs_y*yscale1+ypreshift1)+ypostshift1)+weight2*((oldifs_x*xscale2+xpreshift2)*sin(angle2)+cos(angle2)*(oldifs_y*yscale2+ypreshift2)+ypostshift2)+weight3*((oldifs_x*xscale3+xpreshift3)*sin(angle3)+cos(angle3)*(oldifs_y*yscale3+ypreshift3)+ypostshift3)+weight4*((oldifs_x*xscale4+xpreshift4)*sin(angle4)+cos(angle4)*(oldifs_y*yscale4+ypreshift4)+ypostshift4)+weight5*((oldifs_x*xscale5+xpreshift5)*sin(angle5)+cos(angle5)*(oldifs_y*yscale5+ypreshift5)+ypostshift5)+weight6*((oldifs_x*xscale6+xpreshift6)*sin(angle6)+cos(angle6)*(oldifs_y*yscale6+ypreshift6)+ypostshift6)+weight7*((oldifs_x*xscale7+xpreshift7)*sin(angle7)+cos(angle7)*(oldifs_y*yscale7+ypreshift7)+ypostshift7)+weight8*((oldifs_x*xscale8+xpreshift8)*sin(angle8)+cos(angle8)*(oldifs_y*yscale8+ypreshift8)+ypostshift8)+weight9*((oldifs_x*xscale9+xpreshift9)*sin(angle9)+cos(angle9)*(oldifs_y*yscale9+ypreshift9)+ypostshift9)+weightA*((oldifs_x*xscaleA+xpreshiftA)*sin(angleA)+cos(angleA)*(oldifs_y*yscaleA+ypreshiftA)+ypostshiftA)
                            ifs_y=weight1*((oldifs_x*xscale1+xpreshift1)*Math.sin(angle1)+Math.cos(angle1)*(oldifs_y*yscale1+ypreshift1)+ypostshift1)+weight2*((oldifs_x*xscale2+xpreshift2)*Math.sin(angle2)+Math.cos(angle2)*(oldifs_y*yscale2+ypreshift2)+ypostshift2)+weight3*((oldifs_x*xscale3+xpreshift3)*Math.sin(angle3)+Math.cos(angle3)*(oldifs_y*yscale3+ypreshift3)+ypostshift3)+weight4*((oldifs_x*xscale4+xpreshift4)*Math.sin(angle4)+Math.cos(angle4)*(oldifs_y*yscale4+ypreshift4)+ypostshift4)+weight5*((oldifs_x*xscale5+xpreshift5)*Math.sin(angle5)+Math.cos(angle5)*(oldifs_y*yscale5+ypreshift5)+ypostshift5)+weight6*((oldifs_x*xscale6+xpreshift6)*Math.sin(angle6)+Math.cos(angle6)*(oldifs_y*yscale6+ypreshift6)+ypostshift6)+weight7*((oldifs_x*xscale7+xpreshift7)*Math.sin(angle7)+Math.cos(angle7)*(oldifs_y*yscale7+ypreshift7)+ypostshift7)+weight8*((oldifs_x*xscale8+xpreshift8)*Math.sin(angle8)+Math.cos(angle8)*(oldifs_y*yscale8+ypreshift8)+ypostshift8)+weight9*((oldifs_x*xscale9+xpreshift9)*Math.sin(angle9)+Math.cos(angle9)*(oldifs_y*yscale9+ypreshift9)+ypostshift9)+weightA*((oldifs_x*xscaleA+xpreshiftA)*Math.sin(angleA)+Math.cos(angleA)*(oldifs_y*yscaleA+ypreshiftA)+ypostshiftA);

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=px;
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=py;
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData ifs_affine_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 24;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double big;
      int number;
      double angle1;
      double xscale1;
      double yscale1;
      double xpreshift1;
      double ypreshift1;
      double xpostshift1;
      double ypostshift1;
      double angle2;
      double xscale2;
      double yscale2;
      double xpreshift2;
      double ypreshift2;
      double xpostshift2;
      double ypostshift2;
      double angle3;
      double xscale3;
      double yscale3;
      double xpreshift3;
      double ypreshift3;
      double xpostshift3;
      double ypostshift3;
      double angle4;
      double xscale4;
      double yscale4;
      double xpreshift4;
      double ypreshift4;
      double xpostshift4;
      double ypostshift4;
      double angle5;
      double xscale5;
      double yscale5;
      double xpreshift5;
      double ypreshift5;
      double xpostshift5;
      double ypostshift5;
      double angle6;
      double xscale6;
      double yscale6;
      double xpreshift6;
      double ypreshift6;
      double xpostshift6;
      double ypostshift6;
      double angle7;
      double xscale7;
      double yscale7;
      double xpreshift7;
      double ypreshift7;
      double xpostshift7;
      double ypostshift7;
      double angle8;
      double xscale8;
      double yscale8;
      double xpreshift8;
      double ypreshift8;
      double xpostshift8;
      double ypostshift8;
      double angle9;
      double xscale9;
      double yscale9;
      double xpreshift9;
      double ypreshift9;
      double xpostshift9;
      double ypostshift9;
      double angleA;
      double xscaleA;
      double yscaleA;
      double xpreshiftA;
      double ypreshiftA;
      double xpostshiftA;
      double ypostshiftA;
      if (InheritParameters) {
          big=myMath.getVariable("big");
          number=(int) myMath.getVariable("number");
          angle1=myMath.getVariable("angle1");
          xscale1=myMath.getVariable("xscale1");
          yscale1=myMath.getVariable("yscale1");
          xpreshift1=myMath.getVariable("xpreshift1");
          ypreshift1=myMath.getVariable("ypreshift1");
          xpostshift1=myMath.getVariable("xpostshift1");
          ypostshift1=myMath.getVariable("ypostshift1");
          angle2=myMath.getVariable("angle2");
          xscale2=myMath.getVariable("xscale2");
          yscale2=myMath.getVariable("yscale2");
          xpreshift2=myMath.getVariable("xpreshift2");
          ypreshift2=myMath.getVariable("ypreshift2");
          xpostshift2=myMath.getVariable("xpostshift2");
          ypostshift2=myMath.getVariable("ypostshift2");
          angle3=myMath.getVariable("angle3");
          xscale3=myMath.getVariable("xscale3");
          yscale3=myMath.getVariable("yscale3");
          xpreshift3=myMath.getVariable("xpreshift3");
          ypreshift3=myMath.getVariable("ypreshift3");
          xpostshift3=myMath.getVariable("xpostshift3");
          ypostshift3=myMath.getVariable("ypostshift3");
          angle4=myMath.getVariable("angle4");
          xscale4=myMath.getVariable("xscale4");
          yscale4=myMath.getVariable("yscale4");
          xpreshift4=myMath.getVariable("xpreshift4");
          ypreshift4=myMath.getVariable("ypreshift4");
          xpostshift4=myMath.getVariable("xpostshift4");
          ypostshift4=myMath.getVariable("ypostshift4");
          angle5=myMath.getVariable("angle5");
          xscale5=myMath.getVariable("xscale5");
          yscale5=myMath.getVariable("yscale5");
          xpreshift5=myMath.getVariable("xpreshift5");
          ypreshift5=myMath.getVariable("ypreshift5");
          xpostshift5=myMath.getVariable("xpostshift5");
          ypostshift5=myMath.getVariable("ypostshift5");
          angle6=myMath.getVariable("angle6");
          xscale6=myMath.getVariable("xscale6");
          yscale6=myMath.getVariable("yscale6");
          xpreshift6=myMath.getVariable("xpreshift6");
          ypreshift6=myMath.getVariable("ypreshift6");
          xpostshift6=myMath.getVariable("xpostshift6");
          ypostshift6=myMath.getVariable("ypostshift6");
          angle7=myMath.getVariable("angle7");
          xscale7=myMath.getVariable("xscale7");
          yscale7=myMath.getVariable("yscale7");
          xpreshift7=myMath.getVariable("xpreshift7");
          ypreshift7=myMath.getVariable("ypreshift7");
          xpostshift7=myMath.getVariable("xpostshift7");
          ypostshift7=myMath.getVariable("ypostshift7");
          angle8=myMath.getVariable("angle8");
          xscale8=myMath.getVariable("xscale8");
          yscale8=myMath.getVariable("yscale8");
          xpreshift8=myMath.getVariable("xpreshift8");
          ypreshift8=myMath.getVariable("ypreshift8");
          xpostshift8=myMath.getVariable("xpostshift8");
          ypostshift8=myMath.getVariable("ypostshift8");
          angle9=myMath.getVariable("angle9");
          xscale9=myMath.getVariable("xscale9");
          yscale9=myMath.getVariable("yscale9");
          xpreshift9=myMath.getVariable("xpreshift9");
          ypreshift9=myMath.getVariable("ypreshift9");
          xpostshift9=myMath.getVariable("xpostshift9");
          ypostshift9=myMath.getVariable("ypostshift9");
          angleA=myMath.getVariable("angleA");
          xscaleA=myMath.getVariable("xscaleA");
          yscaleA=myMath.getVariable("yscaleA");
          xpreshiftA=myMath.getVariable("xpreshiftA");
          ypreshiftA=myMath.getVariable("ypreshiftA");
          xpostshiftA=myMath.getVariable("xpostshiftA");
          ypostshiftA=myMath.getVariable("ypostshiftA");
      } else {
          big=10000;
          number=3;
          angle1=0;
          xscale1=0.5;
          yscale1=0.5;
          xpreshift1=-0.5;
          ypreshift1=-0.5;
          xpostshift1=0;
          ypostshift1=0;
          angle2=0;
          xscale2=0.5;
          yscale2=0.5;
          xpreshift2=0.5;
          ypreshift2=-0.5;
          xpostshift2=0;
          ypostshift2=0;
          angle3=0;
          xscale3=0.5;
          yscale3=0.5;
          xpreshift3=0;
          ypreshift3=0.5;
          xpostshift3=0;
          ypostshift3=0;
          angle4=0;
          xscale4=0;
          yscale4=0;
          xpreshift4=0;
          ypreshift4=0;
          xpostshift4=0;
          ypostshift4=0;
          angle5=0;
          xscale5=0;
          yscale5=0;
          xpreshift5=0;
          ypreshift5=0;
          xpostshift5=0;
          ypostshift5=0;
          angle6=0;
          xscale6=0;
          yscale6=0;
          xpreshift6=0;
          ypreshift6=0;
          xpostshift6=0;
          ypostshift6=0;
          angle7=0;
          xscale7=0;
          yscale7=0;
          xpreshift7=0;
          ypreshift7=0;
          xpostshift7=0;
          ypostshift7=0;
          angle8=0;
          xscale8=0;
          yscale8=0;
          xpreshift8=0;
          ypreshift8=0;
          xpostshift8=0;
          ypostshift8=0;
          angle9=0;
          xscale9=0;
          yscale9=0;
          xpreshift9=0;
          ypreshift9=0;
          xpostshift9=0;
          ypostshift9=0;
          angleA=0;
          xscaleA=0;
          yscaleA=0;
          xpreshiftA=0;
          ypreshiftA=0;
          xpostshiftA=0;
          ypostshiftA=0;
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      double ifs_x=xval;
      double ifs_y=yval;
      double r=0;
      int choice=0;
      int weight1=0;
      int weight2=0;
      int weight3=0;
      int weight4=0;
      int weight5=0;
      int weight6=0;
      int weight7=0;
      int weight8=0;
      int weight9=0;
      int weightA=0;
      angle1=angle1*Math.PI/180;
      angle2=angle2*Math.PI/180;
      angle3=angle3*Math.PI/180;
      angle4=angle4*Math.PI/180;
      angle5=angle5*Math.PI/180;
      angle6=angle6*Math.PI/180;
      angle7=angle7*Math.PI/180;
      angle8=angle8*Math.PI/180;
      angle9=angle9*Math.PI/180;
      angleA=angleA*Math.PI/180;
      double oldifs_x=0;
      double oldifs_y=0;
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, ifs_x, ifs_y);
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'Set' Directive: set:r=(ifs_x*ifs_x+ifs_y*ifs_y)
              r=(ifs_x*ifs_x+ifs_y*ifs_y);

              // 'ExitIf' Directive: exitif:r>big;palette:0
              if (r>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:choice=RndInt(number)
              choice=math.Misc.RndInt(number);

              // 'Set' Directive: set:weight1=Delta(1,choice)
              weight1=math.Misc.Delta(1,choice);

              // 'Set' Directive: set:weight2=Delta(2,choice)
              weight2=math.Misc.Delta(2,choice);

              // 'Set' Directive: set:weight3=Delta(3,choice)
              weight3=math.Misc.Delta(3,choice);

              // 'Set' Directive: set:weight4=Delta(4,choice)
              weight4=math.Misc.Delta(4,choice);

              // 'Set' Directive: set:weight5=Delta(5,choice)
              weight5=math.Misc.Delta(5,choice);

              // 'Set' Directive: set:weight6=Delta(6,choice)
              weight6=math.Misc.Delta(6,choice);

              // 'Set' Directive: set:weight7=Delta(7,choice)
              weight7=math.Misc.Delta(7,choice);

              // 'Set' Directive: set:weight8=Delta(8,choice)
              weight8=math.Misc.Delta(8,choice);

              // 'Set' Directive: set:weight9=Delta(9,choice)
              weight9=math.Misc.Delta(9,choice);

              // 'Set' Directive: set:weightA=Delta(10,choice)
              weightA=math.Misc.Delta(10,choice);

              // 'Set' Directive: set:oldifs_x=ifs_x
              oldifs_x=ifs_x;

              // 'Set' Directive: set:oldifs_y=ifs_y
              oldifs_y=ifs_y;

              // 'Set' Directive: set:ifs_x=weight1*((oldifs_x*xscale1+xpreshift1)*cos(angle1)-sin(angle1)*(oldifs_y*yscale1+ypreshift1)+xpostshift1)+weight2*((oldifs_x*xscale2+xpreshift2)*cos(angle2)-sin(angle2)*(oldifs_y*yscale2+ypreshift2)+xpostshift2)+weight3*((oldifs_x*xscale3+xpreshift3)*cos(angle3)-sin(angle3)*(oldifs_y*yscale3+ypreshift3)+xpostshift3)+weight4*((oldifs_x*xscale4+xpreshift4)*cos(angle4)-sin(angle4)*(oldifs_y*yscale4+ypreshift4)+xpostshift4)+weight5*((oldifs_x*xscale5+xpreshift5)*cos(angle5)-sin(angle5)*(oldifs_y*yscale5+ypreshift5)+xpostshift5)+weight6*((oldifs_x*xscale6+xpreshift6)*cos(angle6)-sin(angle6)*(oldifs_y*yscale6+ypreshift6)+xpostshift6)+weight7*((oldifs_x*xscale7+xpreshift7)*cos(angle7)-sin(angle7)*(oldifs_y*yscale7+ypreshift7)+xpostshift7)+weight8*((oldifs_x*xscale8+xpreshift8)*cos(angle8)-sin(angle8)*(oldifs_y*yscale8+ypreshift8)+xpostshift8)+weight9*((oldifs_x*xscale9+xpreshift9)*cos(angle9)-sin(angle9)*(oldifs_y*yscale9+ypreshift9)+xpostshift9)+weightA*((oldifs_x*xscaleA+xpreshiftA)*cos(angleA)-sin(angleA)*(oldifs_y*yscaleA+ypreshiftA)+xpostshiftA)
              ifs_x=weight1*((oldifs_x*xscale1+xpreshift1)*Math.cos(angle1)-Math.sin(angle1)*(oldifs_y*yscale1+ypreshift1)+xpostshift1)+weight2*((oldifs_x*xscale2+xpreshift2)*Math.cos(angle2)-Math.sin(angle2)*(oldifs_y*yscale2+ypreshift2)+xpostshift2)+weight3*((oldifs_x*xscale3+xpreshift3)*Math.cos(angle3)-Math.sin(angle3)*(oldifs_y*yscale3+ypreshift3)+xpostshift3)+weight4*((oldifs_x*xscale4+xpreshift4)*Math.cos(angle4)-Math.sin(angle4)*(oldifs_y*yscale4+ypreshift4)+xpostshift4)+weight5*((oldifs_x*xscale5+xpreshift5)*Math.cos(angle5)-Math.sin(angle5)*(oldifs_y*yscale5+ypreshift5)+xpostshift5)+weight6*((oldifs_x*xscale6+xpreshift6)*Math.cos(angle6)-Math.sin(angle6)*(oldifs_y*yscale6+ypreshift6)+xpostshift6)+weight7*((oldifs_x*xscale7+xpreshift7)*Math.cos(angle7)-Math.sin(angle7)*(oldifs_y*yscale7+ypreshift7)+xpostshift7)+weight8*((oldifs_x*xscale8+xpreshift8)*Math.cos(angle8)-Math.sin(angle8)*(oldifs_y*yscale8+ypreshift8)+xpostshift8)+weight9*((oldifs_x*xscale9+xpreshift9)*Math.cos(angle9)-Math.sin(angle9)*(oldifs_y*yscale9+ypreshift9)+xpostshift9)+weightA*((oldifs_x*xscaleA+xpreshiftA)*Math.cos(angleA)-Math.sin(angleA)*(oldifs_y*yscaleA+ypreshiftA)+xpostshiftA);

              // 'Set' Directive: set:ifs_y=weight1*((oldifs_x*xscale1+xpreshift1)*sin(angle1)+cos(angle1)*(oldifs_y*yscale1+ypreshift1)+ypostshift1)+weight2*((oldifs_x*xscale2+xpreshift2)*sin(angle2)+cos(angle2)*(oldifs_y*yscale2+ypreshift2)+ypostshift2)+weight3*((oldifs_x*xscale3+xpreshift3)*sin(angle3)+cos(angle3)*(oldifs_y*yscale3+ypreshift3)+ypostshift3)+weight4*((oldifs_x*xscale4+xpreshift4)*sin(angle4)+cos(angle4)*(oldifs_y*yscale4+ypreshift4)+ypostshift4)+weight5*((oldifs_x*xscale5+xpreshift5)*sin(angle5)+cos(angle5)*(oldifs_y*yscale5+ypreshift5)+ypostshift5)+weight6*((oldifs_x*xscale6+xpreshift6)*sin(angle6)+cos(angle6)*(oldifs_y*yscale6+ypreshift6)+ypostshift6)+weight7*((oldifs_x*xscale7+xpreshift7)*sin(angle7)+cos(angle7)*(oldifs_y*yscale7+ypreshift7)+ypostshift7)+weight8*((oldifs_x*xscale8+xpreshift8)*sin(angle8)+cos(angle8)*(oldifs_y*yscale8+ypreshift8)+ypostshift8)+weight9*((oldifs_x*xscale9+xpreshift9)*sin(angle9)+cos(angle9)*(oldifs_y*yscale9+ypreshift9)+ypostshift9)+weightA*((oldifs_x*xscaleA+xpreshiftA)*sin(angleA)+cos(angleA)*(oldifs_y*yscaleA+ypreshiftA)+ypostshiftA)
              ifs_y=weight1*((oldifs_x*xscale1+xpreshift1)*Math.sin(angle1)+Math.cos(angle1)*(oldifs_y*yscale1+ypreshift1)+ypostshift1)+weight2*((oldifs_x*xscale2+xpreshift2)*Math.sin(angle2)+Math.cos(angle2)*(oldifs_y*yscale2+ypreshift2)+ypostshift2)+weight3*((oldifs_x*xscale3+xpreshift3)*Math.sin(angle3)+Math.cos(angle3)*(oldifs_y*yscale3+ypreshift3)+ypostshift3)+weight4*((oldifs_x*xscale4+xpreshift4)*Math.sin(angle4)+Math.cos(angle4)*(oldifs_y*yscale4+ypreshift4)+ypostshift4)+weight5*((oldifs_x*xscale5+xpreshift5)*Math.sin(angle5)+Math.cos(angle5)*(oldifs_y*yscale5+ypreshift5)+ypostshift5)+weight6*((oldifs_x*xscale6+xpreshift6)*Math.sin(angle6)+Math.cos(angle6)*(oldifs_y*yscale6+ypreshift6)+ypostshift6)+weight7*((oldifs_x*xscale7+xpreshift7)*Math.sin(angle7)+Math.cos(angle7)*(oldifs_y*yscale7+ypreshift7)+ypostshift7)+weight8*((oldifs_x*xscale8+xpreshift8)*Math.sin(angle8)+Math.cos(angle8)*(oldifs_y*yscale8+ypreshift8)+ypostshift8)+weight9*((oldifs_x*xscale9+xpreshift9)*Math.sin(angle9)+Math.cos(angle9)*(oldifs_y*yscale9+ypreshift9)+ypostshift9)+weightA*((oldifs_x*xscaleA+xpreshiftA)*Math.sin(angleA)+Math.cos(angleA)*(oldifs_y*yscaleA+ypreshiftA)+ypostshiftA);

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int ifs_carpet(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 25;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            double big;
            double angle1;
            double xscale1;
            double xshift1;
            double yscale1;
            double yshift1;
            double angle2;
            double xscale2;
            double xshift2;
            double yscale2;
            double yshift2;
            double angle3;
            double xscale3;
            double xshift3;
            double yscale3;
            double yshift3;
            double angle4;
            double xscale4;
            double xshift4;
            double yscale4;
            double yshift4;
            double angle5;
            double xscale5;
            double xshift5;
            double yscale5;
            double yshift5;
            double angle6;
            double xscale6;
            double xshift6;
            double yscale6;
            double yshift6;
            double angle7;
            double xscale7;
            double xshift7;
            double yscale7;
            double yshift7;
            double angle8;
            double xscale8;
            double xshift8;
            double yscale8;
            double yshift8;
            if (InheritParameters) {
                big=myMath.getVariable("big");
                angle1=myMath.getVariable("angle1");
                xscale1=myMath.getVariable("xscale1");
                xshift1=myMath.getVariable("xshift1");
                yscale1=myMath.getVariable("yscale1");
                yshift1=myMath.getVariable("yshift1");
                angle2=myMath.getVariable("angle2");
                xscale2=myMath.getVariable("xscale2");
                xshift2=myMath.getVariable("xshift2");
                yscale2=myMath.getVariable("yscale2");
                yshift2=myMath.getVariable("yshift2");
                angle3=myMath.getVariable("angle3");
                xscale3=myMath.getVariable("xscale3");
                xshift3=myMath.getVariable("xshift3");
                yscale3=myMath.getVariable("yscale3");
                yshift3=myMath.getVariable("yshift3");
                angle4=myMath.getVariable("angle4");
                xscale4=myMath.getVariable("xscale4");
                xshift4=myMath.getVariable("xshift4");
                yscale4=myMath.getVariable("yscale4");
                yshift4=myMath.getVariable("yshift4");
                angle5=myMath.getVariable("angle5");
                xscale5=myMath.getVariable("xscale5");
                xshift5=myMath.getVariable("xshift5");
                yscale5=myMath.getVariable("yscale5");
                yshift5=myMath.getVariable("yshift5");
                angle6=myMath.getVariable("angle6");
                xscale6=myMath.getVariable("xscale6");
                xshift6=myMath.getVariable("xshift6");
                yscale6=myMath.getVariable("yscale6");
                yshift6=myMath.getVariable("yshift6");
                angle7=myMath.getVariable("angle7");
                xscale7=myMath.getVariable("xscale7");
                xshift7=myMath.getVariable("xshift7");
                yscale7=myMath.getVariable("yscale7");
                yshift7=myMath.getVariable("yshift7");
                angle8=myMath.getVariable("angle8");
                xscale8=myMath.getVariable("xscale8");
                xshift8=myMath.getVariable("xshift8");
                yscale8=myMath.getVariable("yscale8");
                yshift8=myMath.getVariable("yshift8");
            } else {
                big=10000;
                angle1=0;
                xscale1=1/3;
                xshift1=-1;
                yscale1=1/3;
                yshift1=-1;
                angle2=0;
                xscale2=1/3;
                xshift2=-1;
                yscale2=1/3;
                yshift2=0;
                angle3=0;
                xscale3=1/3;
                xshift3=-1;
                yscale3=1/3;
                yshift3=1;
                angle4=0;
                xscale4=1/3;
                xshift4=0;
                yscale4=1/3;
                yshift4=1;
                angle5=0;
                xscale5=1/3;
                xshift5=1;
                yscale5=1/3;
                yshift5=1;
                angle6=0;
                xscale6=1/3;
                xshift6=1;
                yscale6=1/3;
                yshift6=0;
                angle7=0;
                xscale7=1/3;
                xshift7=1;
                yscale7=1/3;
                yshift7=-1;
                angle8=0;
                xscale8=1/3;
                xshift8=0;
                yscale8=1/3;
                yshift8=-1;
            }

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    double ifs_x=xval;
                    double ifs_y=yval;
                    double r=0;
                    int choice=0;
                    int weight1=0;
                    int weight2=0;
                    int weight3=0;
                    int weight4=0;
                    int weight5=0;
                    int weight6=0;
                    int weight7=0;
                    int weight8=0;
                    angle1=angle1*Math.PI/180;
                    angle2=angle2*Math.PI/180;
                    angle3=angle3*Math.PI/180;
                    angle4=angle4*Math.PI/180;
                    angle5=angle5*Math.PI/180;
                    angle6=angle6*Math.PI/180;
                    angle7=angle7*Math.PI/180;
                    angle8=angle8*Math.PI/180;
                    double oldifs_x=0;
                    double oldifs_y=0;

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Set' Directive: set:r=(ifs_x*ifs_x+ifs_y*ifs_y)
                            r=(ifs_x*ifs_x+ifs_y*ifs_y);

                            // 'ExitIf' Directive: exitif:r>big;palette:0
                            if (r>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Set' Directive: set:choice=RndInt(8)
                            choice=math.Misc.RndInt(8);

                            // 'Set' Directive: set:weight1=Delta(1,choice)
                            weight1=math.Misc.Delta(1,choice);

                            // 'Set' Directive: set:weight2=Delta(2,choice)
                            weight2=math.Misc.Delta(2,choice);

                            // 'Set' Directive: set:weight3=Delta(3,choice)
                            weight3=math.Misc.Delta(3,choice);

                            // 'Set' Directive: set:weight4=Delta(4,choice)
                            weight4=math.Misc.Delta(4,choice);

                            // 'Set' Directive: set:weight5=Delta(5,choice)
                            weight5=math.Misc.Delta(5,choice);

                            // 'Set' Directive: set:weight6=Delta(6,choice)
                            weight6=math.Misc.Delta(6,choice);

                            // 'Set' Directive: set:weight7=Delta(7,choice)
                            weight7=math.Misc.Delta(7,choice);

                            // 'Set' Directive: set:weight8=Delta(8,choice)
                            weight8=math.Misc.Delta(8,choice);

                            // 'Set' Directive: set:oldifs_x=ifs_x
                            oldifs_x=ifs_x;

                            // 'Set' Directive: set:oldifs_y=ifs_y
                            oldifs_y=ifs_y;

                            // 'Set' Directive: set:ifs_x=weight1*(oldifs_x*xscale1*cos(angle1)-sin(angle1)*oldifs_y*yscale1+xshift1)+weight2*(oldifs_x*xscale2*cos(angle2)-sin(angle2)*oldifs_y*yscale2+xshift2)+weight3*(oldifs_x*xscale3*cos(angle3)-sin(angle3)*oldifs_y*yscale3+xshift3)+weight4*(oldifs_x*xscale4*cos(angle4)-sin(angle4)*oldifs_y*yscale4+xshift4)+weight5*(oldifs_x*xscale5*cos(angle5)-sin(angle5)*oldifs_y*yscale5+xshift5)+weight6*(oldifs_x*xscale6*cos(angle6)-sin(angle6)*oldifs_y*yscale6+xshift6)+weight7*(oldifs_x*xscale7*cos(angle7)-sin(angle7)*oldifs_y*yscale7+xshift7)+weight8*(oldifs_x*xscale8*cos(angle8)-sin(angle8)*oldifs_y*yscale8+xshift8)
                            ifs_x=weight1*(oldifs_x*xscale1*Math.cos(angle1)-Math.sin(angle1)*oldifs_y*yscale1+xshift1)+weight2*(oldifs_x*xscale2*Math.cos(angle2)-Math.sin(angle2)*oldifs_y*yscale2+xshift2)+weight3*(oldifs_x*xscale3*Math.cos(angle3)-Math.sin(angle3)*oldifs_y*yscale3+xshift3)+weight4*(oldifs_x*xscale4*Math.cos(angle4)-Math.sin(angle4)*oldifs_y*yscale4+xshift4)+weight5*(oldifs_x*xscale5*Math.cos(angle5)-Math.sin(angle5)*oldifs_y*yscale5+xshift5)+weight6*(oldifs_x*xscale6*Math.cos(angle6)-Math.sin(angle6)*oldifs_y*yscale6+xshift6)+weight7*(oldifs_x*xscale7*Math.cos(angle7)-Math.sin(angle7)*oldifs_y*yscale7+xshift7)+weight8*(oldifs_x*xscale8*Math.cos(angle8)-Math.sin(angle8)*oldifs_y*yscale8+xshift8);

                            // 'Set' Directive: set:ifs_y=weight1*(oldifs_x*xscale1*sin(angle1)+cos(angle1)*oldifs_y*yscale1+yshift1)+weight2*(oldifs_x*xscale2*sin(angle2)+cos(angle2)*oldifs_y*yscale2+yshift2)+weight3*(oldifs_x*xscale3*sin(angle3)+cos(angle3)*oldifs_y*yscale3+yshift3)+weight4*(oldifs_x*xscale4*sin(angle4)+cos(angle4)*oldifs_y*yscale4+yshift4)+weight5*(oldifs_x*xscale5*sin(angle5)+cos(angle5)*oldifs_y*yscale5+yshift5)+weight6*(oldifs_x*xscale6*sin(angle6)+cos(angle6)*oldifs_y*yscale6+yshift6)+weight7*(oldifs_x*xscale7*sin(angle7)+cos(angle7)*oldifs_y*yscale7+yshift7)+weight8*(oldifs_x*xscale8*sin(angle8)+cos(angle8)*oldifs_y*yscale8+yshift8)
                            ifs_y=weight1*(oldifs_x*xscale1*Math.sin(angle1)+Math.cos(angle1)*oldifs_y*yscale1+yshift1)+weight2*(oldifs_x*xscale2*Math.sin(angle2)+Math.cos(angle2)*oldifs_y*yscale2+yshift2)+weight3*(oldifs_x*xscale3*Math.sin(angle3)+Math.cos(angle3)*oldifs_y*yscale3+yshift3)+weight4*(oldifs_x*xscale4*Math.sin(angle4)+Math.cos(angle4)*oldifs_y*yscale4+yshift4)+weight5*(oldifs_x*xscale5*Math.sin(angle5)+Math.cos(angle5)*oldifs_y*yscale5+yshift5)+weight6*(oldifs_x*xscale6*Math.sin(angle6)+Math.cos(angle6)*oldifs_y*yscale6+yshift6)+weight7*(oldifs_x*xscale7*Math.sin(angle7)+Math.cos(angle7)*oldifs_y*yscale7+yshift7)+weight8*(oldifs_x*xscale8*Math.sin(angle8)+Math.cos(angle8)*oldifs_y*yscale8+yshift8);

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData ifs_carpet_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 25;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double big;
      double angle1;
      double xscale1;
      double xshift1;
      double yscale1;
      double yshift1;
      double angle2;
      double xscale2;
      double xshift2;
      double yscale2;
      double yshift2;
      double angle3;
      double xscale3;
      double xshift3;
      double yscale3;
      double yshift3;
      double angle4;
      double xscale4;
      double xshift4;
      double yscale4;
      double yshift4;
      double angle5;
      double xscale5;
      double xshift5;
      double yscale5;
      double yshift5;
      double angle6;
      double xscale6;
      double xshift6;
      double yscale6;
      double yshift6;
      double angle7;
      double xscale7;
      double xshift7;
      double yscale7;
      double yshift7;
      double angle8;
      double xscale8;
      double xshift8;
      double yscale8;
      double yshift8;
      if (InheritParameters) {
          big=myMath.getVariable("big");
          angle1=myMath.getVariable("angle1");
          xscale1=myMath.getVariable("xscale1");
          xshift1=myMath.getVariable("xshift1");
          yscale1=myMath.getVariable("yscale1");
          yshift1=myMath.getVariable("yshift1");
          angle2=myMath.getVariable("angle2");
          xscale2=myMath.getVariable("xscale2");
          xshift2=myMath.getVariable("xshift2");
          yscale2=myMath.getVariable("yscale2");
          yshift2=myMath.getVariable("yshift2");
          angle3=myMath.getVariable("angle3");
          xscale3=myMath.getVariable("xscale3");
          xshift3=myMath.getVariable("xshift3");
          yscale3=myMath.getVariable("yscale3");
          yshift3=myMath.getVariable("yshift3");
          angle4=myMath.getVariable("angle4");
          xscale4=myMath.getVariable("xscale4");
          xshift4=myMath.getVariable("xshift4");
          yscale4=myMath.getVariable("yscale4");
          yshift4=myMath.getVariable("yshift4");
          angle5=myMath.getVariable("angle5");
          xscale5=myMath.getVariable("xscale5");
          xshift5=myMath.getVariable("xshift5");
          yscale5=myMath.getVariable("yscale5");
          yshift5=myMath.getVariable("yshift5");
          angle6=myMath.getVariable("angle6");
          xscale6=myMath.getVariable("xscale6");
          xshift6=myMath.getVariable("xshift6");
          yscale6=myMath.getVariable("yscale6");
          yshift6=myMath.getVariable("yshift6");
          angle7=myMath.getVariable("angle7");
          xscale7=myMath.getVariable("xscale7");
          xshift7=myMath.getVariable("xshift7");
          yscale7=myMath.getVariable("yscale7");
          yshift7=myMath.getVariable("yshift7");
          angle8=myMath.getVariable("angle8");
          xscale8=myMath.getVariable("xscale8");
          xshift8=myMath.getVariable("xshift8");
          yscale8=myMath.getVariable("yscale8");
          yshift8=myMath.getVariable("yshift8");
      } else {
          big=10000;
          angle1=0;
          xscale1=1/3;
          xshift1=-1;
          yscale1=1/3;
          yshift1=-1;
          angle2=0;
          xscale2=1/3;
          xshift2=-1;
          yscale2=1/3;
          yshift2=0;
          angle3=0;
          xscale3=1/3;
          xshift3=-1;
          yscale3=1/3;
          yshift3=1;
          angle4=0;
          xscale4=1/3;
          xshift4=0;
          yscale4=1/3;
          yshift4=1;
          angle5=0;
          xscale5=1/3;
          xshift5=1;
          yscale5=1/3;
          yshift5=1;
          angle6=0;
          xscale6=1/3;
          xshift6=1;
          yscale6=1/3;
          yshift6=0;
          angle7=0;
          xscale7=1/3;
          xshift7=1;
          yscale7=1/3;
          yshift7=-1;
          angle8=0;
          xscale8=1/3;
          xshift8=0;
          yscale8=1/3;
          yshift8=-1;
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      double ifs_x=xval;
      double ifs_y=yval;
      double r=0;
      int choice=0;
      int weight1=0;
      int weight2=0;
      int weight3=0;
      int weight4=0;
      int weight5=0;
      int weight6=0;
      int weight7=0;
      int weight8=0;
      angle1=angle1*Math.PI/180;
      angle2=angle2*Math.PI/180;
      angle3=angle3*Math.PI/180;
      angle4=angle4*Math.PI/180;
      angle5=angle5*Math.PI/180;
      angle6=angle6*Math.PI/180;
      angle7=angle7*Math.PI/180;
      angle8=angle8*Math.PI/180;
      double oldifs_x=0;
      double oldifs_y=0;
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, ifs_x, ifs_y);
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'Set' Directive: set:r=(ifs_x*ifs_x+ifs_y*ifs_y)
              r=(ifs_x*ifs_x+ifs_y*ifs_y);

              // 'ExitIf' Directive: exitif:r>big;palette:0
              if (r>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:choice=RndInt(8)
              choice=math.Misc.RndInt(8);

              // 'Set' Directive: set:weight1=Delta(1,choice)
              weight1=math.Misc.Delta(1,choice);

              // 'Set' Directive: set:weight2=Delta(2,choice)
              weight2=math.Misc.Delta(2,choice);

              // 'Set' Directive: set:weight3=Delta(3,choice)
              weight3=math.Misc.Delta(3,choice);

              // 'Set' Directive: set:weight4=Delta(4,choice)
              weight4=math.Misc.Delta(4,choice);

              // 'Set' Directive: set:weight5=Delta(5,choice)
              weight5=math.Misc.Delta(5,choice);

              // 'Set' Directive: set:weight6=Delta(6,choice)
              weight6=math.Misc.Delta(6,choice);

              // 'Set' Directive: set:weight7=Delta(7,choice)
              weight7=math.Misc.Delta(7,choice);

              // 'Set' Directive: set:weight8=Delta(8,choice)
              weight8=math.Misc.Delta(8,choice);

              // 'Set' Directive: set:oldifs_x=ifs_x
              oldifs_x=ifs_x;

              // 'Set' Directive: set:oldifs_y=ifs_y
              oldifs_y=ifs_y;

              // 'Set' Directive: set:ifs_x=weight1*(oldifs_x*xscale1*cos(angle1)-sin(angle1)*oldifs_y*yscale1+xshift1)+weight2*(oldifs_x*xscale2*cos(angle2)-sin(angle2)*oldifs_y*yscale2+xshift2)+weight3*(oldifs_x*xscale3*cos(angle3)-sin(angle3)*oldifs_y*yscale3+xshift3)+weight4*(oldifs_x*xscale4*cos(angle4)-sin(angle4)*oldifs_y*yscale4+xshift4)+weight5*(oldifs_x*xscale5*cos(angle5)-sin(angle5)*oldifs_y*yscale5+xshift5)+weight6*(oldifs_x*xscale6*cos(angle6)-sin(angle6)*oldifs_y*yscale6+xshift6)+weight7*(oldifs_x*xscale7*cos(angle7)-sin(angle7)*oldifs_y*yscale7+xshift7)+weight8*(oldifs_x*xscale8*cos(angle8)-sin(angle8)*oldifs_y*yscale8+xshift8)
              ifs_x=weight1*(oldifs_x*xscale1*Math.cos(angle1)-Math.sin(angle1)*oldifs_y*yscale1+xshift1)+weight2*(oldifs_x*xscale2*Math.cos(angle2)-Math.sin(angle2)*oldifs_y*yscale2+xshift2)+weight3*(oldifs_x*xscale3*Math.cos(angle3)-Math.sin(angle3)*oldifs_y*yscale3+xshift3)+weight4*(oldifs_x*xscale4*Math.cos(angle4)-Math.sin(angle4)*oldifs_y*yscale4+xshift4)+weight5*(oldifs_x*xscale5*Math.cos(angle5)-Math.sin(angle5)*oldifs_y*yscale5+xshift5)+weight6*(oldifs_x*xscale6*Math.cos(angle6)-Math.sin(angle6)*oldifs_y*yscale6+xshift6)+weight7*(oldifs_x*xscale7*Math.cos(angle7)-Math.sin(angle7)*oldifs_y*yscale7+xshift7)+weight8*(oldifs_x*xscale8*Math.cos(angle8)-Math.sin(angle8)*oldifs_y*yscale8+xshift8);

              // 'Set' Directive: set:ifs_y=weight1*(oldifs_x*xscale1*sin(angle1)+cos(angle1)*oldifs_y*yscale1+yshift1)+weight2*(oldifs_x*xscale2*sin(angle2)+cos(angle2)*oldifs_y*yscale2+yshift2)+weight3*(oldifs_x*xscale3*sin(angle3)+cos(angle3)*oldifs_y*yscale3+yshift3)+weight4*(oldifs_x*xscale4*sin(angle4)+cos(angle4)*oldifs_y*yscale4+yshift4)+weight5*(oldifs_x*xscale5*sin(angle5)+cos(angle5)*oldifs_y*yscale5+yshift5)+weight6*(oldifs_x*xscale6*sin(angle6)+cos(angle6)*oldifs_y*yscale6+yshift6)+weight7*(oldifs_x*xscale7*sin(angle7)+cos(angle7)*oldifs_y*yscale7+yshift7)+weight8*(oldifs_x*xscale8*sin(angle8)+cos(angle8)*oldifs_y*yscale8+yshift8)
              ifs_y=weight1*(oldifs_x*xscale1*Math.sin(angle1)+Math.cos(angle1)*oldifs_y*yscale1+yshift1)+weight2*(oldifs_x*xscale2*Math.sin(angle2)+Math.cos(angle2)*oldifs_y*yscale2+yshift2)+weight3*(oldifs_x*xscale3*Math.sin(angle3)+Math.cos(angle3)*oldifs_y*yscale3+yshift3)+weight4*(oldifs_x*xscale4*Math.sin(angle4)+Math.cos(angle4)*oldifs_y*yscale4+yshift4)+weight5*(oldifs_x*xscale5*Math.sin(angle5)+Math.cos(angle5)*oldifs_y*yscale5+yshift5)+weight6*(oldifs_x*xscale6*Math.sin(angle6)+Math.cos(angle6)*oldifs_y*yscale6+yshift6)+weight7*(oldifs_x*xscale7*Math.sin(angle7)+Math.cos(angle7)*oldifs_y*yscale7+yshift7)+weight8*(oldifs_x*xscale8*Math.sin(angle8)+Math.cos(angle8)*oldifs_y*yscale8+yshift8);

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int ifs_sierp(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 26;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            double big;
            double t;
            double angle1;
            double xscale1;
            double xshift1;
            double yscale1;
            double yshift1;
            double angle2;
            double xscale2;
            double xshift2;
            double yscale2;
            double yshift2;
            if (InheritParameters) {
                big=myMath.getVariable("big");
                t=myMath.getVariable("t");
                angle1=myMath.getVariable("angle1");
                xscale1=myMath.getVariable("xscale1");
                xshift1=myMath.getVariable("xshift1");
                yscale1=myMath.getVariable("yscale1");
                yshift1=myMath.getVariable("yshift1");
                angle2=myMath.getVariable("angle2");
                xscale2=myMath.getVariable("xscale2");
                xshift2=myMath.getVariable("xshift2");
                yscale2=myMath.getVariable("yscale2");
                yshift2=myMath.getVariable("yshift2");
            } else {
                big=10000;
                t=0;
                angle1=0;
                xscale1=0.5;
                xshift1=-.5;
                yscale1=0.5;
                yshift1=-.5;
                angle2=0;
                xscale2=0.5;
                xshift2=.5;
                yscale2=0.5;
                yshift2=-.5;
            }

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    double angle3=360*t;
                    double xscale3=0.5;
                    double xshift3=0;
                    double yscale3=0.5;
                    double yshift3=0.5;
                    double ifs_x=xval;
                    double ifs_y=yval;
                    double r=0;
                    int choice=0;
                    int weight1=0;
                    int weight2=0;
                    int weight3=0;
                    angle1=angle1*Math.PI/180;
                    angle2=angle2*Math.PI/180;
                    angle3=angle3*Math.PI/180;
                    double oldifs_x=0;
                    double oldifs_y=0;

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Set' Directive: set:r=sqrt(ifs_x*ifs_x+ifs_y*ifs_y)
                            r=Math.sqrt(ifs_x*ifs_x+ifs_y*ifs_y);

                            // 'ExitIf' Directive: exitif:r>big;palette:0
                            if (r>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Set' Directive: set:choice=RndInt(3)
                            choice=math.Misc.RndInt(3);

                            // 'Set' Directive: set:weight1=Delta(1,choice)
                            weight1=math.Misc.Delta(1,choice);

                            // 'Set' Directive: set:weight2=Delta(2,choice)
                            weight2=math.Misc.Delta(2,choice);

                            // 'Set' Directive: set:weight3=Delta(3,choice)
                            weight3=math.Misc.Delta(3,choice);

                            // 'Set' Directive: set:oldifs_x=ifs_x
                            oldifs_x=ifs_x;

                            // 'Set' Directive: set:oldifs_y=ifs_y
                            oldifs_y=ifs_y;

                            // 'Set' Directive: set:ifs_x=weight1*(oldifs_x*xscale1*cos(angle1)-sin(angle1)*oldifs_y*yscale1+xshift1)+weight2*(oldifs_x*xscale2*cos(angle2)-sin(angle2)*oldifs_y*yscale2+xshift2)+weight3*(oldifs_x*xscale3*cos(angle3)-sin(angle3)*oldifs_y*yscale3+xshift3)
                            ifs_x=weight1*(oldifs_x*xscale1*Math.cos(angle1)-Math.sin(angle1)*oldifs_y*yscale1+xshift1)+weight2*(oldifs_x*xscale2*Math.cos(angle2)-Math.sin(angle2)*oldifs_y*yscale2+xshift2)+weight3*(oldifs_x*xscale3*Math.cos(angle3)-Math.sin(angle3)*oldifs_y*yscale3+xshift3);

                            // 'Set' Directive: set:ifs_y=weight1*(oldifs_x*xscale1*sin(angle1)+cos(angle1)*oldifs_y*yscale1+yshift1)+weight2*(oldifs_x*xscale2*sin(angle2)+cos(angle2)*oldifs_y*yscale2+yshift2)+weight3*(oldifs_x*xscale3*sin(angle3)+cos(angle3)*oldifs_y*yscale3+yshift3)
                            ifs_y=weight1*(oldifs_x*xscale1*Math.sin(angle1)+Math.cos(angle1)*oldifs_y*yscale1+yshift1)+weight2*(oldifs_x*xscale2*Math.sin(angle2)+Math.cos(angle2)*oldifs_y*yscale2+yshift2)+weight3*(oldifs_x*xscale3*Math.sin(angle3)+Math.cos(angle3)*oldifs_y*yscale3+yshift3);

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData ifs_sierp_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 26;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double big;
      double t;
      double angle1;
      double xscale1;
      double xshift1;
      double yscale1;
      double yshift1;
      double angle2;
      double xscale2;
      double xshift2;
      double yscale2;
      double yshift2;
      if (InheritParameters) {
          big=myMath.getVariable("big");
          t=myMath.getVariable("t");
          angle1=myMath.getVariable("angle1");
          xscale1=myMath.getVariable("xscale1");
          xshift1=myMath.getVariable("xshift1");
          yscale1=myMath.getVariable("yscale1");
          yshift1=myMath.getVariable("yshift1");
          angle2=myMath.getVariable("angle2");
          xscale2=myMath.getVariable("xscale2");
          xshift2=myMath.getVariable("xshift2");
          yscale2=myMath.getVariable("yscale2");
          yshift2=myMath.getVariable("yshift2");
      } else {
          big=10000;
          t=0;
          angle1=0;
          xscale1=0.5;
          xshift1=-.5;
          yscale1=0.5;
          yshift1=-.5;
          angle2=0;
          xscale2=0.5;
          xshift2=.5;
          yscale2=0.5;
          yshift2=-.5;
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      double angle3=360*t;
      double xscale3=0.5;
      double xshift3=0;
      double yscale3=0.5;
      double yshift3=0.5;
      double ifs_x=xval;
      double ifs_y=yval;
      double r=0;
      int choice=0;
      int weight1=0;
      int weight2=0;
      int weight3=0;
      angle1=angle1*Math.PI/180;
      angle2=angle2*Math.PI/180;
      angle3=angle3*Math.PI/180;
      double oldifs_x=0;
      double oldifs_y=0;
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, ifs_x, ifs_y);
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'Set' Directive: set:r=sqrt(ifs_x*ifs_x+ifs_y*ifs_y)
              r=Math.sqrt(ifs_x*ifs_x+ifs_y*ifs_y);

              // 'ExitIf' Directive: exitif:r>big;palette:0
              if (r>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:choice=RndInt(3)
              choice=math.Misc.RndInt(3);

              // 'Set' Directive: set:weight1=Delta(1,choice)
              weight1=math.Misc.Delta(1,choice);

              // 'Set' Directive: set:weight2=Delta(2,choice)
              weight2=math.Misc.Delta(2,choice);

              // 'Set' Directive: set:weight3=Delta(3,choice)
              weight3=math.Misc.Delta(3,choice);

              // 'Set' Directive: set:oldifs_x=ifs_x
              oldifs_x=ifs_x;

              // 'Set' Directive: set:oldifs_y=ifs_y
              oldifs_y=ifs_y;

              // 'Set' Directive: set:ifs_x=weight1*(oldifs_x*xscale1*cos(angle1)-sin(angle1)*oldifs_y*yscale1+xshift1)+weight2*(oldifs_x*xscale2*cos(angle2)-sin(angle2)*oldifs_y*yscale2+xshift2)+weight3*(oldifs_x*xscale3*cos(angle3)-sin(angle3)*oldifs_y*yscale3+xshift3)
              ifs_x=weight1*(oldifs_x*xscale1*Math.cos(angle1)-Math.sin(angle1)*oldifs_y*yscale1+xshift1)+weight2*(oldifs_x*xscale2*Math.cos(angle2)-Math.sin(angle2)*oldifs_y*yscale2+xshift2)+weight3*(oldifs_x*xscale3*Math.cos(angle3)-Math.sin(angle3)*oldifs_y*yscale3+xshift3);

              // 'Set' Directive: set:ifs_y=weight1*(oldifs_x*xscale1*sin(angle1)+cos(angle1)*oldifs_y*yscale1+yshift1)+weight2*(oldifs_x*xscale2*sin(angle2)+cos(angle2)*oldifs_y*yscale2+yshift2)+weight3*(oldifs_x*xscale3*sin(angle3)+cos(angle3)*oldifs_y*yscale3+yshift3)
              ifs_y=weight1*(oldifs_x*xscale1*Math.sin(angle1)+Math.cos(angle1)*oldifs_y*yscale1+yshift1)+weight2*(oldifs_x*xscale2*Math.sin(angle2)+Math.cos(angle2)*oldifs_y*yscale2+yshift2)+weight3*(oldifs_x*xscale3*Math.sin(angle3)+Math.cos(angle3)*oldifs_y*yscale3+yshift3);

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int ifs_dragons(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 27;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            double big;
            double t;
            double angle1;
            double xscale1;
            double yscale1;
            double xpreshift1;
            double ypreshift1;
            double xpostshift1;
            double ypostshift1;
            if (InheritParameters) {
                big=myMath.getVariable("big");
                t=myMath.getVariable("t");
                angle1=myMath.getVariable("angle1");
                xscale1=myMath.getVariable("xscale1");
                yscale1=myMath.getVariable("yscale1");
                xpreshift1=myMath.getVariable("xpreshift1");
                ypreshift1=myMath.getVariable("ypreshift1");
                xpostshift1=myMath.getVariable("xpostshift1");
                ypostshift1=myMath.getVariable("ypostshift1");
            } else {
                big=10000;
                t=0;
                angle1=45;
                xscale1=0.7071068;
                yscale1=0.7071068;
                xpreshift1=0;
                ypreshift1=0;
                xpostshift1=0;
                ypostshift1=0;
            }

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    double angle2=180*t-45;
                    double xscale2=0.7071068;
                    double yscale2=0.7071068;
                    double xpreshift2=0.5*(1+t);
                    double ypreshift2=0.5*(1-t);
                    double xpostshift2=0;
                    double ypostshift2=0;
                    double ifs_x=xval;
                    double ifs_y=yval;
                    double r=0;
                    int choice=0;
                    int weight1=0;
                    int weight2=0;
                    angle1=angle1*Math.PI/180;
                    angle2=angle2*Math.PI/180;
                    double oldifs_x=0;
                    double oldifs_y=0;

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Set' Directive: set:r=sqrt(ifs_x*ifs_x+ifs_y*ifs_y)
                            r=Math.sqrt(ifs_x*ifs_x+ifs_y*ifs_y);

                            // 'ExitIf' Directive: exitif:r>big;palette:0
                            if (r>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Set' Directive: set:choice=RndInt(2)
                            choice=math.Misc.RndInt(2);

                            // 'Set' Directive: set:weight1=Delta(1,choice)
                            weight1=math.Misc.Delta(1,choice);

                            // 'Set' Directive: set:weight2=Delta(2,choice)
                            weight2=math.Misc.Delta(2,choice);

                            // 'Set' Directive: set:oldifs_x=ifs_x
                            oldifs_x=ifs_x;

                            // 'Set' Directive: set:oldifs_y=ifs_y
                            oldifs_y=ifs_y;

                            // 'Set' Directive: set:ifs_x=weight1*((oldifs_x*xscale1+xpreshift1)*cos(angle1)-sin(angle1)*(oldifs_y*yscale1+ypreshift1)+xpostshift1)+weight2*((oldifs_x*xscale2+xpreshift2)*cos(angle2)-sin(angle2)*(oldifs_y*yscale2+ypreshift2)+xpostshift2)
                            ifs_x=weight1*((oldifs_x*xscale1+xpreshift1)*Math.cos(angle1)-Math.sin(angle1)*(oldifs_y*yscale1+ypreshift1)+xpostshift1)+weight2*((oldifs_x*xscale2+xpreshift2)*Math.cos(angle2)-Math.sin(angle2)*(oldifs_y*yscale2+ypreshift2)+xpostshift2);

                            // 'Set' Directive: set:ifs_y=weight1*((oldifs_x*xscale1+xpreshift1)*sin(angle1)+cos(angle1)*(oldifs_y*yscale1+ypreshift1)+ypostshift1)+weight2*((oldifs_x*xscale2+xpreshift2)*sin(angle2)+cos(angle2)*(oldifs_y*yscale2+ypreshift2)+ypostshift2)
                            ifs_y=weight1*((oldifs_x*xscale1+xpreshift1)*Math.sin(angle1)+Math.cos(angle1)*(oldifs_y*yscale1+ypreshift1)+ypostshift1)+weight2*((oldifs_x*xscale2+xpreshift2)*Math.sin(angle2)+Math.cos(angle2)*(oldifs_y*yscale2+ypreshift2)+ypostshift2);

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData ifs_dragons_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 27;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double big;
      double t;
      double angle1;
      double xscale1;
      double yscale1;
      double xpreshift1;
      double ypreshift1;
      double xpostshift1;
      double ypostshift1;
      if (InheritParameters) {
          big=myMath.getVariable("big");
          t=myMath.getVariable("t");
          angle1=myMath.getVariable("angle1");
          xscale1=myMath.getVariable("xscale1");
          yscale1=myMath.getVariable("yscale1");
          xpreshift1=myMath.getVariable("xpreshift1");
          ypreshift1=myMath.getVariable("ypreshift1");
          xpostshift1=myMath.getVariable("xpostshift1");
          ypostshift1=myMath.getVariable("ypostshift1");
      } else {
          big=10000;
          t=0;
          angle1=45;
          xscale1=0.7071068;
          yscale1=0.7071068;
          xpreshift1=0;
          ypreshift1=0;
          xpostshift1=0;
          ypostshift1=0;
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      double angle2=180*t-45;
      double xscale2=0.7071068;
      double yscale2=0.7071068;
      double xpreshift2=0.5*(1+t);
      double ypreshift2=0.5*(1-t);
      double xpostshift2=0;
      double ypostshift2=0;
      double ifs_x=xval;
      double ifs_y=yval;
      double r=0;
      int choice=0;
      int weight1=0;
      int weight2=0;
      angle1=angle1*Math.PI/180;
      angle2=angle2*Math.PI/180;
      double oldifs_x=0;
      double oldifs_y=0;
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, ifs_x, ifs_y);
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'Set' Directive: set:r=sqrt(ifs_x*ifs_x+ifs_y*ifs_y)
              r=Math.sqrt(ifs_x*ifs_x+ifs_y*ifs_y);

              // 'ExitIf' Directive: exitif:r>big;palette:0
              if (r>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:choice=RndInt(2)
              choice=math.Misc.RndInt(2);

              // 'Set' Directive: set:weight1=Delta(1,choice)
              weight1=math.Misc.Delta(1,choice);

              // 'Set' Directive: set:weight2=Delta(2,choice)
              weight2=math.Misc.Delta(2,choice);

              // 'Set' Directive: set:oldifs_x=ifs_x
              oldifs_x=ifs_x;

              // 'Set' Directive: set:oldifs_y=ifs_y
              oldifs_y=ifs_y;

              // 'Set' Directive: set:ifs_x=weight1*((oldifs_x*xscale1+xpreshift1)*cos(angle1)-sin(angle1)*(oldifs_y*yscale1+ypreshift1)+xpostshift1)+weight2*((oldifs_x*xscale2+xpreshift2)*cos(angle2)-sin(angle2)*(oldifs_y*yscale2+ypreshift2)+xpostshift2)
              ifs_x=weight1*((oldifs_x*xscale1+xpreshift1)*Math.cos(angle1)-Math.sin(angle1)*(oldifs_y*yscale1+ypreshift1)+xpostshift1)+weight2*((oldifs_x*xscale2+xpreshift2)*Math.cos(angle2)-Math.sin(angle2)*(oldifs_y*yscale2+ypreshift2)+xpostshift2);

              // 'Set' Directive: set:ifs_y=weight1*((oldifs_x*xscale1+xpreshift1)*sin(angle1)+cos(angle1)*(oldifs_y*yscale1+ypreshift1)+ypostshift1)+weight2*((oldifs_x*xscale2+xpreshift2)*sin(angle2)+cos(angle2)*(oldifs_y*yscale2+ypreshift2)+ypostshift2)
              ifs_y=weight1*((oldifs_x*xscale1+xpreshift1)*Math.sin(angle1)+Math.cos(angle1)*(oldifs_y*yscale1+ypreshift1)+ypostshift1)+weight2*((oldifs_x*xscale2+xpreshift2)*Math.sin(angle2)+Math.cos(angle2)*(oldifs_y*yscale2+ypreshift2)+ypostshift2);

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int Henon_Real_K_Int(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 28;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            double a;
            double c;
            double big;
            double extra;
            if (InheritParameters) {
                a=myMath.getVariable("a");
                c=myMath.getVariable("c");
                big=myMath.getVariable("big");
                extra=myMath.getVariable("extra");
            } else {
                a=.3;
                c=-3;
                big=1;
                extra=100000;
            }

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    double h1=xval;
                    double h2=yval;
                    double i1=xval;
                    double i2=yval;
                    double old_h1=0;
                    double old_i2=0;
                    double sq_magh=0;
                    double sq_magi=0;

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Set' Directive: set:sq_magh=h1*h1+h2*h2-extra
                            sq_magh=h1*h1+h2*h2-extra;

                            // 'ExitIf' Directive: exitif:sq_magh>big;palette:0
                            if (sq_magh>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Set' Directive: set:old_h1=h1
                            old_h1=h1;

                            // 'Set' Directive: set:h1=h1*h1+c-a*h2
                            h1=h1*h1+c-a*h2;

                            // 'Set' Directive: set:h2=old_h1
                            h2=old_h1;

                            // 'Set' Directive: set:sq_magi=i1*i1+i2*i2-extra
                            sq_magi=i1*i1+i2*i2-extra;

                            // 'ExitIf' Directive: exitif:sq_magi>big;palette:0
                            if (sq_magi>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Set' Directive: set:old_i2=i2
                            old_i2=i2;

                            // 'Set' Directive: set:i2=(i2*i2+c-i1)/a
                            i2=(i2*i2+c-i1)/a;

                            // 'Set' Directive: set:i1=old_i2
                            i1=old_i2;

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData Henon_Real_K_Int_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 28;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double a;
      double c;
      double big;
      double extra;
      if (InheritParameters) {
          a=myMath.getVariable("a");
          c=myMath.getVariable("c");
          big=myMath.getVariable("big");
          extra=myMath.getVariable("extra");
      } else {
          a=.3;
          c=-3;
          big=1;
          extra=100000;
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      double h1=xval;
      double h2=yval;
      double i1=xval;
      double i2=yval;
      double old_h1=0;
      double old_i2=0;
      double sq_magh=0;
      double sq_magi=0;
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, h1, h2);
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'Set' Directive: set:sq_magh=h1*h1+h2*h2-extra
              sq_magh=h1*h1+h2*h2-extra;

              // 'ExitIf' Directive: exitif:sq_magh>big;palette:0
              if (sq_magh>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:old_h1=h1
              old_h1=h1;

              // 'Set' Directive: set:h1=h1*h1+c-a*h2
              h1=h1*h1+c-a*h2;

              // 'Set' Directive: set:h2=old_h1
              h2=old_h1;

              // 'Set' Directive: set:sq_magi=i1*i1+i2*i2-extra
              sq_magi=i1*i1+i2*i2-extra;

              // 'ExitIf' Directive: exitif:sq_magi>big;palette:0
              if (sq_magi>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:old_i2=i2
              old_i2=i2;

              // 'Set' Directive: set:i2=(i2*i2+c-i1)/a
              i2=(i2*i2+c-i1)/a;

              // 'Set' Directive: set:i1=old_i2
              i1=old_i2;

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int Henon_Real_K_Union(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 29;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            double a;
            double c;
            double big;
            double extra;
            if (InheritParameters) {
                a=myMath.getVariable("a");
                c=myMath.getVariable("c");
                big=myMath.getVariable("big");
                extra=myMath.getVariable("extra");
            } else {
                a=.3;
                c=-3;
                big=1;
                extra=100000;
            }

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    double h1=xval;
                    double h2=yval;
                    double i1=xval;
                    double i2=yval;
                    double old_h1=0;
                    double old_i2=0;
                    double sq_magh=0;
                    double sq_magi=0;

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Set' Directive: set:sq_magh=h1*h1+h2*h2-extra
                            sq_magh=h1*h1+h2*h2-extra;

                            // 'ExitIf' Directive: exitif:sq_magh>big;palette:0
                            if (sq_magh>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Set' Directive: set:old_h1=h1
                            old_h1=h1;

                            // 'Set' Directive: set:h1=h1*h1+c-a*h2
                            h1=h1*h1+c-a*h2;

                            // 'Set' Directive: set:h2=old_h1
                            h2=old_h1;

                        } // End of Block 0

                        Block1: // Start of Block 1
                        if (!BlockExit[1]) {

                            // 'Set' Directive: set:sq_magi=i1*i1+i2*i2-extra
                            sq_magi=i1*i1+i2*i2-extra;

                            // 'ExitIf' Directive: exitif:sq_magi>big;palette:1
                            if (sq_magi>big) {
                                DidExit++;
                                BlockExit[1] = true;
                                Palette[1]=1;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[1][x][y] = iter;
                                break Block1;
                            }

                            // 'Set' Directive: set:old_i2=i2
                            old_i2=i2;

                            // 'Set' Directive: set:i2=(i2*i2+c-i1)/a
                            i2=(i2*i2+c-i1)/a;

                            // 'Set' Directive: set:i1=old_i2
                            i1=old_i2;

                        } // End of Block 1

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData Henon_Real_K_Union_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 29;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double a;
      double c;
      double big;
      double extra;
      if (InheritParameters) {
          a=myMath.getVariable("a");
          c=myMath.getVariable("c");
          big=myMath.getVariable("big");
          extra=myMath.getVariable("extra");
      } else {
          a=.3;
          c=-3;
          big=1;
          extra=100000;
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      double h1=xval;
      double h2=yval;
      double i1=xval;
      double i2=yval;
      double old_h1=0;
      double old_i2=0;
      double sq_magh=0;
      double sq_magi=0;
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, h1, h2);
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'Set' Directive: set:sq_magh=h1*h1+h2*h2-extra
              sq_magh=h1*h1+h2*h2-extra;

              // 'ExitIf' Directive: exitif:sq_magh>big;palette:0
              if (sq_magh>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:old_h1=h1
              old_h1=h1;

              // 'Set' Directive: set:h1=h1*h1+c-a*h2
              h1=h1*h1+c-a*h2;

              // 'Set' Directive: set:h2=old_h1
              h2=old_h1;

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

  public TraceData Henon_Real_K_Union_1_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 29;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double a;
      double c;
      double big;
      double extra;
      if (InheritParameters) {
          a=myMath.getVariable("a");
          c=myMath.getVariable("c");
          big=myMath.getVariable("big");
          extra=myMath.getVariable("extra");
      } else {
          a=.3;
          c=-3;
          big=1;
          extra=100000;
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      double h1=xval;
      double h2=yval;
      double i1=xval;
      double i2=yval;
      double old_h1=0;
      double old_i2=0;
      double sq_magh=0;
      double sq_magi=0;
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, i1, i2);
          // User-defined directives are executed for each iteration in inner loop: 

          Block1: // Start of Block 1
          if (!BlockExit[1]) {

              // 'Set' Directive: set:sq_magi=i1*i1+i2*i2-extra
              sq_magi=i1*i1+i2*i2-extra;

              // 'ExitIf' Directive: exitif:sq_magi>big;palette:1
              if (sq_magi>big) {
                  DidExit++;
                  BlockExit[1] = true;
                  break Block1;
              }

              // 'Set' Directive: set:old_i2=i2
              old_i2=i2;

              // 'Set' Directive: set:i2=(i2*i2+c-i1)/a
              i2=(i2*i2+c-i1)/a;

              // 'Set' Directive: set:i1=old_i2
              i1=old_i2;

          } // End of Block 1
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int Henon_Real_Kminus(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 30;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            double a;
            double c;
            double big;
            if (InheritParameters) {
                a=myMath.getVariable("a");
                c=myMath.getVariable("c");
                big=myMath.getVariable("big");
            } else {
                a=.3;
                c=-3;
                big=100;
            }

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    double i1=xval;
                    double i2=yval;
                    double old_i2=0;
                    double sq_magi=0;

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Set' Directive: set:sq_magi=i1*i1+i2*i2
                            sq_magi=i1*i1+i2*i2;

                            // 'ExitIf' Directive: exitif:sq_magi>big;palette:0
                            if (sq_magi>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Set' Directive: set:old_i2=i2
                            old_i2=i2;

                            // 'Set' Directive: set:i2=(i2*i2+c-i1)/a
                            i2=(i2*i2+c-i1)/a;

                            // 'Set' Directive: set:i1=old_i2
                            i1=old_i2;

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData Henon_Real_Kminus_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 30;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double a;
      double c;
      double big;
      if (InheritParameters) {
          a=myMath.getVariable("a");
          c=myMath.getVariable("c");
          big=myMath.getVariable("big");
      } else {
          a=.3;
          c=-3;
          big=100;
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      double i1=xval;
      double i2=yval;
      double old_i2=0;
      double sq_magi=0;
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, i1, i2);
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'Set' Directive: set:sq_magi=i1*i1+i2*i2
              sq_magi=i1*i1+i2*i2;

              // 'ExitIf' Directive: exitif:sq_magi>big;palette:0
              if (sq_magi>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:old_i2=i2
              old_i2=i2;

              // 'Set' Directive: set:i2=(i2*i2+c-i1)/a
              i2=(i2*i2+c-i1)/a;

              // 'Set' Directive: set:i1=old_i2
              i1=old_i2;

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int Henon_Real_Kplus(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 31;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            double a;
            double c;
            double big;
            if (InheritParameters) {
                a=myMath.getVariable("a");
                c=myMath.getVariable("c");
                big=myMath.getVariable("big");
            } else {
                a=.3;
                c=-3;
                big=100;
            }

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    double h1=xval;
                    double h2=yval;
                    double old_h1=0;
                    double sq_magh=0;

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Set' Directive: set:sq_magh=h1*h1+h2*h2
                            sq_magh=h1*h1+h2*h2;

                            // 'ExitIf' Directive: exitif:sq_magh>big;palette:0
                            if (sq_magh>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Set' Directive: set:old_h1=h1
                            old_h1=h1;

                            // 'Set' Directive: set:h1=h1*h1+c-a*h2
                            h1=h1*h1+c-a*h2;

                            // 'Set' Directive: set:h2=old_h1
                            h2=old_h1;

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData Henon_Real_Kplus_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 31;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double a;
      double c;
      double big;
      if (InheritParameters) {
          a=myMath.getVariable("a");
          c=myMath.getVariable("c");
          big=myMath.getVariable("big");
      } else {
          a=.3;
          c=-3;
          big=100;
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      double h1=xval;
      double h2=yval;
      double old_h1=0;
      double sq_magh=0;
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, h1, h2);
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'Set' Directive: set:sq_magh=h1*h1+h2*h2
              sq_magh=h1*h1+h2*h2;

              // 'ExitIf' Directive: exitif:sq_magh>big;palette:0
              if (sq_magh>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:old_h1=h1
              old_h1=h1;

              // 'Set' Directive: set:h1=h1*h1+c-a*h2
              h1=h1*h1+c-a*h2;

              // 'Set' Directive: set:h2=old_h1
              h2=old_h1;

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int henon_unstable_manifold(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 32;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            double big;
            Complex C;
            Complex A;
            Complex zoom;
            if (InheritParameters) {
                big=myMath.getVariable("big");
                x_value=myMath.getVariable("C_x");
                y_value=myMath.getVariable("C_y");
                C=new Complex(x_value, y_value);
                x_value=myMath.getVariable("A_x");
                y_value=myMath.getVariable("A_y");
                A=new Complex(x_value, y_value);
                x_value=myMath.getVariable("zoom_x");
                y_value=myMath.getVariable("zoom_y");
                zoom=new Complex(x_value, y_value);
            } else {
                big=100;
                C=new Complex(.3,.1);
                A=new Complex(.3,0);
                zoom=new Complex(0.00001,0);
            }

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex A1=((A.Add(1)).Mult(0.5));
                    Complex fixedpoint=A1.Add(dummyC.Sqrt(dummyC.Sub(A1.Pow(2),C)));
                    Complex lambda=fixedpoint.Add(dummyC.Sqrt(dummyC.Sub(fixedpoint.Pow(2),A)));
                    Complex oldZ=new Complex(xval,yval);
                    oldZ=dummyC.Mult(oldZ,zoom);
                    Complex Z=(fixedpoint.Add(lambda.Mult(oldZ)));
                    Complex W=(fixedpoint.Add(oldZ));

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'ExitIf' Directive: exitif:SqNorm(Z)>big;palette:0;split:GetIm(Z);palette:1,2
                            if (dummyC.SqNorm(Z)>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                SplitResult[0]=dummyC.GetIm(Z);
                                break Block0;
                            }

                            // 'Set' Directive: set:oldZ=Z
                            oldZ=Z;

                            // 'Set' Directive: set:Z=((oldZ^2)+C)-(A*W)
                            Z=dummyC.Sub(((oldZ.Pow(2)).Add(C)),(A.Mult(W)));

                            // 'Set' Directive: set:W=oldZ
                            W=oldZ;

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData henon_unstable_manifold_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 32;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double big;
      Complex C;
      Complex A;
      Complex zoom;
      if (InheritParameters) {
          big=myMath.getVariable("big");
          x_value=myMath.getVariable("C_x");
          y_value=myMath.getVariable("C_y");
          C=new Complex(x_value, y_value);
          x_value=myMath.getVariable("A_x");
          y_value=myMath.getVariable("A_y");
          A=new Complex(x_value, y_value);
          x_value=myMath.getVariable("zoom_x");
          y_value=myMath.getVariable("zoom_y");
          zoom=new Complex(x_value, y_value);
      } else {
          big=100;
          C=new Complex(.3,.1);
          A=new Complex(.3,0);
          zoom=new Complex(0.00001,0);
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex A1=((A.Add(1)).Mult(0.5));
      Complex fixedpoint=A1.Add(dummyC.Sqrt(dummyC.Sub(A1.Pow(2),C)));
      Complex lambda=fixedpoint.Add(dummyC.Sqrt(dummyC.Sub(fixedpoint.Pow(2),A)));
      Complex oldZ=new Complex(xval,yval);
      oldZ=dummyC.Mult(oldZ,zoom);
      Complex Z=(fixedpoint.Add(lambda.Mult(oldZ)));
      Complex W=(fixedpoint.Add(oldZ));
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(Z), dummyC.GetIm(Z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'ExitIf' Directive: exitif:SqNorm(Z)>big;palette:0;split:GetIm(Z);palette:1,2
              if (dummyC.SqNorm(Z)>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:oldZ=Z
              oldZ=Z;

              // 'Set' Directive: set:Z=((oldZ^2)+C)-(A*W)
              Z=dummyC.Sub(((oldZ.Pow(2)).Add(C)),(A.Mult(W)));

              // 'Set' Directive: set:W=oldZ
              W=oldZ;

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int newton_sixth_julia(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 33;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;
            int basinindex;
            if (deTool.ImageArray.ParentFrameIndex[TargetFrameIndex] > -1) {
                basinindex = deTool.ImageArray.ParentFrameIndex[TargetFrameIndex];
            } else {
                basinindex = TargetFrameIndex;
            }
            int[] detectx0 = new int[MaxIter];
            int[] detecty0 = new int[MaxIter];
            double[] detectxval0 = new double[MaxIter];
            double[] detectyval0 = new double[MaxIter];

            // initialize parameters
            double tiny;
            double r3d2;
            Complex r1;
            Complex r2;
            Complex r3;
            Complex r4;
            Complex r5;
            Complex r6;
            if (InheritParameters) {
                tiny=myMath.getVariable("tiny");
                r3d2=myMath.getVariable("r3d2");
                x_value=myMath.getVariable("r1_x");
                y_value=myMath.getVariable("r1_y");
                r1=new Complex(x_value, y_value);
                x_value=myMath.getVariable("r2_x");
                y_value=myMath.getVariable("r2_y");
                r2=new Complex(x_value, y_value);
                x_value=myMath.getVariable("r3_x");
                y_value=myMath.getVariable("r3_y");
                r3=new Complex(x_value, y_value);
                x_value=myMath.getVariable("r4_x");
                y_value=myMath.getVariable("r4_y");
                r4=new Complex(x_value, y_value);
                x_value=myMath.getVariable("r5_x");
                y_value=myMath.getVariable("r5_y");
                r5=new Complex(x_value, y_value);
                x_value=myMath.getVariable("r6_x");
                y_value=myMath.getVariable("r6_y");
                r6=new Complex(x_value, y_value);
            } else {
                tiny=0.00001;
                r3d2=0.86602540378443864675;
                r1=new Complex(1,0);
                r2=new Complex(0.5,r3d2);
                r3=new Complex(.5,math.Misc.Neg(r3d2));
                r4=new Complex(-1,0);
                r5=new Complex(-.5,math.Misc.Neg(r3d2));
                r6=new Complex(-.5,r3d2);
            }

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex z=new Complex(xval, yval);
                    Complex L1=new Complex(0,0);
                    Complex L2=new Complex(0,0);
                    Complex L3=new Complex(0,0);
                    Complex L4=new Complex(0,0);
                    Complex L5=new Complex(0,0);
                    Complex L6=new Complex(0,0);
                    Complex L123=new Complex(0,0);
                    Complex L456=new Complex(0,0);

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Detect' Directive: detect:xval:GetRe(z);yval:GetIm(z)
                            if (DCD) {
                                detectxval0[iter] = dummyC.GetRe(z);
                                detectyval0[iter] = dummyC.GetIm(z);
                                detectx0[iter] = deTool.GetXFromVal(basinindex, detectxval0[iter]);
                                detecty0[iter] = deTool.GetYFromVal(basinindex, detectyval0[iter]);
                            }

                            // 'Set' Directive: set:L1=Sub(z,r1)
                            L1=dummyC.Sub(z,r1);

                            // 'ExitIf' Directive: exitif:SqNorm(L1)<tiny;palette:0
                            if (dummyC.SqNorm(L1)<tiny) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Set' Directive: set:L2=Sub(z,r2)
                            L2=dummyC.Sub(z,r2);

                            // 'ExitIf' Directive: exitif:SqNorm(L2)<tiny;palette:1
                            if (dummyC.SqNorm(L2)<tiny) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=1;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Set' Directive: set:L3=Sub(z,r3)
                            L3=dummyC.Sub(z,r3);

                            // 'ExitIf' Directive: exitif:SqNorm(L3)<tiny;palette:2
                            if (dummyC.SqNorm(L3)<tiny) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=2;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Set' Directive: set:L4=Sub(z,r4)
                            L4=dummyC.Sub(z,r4);

                            // 'ExitIf' Directive: exitif:SqNorm(L4)<tiny;palette:3
                            if (dummyC.SqNorm(L4)<tiny) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=3;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Set' Directive: set:L5=Sub(z,r5)
                            L5=dummyC.Sub(z,r5);

                            // 'ExitIf' Directive: exitif:SqNorm(L5)<tiny;palette:4
                            if (dummyC.SqNorm(L5)<tiny) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=4;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Set' Directive: set:L6=Sub(z,r6)
                            L6=dummyC.Sub(z,r6);

                            // 'ExitIf' Directive: exitif:SqNorm(L6)<tiny;palette:5
                            if (dummyC.SqNorm(L6)<tiny) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=5;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Set' Directive: set:L123=(L1*L2*L3)
                            L123=(L1.Mult(L2).Mult(L3));

                            // 'Set' Directive: set:L456=(L4*L5*L6)
                            L456=(L4.Mult(L5).Mult(L6));

                            // 'Set' Directive: set:z=z-((L123*L456)/((L2*L3*L456)+(L1*L2*L456)+(L1*L3*L456)+(L123*L4*L6)+(L123*L5*L6)+(L123*L4*L5)))
                            z=dummyC.Sub(z,((L123.Mult(L456)).Div((L2.Mult(L3).Mult(L456)).Add(L1.Mult(L2).Mult(L456)).Add(L1.Mult(L3).Mult(L456)).Add(L123.Mult(L4).Mult(L6)).Add(L123.Mult(L5).Mult(L6)).Add(L123.Mult(L4).Mult(L5)))));

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=0;
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=0;
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[1]=dummyC.GetRe(r1);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[1]=dummyC.GetIm(r1);
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[2]=dummyC.GetRe(r2);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[2]=dummyC.GetIm(r2);
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[3]=dummyC.GetRe(r3);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[3]=dummyC.GetIm(r3);
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[4]=dummyC.GetRe(r4);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[4]=dummyC.GetIm(r4);
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[5]=dummyC.GetRe(r5);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[5]=dummyC.GetIm(r5);
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[6]=dummyC.GetRe(r6);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[6]=dummyC.GetIm(r6);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData newton_sixth_julia_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 33;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double tiny;
      double r3d2;
      Complex r1;
      Complex r2;
      Complex r3;
      Complex r4;
      Complex r5;
      Complex r6;
      if (InheritParameters) {
          tiny=myMath.getVariable("tiny");
          r3d2=myMath.getVariable("r3d2");
          x_value=myMath.getVariable("r1_x");
          y_value=myMath.getVariable("r1_y");
          r1=new Complex(x_value, y_value);
          x_value=myMath.getVariable("r2_x");
          y_value=myMath.getVariable("r2_y");
          r2=new Complex(x_value, y_value);
          x_value=myMath.getVariable("r3_x");
          y_value=myMath.getVariable("r3_y");
          r3=new Complex(x_value, y_value);
          x_value=myMath.getVariable("r4_x");
          y_value=myMath.getVariable("r4_y");
          r4=new Complex(x_value, y_value);
          x_value=myMath.getVariable("r5_x");
          y_value=myMath.getVariable("r5_y");
          r5=new Complex(x_value, y_value);
          x_value=myMath.getVariable("r6_x");
          y_value=myMath.getVariable("r6_y");
          r6=new Complex(x_value, y_value);
      } else {
          tiny=0.00001;
          r3d2=0.86602540378443864675;
          r1=new Complex(1,0);
          r2=new Complex(0.5,r3d2);
          r3=new Complex(.5,math.Misc.Neg(r3d2));
          r4=new Complex(-1,0);
          r5=new Complex(-.5,math.Misc.Neg(r3d2));
          r6=new Complex(-.5,r3d2);
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex z=new Complex(xval, yval);
      Complex L1=new Complex(0,0);
      Complex L2=new Complex(0,0);
      Complex L3=new Complex(0,0);
      Complex L4=new Complex(0,0);
      Complex L5=new Complex(0,0);
      Complex L6=new Complex(0,0);
      Complex L123=new Complex(0,0);
      Complex L456=new Complex(0,0);
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(z), dummyC.GetIm(z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'Set' Directive: set:L1=Sub(z,r1)
              L1=dummyC.Sub(z,r1);

              // 'ExitIf' Directive: exitif:SqNorm(L1)<tiny;palette:0
              if (dummyC.SqNorm(L1)<tiny) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:L2=Sub(z,r2)
              L2=dummyC.Sub(z,r2);

              // 'ExitIf' Directive: exitif:SqNorm(L2)<tiny;palette:1
              if (dummyC.SqNorm(L2)<tiny) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:L3=Sub(z,r3)
              L3=dummyC.Sub(z,r3);

              // 'ExitIf' Directive: exitif:SqNorm(L3)<tiny;palette:2
              if (dummyC.SqNorm(L3)<tiny) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:L4=Sub(z,r4)
              L4=dummyC.Sub(z,r4);

              // 'ExitIf' Directive: exitif:SqNorm(L4)<tiny;palette:3
              if (dummyC.SqNorm(L4)<tiny) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:L5=Sub(z,r5)
              L5=dummyC.Sub(z,r5);

              // 'ExitIf' Directive: exitif:SqNorm(L5)<tiny;palette:4
              if (dummyC.SqNorm(L5)<tiny) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:L6=Sub(z,r6)
              L6=dummyC.Sub(z,r6);

              // 'ExitIf' Directive: exitif:SqNorm(L6)<tiny;palette:5
              if (dummyC.SqNorm(L6)<tiny) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:L123=(L1*L2*L3)
              L123=(L1.Mult(L2).Mult(L3));

              // 'Set' Directive: set:L456=(L4*L5*L6)
              L456=(L4.Mult(L5).Mult(L6));

              // 'Set' Directive: set:z=z-((L123*L456)/((L2*L3*L456)+(L1*L2*L456)+(L1*L3*L456)+(L123*L4*L6)+(L123*L5*L6)+(L123*L4*L5)))
              z=dummyC.Sub(z,((L123.Mult(L456)).Div((L2.Mult(L3).Mult(L456)).Add(L1.Mult(L2).Mult(L456)).Add(L1.Mult(L3).Mult(L456)).Add(L123.Mult(L4).Mult(L6)).Add(L123.Mult(L5).Mult(L6)).Add(L123.Mult(L4).Mult(L5)))));

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int newton_nth_julia(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 34;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;
            int basinindex;
            if (deTool.ImageArray.ParentFrameIndex[TargetFrameIndex] > -1) {
                basinindex = deTool.ImageArray.ParentFrameIndex[TargetFrameIndex];
            } else {
                basinindex = TargetFrameIndex;
            }
            int[] detectx0 = new int[MaxIter];
            int[] detecty0 = new int[MaxIter];
            double[] detectxval0 = new double[MaxIter];
            double[] detectyval0 = new double[MaxIter];

            // initialize parameters
            double tiny;
            int n;
            if (InheritParameters) {
                tiny=myMath.getVariable("tiny");
                n=(int) myMath.getVariable("n");
            } else {
                tiny=TINY;
                n=3;
            }

            // User-defined constants: 
            final Complex[] rn=new Complex[n];
            for(int rncounter = 0; rncounter < n; rncounter++) {
                rn[rncounter]=new Complex(Math.cos(2*Math.PI*rncounter/n), sin(2*Math.PI*rncounter/n));
            }

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex temptop=new Complex();
                    Complex tempbottom=new Complex();
                    Complex[] ln=new Complex[n];
                    Complex z=new Complex(xval, yval);

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Detect' Directive: detect:xval:GetRe(z);yval:GetIm(z);palette:12
                            if (DCD) {
                                detectxval0[iter] = dummyC.GetRe(z);
                                detectyval0[iter] = dummyC.GetIm(z);
                                detectx0[iter] = deTool.GetXFromVal(basinindex, detectxval0[iter]);
                                detecty0[iter] = deTool.GetYFromVal(basinindex, detectyval0[iter]);
                            }
                            for(int lncounter = 0; lncounter < n; lncounter++) {
                                ln[lncounter]=dummyC.Sub(z,rn[lncounter]);
                                if (!BlockExit[0]) {
                                    if (dummyC.SqNorm(ln[lncounter])<tiny) {
                                        DidExit++;
                                        BlockExit[0]=true;
                                        Palette[0]=(lncounter%12);
                                        deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                    }
                                }
                            }
                            temptop = ln[0];
                            if(n > 1) {
                                for(int lncounter = 1; lncounter < n; lncounter++) {
                                    temptop = dummyC.Mult(temptop,ln[lncounter]);
                                }
                            }
                            tempbottom = new Complex(0,0);
                            if(n > 1) {
                                for(int lncounter = 0; lncounter < n; lncounter++) {
                                Complex tempvalue;
                                    if (lncounter == 0) {
                                        tempvalue = ln[1];
                                        for(int innerloop = 2; innerloop < n; innerloop++) {
                                            tempvalue = dummyC.Mult(tempvalue,ln[innerloop]);
                                        }
                                    } else {
                                        tempvalue = ln[0];
                                        for(int innerloop = 1; innerloop < n; innerloop++) {
                                            if (innerloop != lncounter) {
                                                tempvalue = dummyC.Mult(tempvalue,ln[innerloop]);
                                            }
                                        }
                                    }
                                    tempbottom = dummyC.Add(tempbottom,tempvalue);
                                }
                            }

                            // 'Set' Directive: set:z=z-temptop/tempbottom
                            z=dummyC.Sub(z,temptop.Div(tempbottom));

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                            // cycle and basin detection code for block 0
                            if (!BlockExit[0]) {
                                switch (CDM) {
                                    case 0: { // pixel level precision
                                        if (deTool.DetectCB(TargetFrameIndex, 0, x, y, basinindex, BDM, detectx0, detecty0, iter)) {
                                            BlockExit[0] = true;
                                            Palette[0]=12;
                                        }
                                    }
                                    break;
                                    case 1: { // Double value level precision
                                        if (deTool.DetectCB(TargetFrameIndex, 0, x, y, basinindex, BDM, detectxval0, detectyval0, iter)) {
                                            BlockExit[0] = true;
                                            Palette[0]=12;
                                        }
                                    }
                                    break;
                                }
                            }
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData newton_nth_julia_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 34;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double tiny;
      int n;
      if (InheritParameters) {
          tiny=myMath.getVariable("tiny");
          n=(int) myMath.getVariable("n");
      } else {
          tiny=TINY;
          n=3;
      }

    // User-defined constants: 
    final Complex[] rn=new Complex[n];
    for(int rncounter = 0; rncounter < n; rncounter++) {
        rn[rncounter]=new Complex(Math.cos(2*Math.PI*rncounter/n), sin(2*Math.PI*rncounter/n));
    }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex temptop=new Complex();
      Complex tempbottom=new Complex();
      Complex[] ln=new Complex[n];
      Complex z=new Complex(xval, yval);
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(z), dummyC.GetIm(z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {
                            for(int lncounter = 0; lncounter < n; lncounter++) {
                                ln[lncounter]=dummyC.Sub(z,rn[lncounter]);
                                if (!BlockExit[0]) {
                                    if (dummyC.SqNorm(ln[lncounter])<tiny) {
                                        DidExit++;
                                        BlockExit[0]=true;
                                        Palette[0]=(lncounter%12);
                                        deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                    }
                                }
                            }
                            temptop = ln[0];
                            if(n > 1) {
                                for(int lncounter = 1; lncounter < n; lncounter++) {
                                    temptop = dummyC.Mult(temptop,ln[lncounter]);
                                }
                            }
                            tempbottom = new Complex(0,0);
                            if(n > 1) {
                                for(int lncounter = 0; lncounter < n; lncounter++) {
                                Complex tempvalue;
                                    if (lncounter == 0) {
                                        tempvalue = ln[1];
                                        for(int innerloop = 2; innerloop < n; innerloop++) {
                                            tempvalue = dummyC.Mult(tempvalue,ln[innerloop]);
                                        }
                                    } else {
                                        tempvalue = ln[0];
                                        for(int innerloop = 1; innerloop < n; innerloop++) {
                                            if (innerloop != lncounter) {
                                                tempvalue = dummyC.Mult(tempvalue,ln[innerloop]);
                                            }
                                        }
                                    }
                                    tempbottom = dummyC.Add(tempbottom,tempvalue);
                                }
                            }

              // 'Set' Directive: set:z=z-temptop/tempbottom
              z=dummyC.Sub(z,temptop.Div(tempbottom));

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int newton_nth_basin_detect_julia(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 35;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;
            int basinindex;
            if (deTool.ImageArray.ParentFrameIndex[TargetFrameIndex] > -1) {
                basinindex = deTool.ImageArray.ParentFrameIndex[TargetFrameIndex];
            } else {
                basinindex = TargetFrameIndex;
            }
            int[] detectx0 = new int[MaxIter];
            int[] detecty0 = new int[MaxIter];
            double[] detectxval0 = new double[MaxIter];
            double[] detectyval0 = new double[MaxIter];

            // initialize parameters
            double tiny;
            int n;
            if (InheritParameters) {
                tiny=myMath.getVariable("tiny");
                n=(int) myMath.getVariable("n");
            } else {
                tiny=TINY;
                n=3;
            }

            // User-defined constants: 
            final Complex[] rn=new Complex[n];
            for(int rncounter = 0; rncounter < n; rncounter++) {
                rn[rncounter]=new Complex(Math.cos(2*Math.PI*rncounter/n), sin(2*Math.PI*rncounter/n));
            }

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex[] ln=new Complex[n];
                    Complex z=new Complex(xval, yval);
                    Complex temptop=new Complex(0,0);
                    Complex tempbottom=new Complex(0,0);

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Detect' Directive: detect:xval:GetRe(z);yval:GetIm(z);palette:12
                            if (DCD) {
                                detectxval0[iter] = dummyC.GetRe(z);
                                detectyval0[iter] = dummyC.GetIm(z);
                                detectx0[iter] = deTool.GetXFromVal(basinindex, detectxval0[iter]);
                                detecty0[iter] = deTool.GetYFromVal(basinindex, detectyval0[iter]);
                            }
                            for(int lncounter = 0; lncounter < n; lncounter++) {
                                ln[lncounter]=dummyC.Sub(z,rn[lncounter]);
                            }
                            temptop = ln[0];
                            if(n > 1) {
                                for(int lncounter = 1; lncounter < n; lncounter++) {
                                    temptop = dummyC.Mult(temptop,ln[lncounter]);
                                }
                            }
                            tempbottom = new Complex(0,0);
                            if(n > 1) {
                                for(int lncounter = 0; lncounter < n; lncounter++) {
                                Complex tempvalue;
                                    if (lncounter == 0) {
                                        tempvalue = ln[1];
                                        for(int innerloop = 2; innerloop < n; innerloop++) {
                                            tempvalue = dummyC.Mult(tempvalue,ln[innerloop]);
                                        }
                                    } else {
                                        tempvalue = ln[0];
                                        for(int innerloop = 1; innerloop < n; innerloop++) {
                                            if (innerloop != lncounter) {
                                                tempvalue = dummyC.Mult(tempvalue,ln[innerloop]);
                                            }
                                        }
                                    }
                                    tempbottom = dummyC.Add(tempbottom,tempvalue);
                                }
                            }

                            // 'Set' Directive: set:z=z-temptop/tempbottom
                            z=dummyC.Sub(z,temptop.Div(tempbottom));

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                            // cycle and basin detection code for block 0
                            if (!BlockExit[0]) {
                                switch (CDM) {
                                    case 0: { // pixel level precision
                                        if (deTool.DetectCB(TargetFrameIndex, 0, x, y, basinindex, BDM, detectx0, detecty0, iter)) {
                                            BlockExit[0] = true;
                                            Palette[0]=12;
                                        }
                                    }
                                    break;
                                    case 1: { // Double value level precision
                                        if (deTool.DetectCB(TargetFrameIndex, 0, x, y, basinindex, BDM, detectxval0, detectyval0, iter)) {
                                            BlockExit[0] = true;
                                            Palette[0]=12;
                                        }
                                    }
                                    break;
                                }
                            }
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData newton_nth_basin_detect_julia_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 35;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double tiny;
      int n;
      if (InheritParameters) {
          tiny=myMath.getVariable("tiny");
          n=(int) myMath.getVariable("n");
      } else {
          tiny=TINY;
          n=3;
      }

    // User-defined constants: 
    final Complex[] rn=new Complex[n];
    for(int rncounter = 0; rncounter < n; rncounter++) {
        rn[rncounter]=new Complex(Math.cos(2*Math.PI*rncounter/n), sin(2*Math.PI*rncounter/n));
    }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex[] ln=new Complex[n];
      Complex z=new Complex(xval, yval);
      Complex temptop=new Complex(0,0);
      Complex tempbottom=new Complex(0,0);
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(z), dummyC.GetIm(z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {
                            for(int lncounter = 0; lncounter < n; lncounter++) {
                                ln[lncounter]=dummyC.Sub(z,rn[lncounter]);
                            }
                            temptop = ln[0];
                            if(n > 1) {
                                for(int lncounter = 1; lncounter < n; lncounter++) {
                                    temptop = dummyC.Mult(temptop,ln[lncounter]);
                                }
                            }
                            tempbottom = new Complex(0,0);
                            if(n > 1) {
                                for(int lncounter = 0; lncounter < n; lncounter++) {
                                Complex tempvalue;
                                    if (lncounter == 0) {
                                        tempvalue = ln[1];
                                        for(int innerloop = 2; innerloop < n; innerloop++) {
                                            tempvalue = dummyC.Mult(tempvalue,ln[innerloop]);
                                        }
                                    } else {
                                        tempvalue = ln[0];
                                        for(int innerloop = 1; innerloop < n; innerloop++) {
                                            if (innerloop != lncounter) {
                                                tempvalue = dummyC.Mult(tempvalue,ln[innerloop]);
                                            }
                                        }
                                    }
                                    tempbottom = dummyC.Add(tempbottom,tempvalue);
                                }
                            }

              // 'Set' Directive: set:z=z-temptop/tempbottom
              z=dummyC.Sub(z,temptop.Div(tempbottom));

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int QuadRatZeroReplFPBoth(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 36;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;
            int basinindex;
            if (deTool.ImageArray.ParentFrameIndex[TargetFrameIndex] > -1) {
                basinindex = deTool.ImageArray.ParentFrameIndex[TargetFrameIndex];
            } else {
                basinindex = TargetFrameIndex;
            }
            int[] detectx0 = new int[MaxIter];
            int[] detecty0 = new int[MaxIter];
            double[] detectxval0 = new double[MaxIter];
            double[] detectyval0 = new double[MaxIter];
            int[] detectx1 = new int[MaxIter];
            int[] detecty1 = new int[MaxIter];
            double[] detectxval1 = new double[MaxIter];
            double[] detectyval1 = new double[MaxIter];

            // initialize parameters
            double tiny;
            double big;
            Complex A;
            if (InheritParameters) {
                tiny=myMath.getVariable("tiny");
                big=myMath.getVariable("big");
                x_value=myMath.getVariable("A_x");
                y_value=myMath.getVariable("A_y");
                A=new Complex(x_value, y_value);
            } else {
                tiny=0.000001;
                big=100000;
                A=new Complex(0,0);
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex B=new Complex(xval,yval);
                    Complex z=new Complex(-1,0);
                    Complex w=new Complex(1,0);
                    Complex v=dummyC.Sqrt(dummyC.Sub((B.Mult(B)),4.0));
                    Complex PreInf1=((B.Add(v)).Mult(-0.5));
                    Complex PreInf2=((dummyC.Sub(B,v)).Mult(-0.5));
                    double r=0;
                    double s=0;

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Set' Directive: set:r=SqNorm(z)
                            r=dummyC.SqNorm(z);

                            // 'ExitIf' Directive: exitif:r>big;palette:0
                            if (r>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'ExitIf' Directive: exitif:SqNorm(z-PreInf1)<tiny;palette:0
                            if (dummyC.SqNorm(dummyC.Sub(z,PreInf1))<tiny) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'ExitIf' Directive: exitif:SqNorm(z-PreInf2)<tiny;palette:0
                            if (dummyC.SqNorm(dummyC.Sub(z,PreInf2))<tiny) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Detect' Directive: detect:xval:GetRe(z);yval:GetIm(z);palette:2
                            if (DCD) {
                                detectxval0[iter] = dummyC.GetRe(z);
                                detectyval0[iter] = dummyC.GetIm(z);
                                detectx0[iter] = deTool.GetXFromVal(basinindex, detectxval0[iter]);
                                detecty0[iter] = deTool.GetYFromVal(basinindex, detectyval0[iter]);
                            }

                            // 'Set' Directive: set:z=(A*z)/(1.0+((B*z)+(z*z)))
                            z=(A.Mult(z)).Div(((B.Mult(z)).Add(z.Mult(z))).Add(1.0));

                        } // End of Block 0

                        Block1: // Start of Block 1
                        if (!BlockExit[1]) {

                            // 'Set' Directive: set:s=SqNorm(w)
                            s=dummyC.SqNorm(w);

                            // 'ExitIf' Directive: exitif:s>big;palette:1
                            if (s>big) {
                                DidExit++;
                                BlockExit[1] = true;
                                Palette[1]=1;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[1][x][y] = iter;
                                break Block1;
                            }

                            // 'ExitIf' Directive: exitif:SqNorm(w-PreInf1)<tiny;palette:1
                            if (dummyC.SqNorm(dummyC.Sub(w,PreInf1))<tiny) {
                                DidExit++;
                                BlockExit[1] = true;
                                Palette[1]=1;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[1][x][y] = iter;
                                break Block1;
                            }

                            // 'ExitIf' Directive: exitif:SqNorm(w-PreInf2)<tiny;palette:1
                            if (dummyC.SqNorm(dummyC.Sub(w,PreInf2))<tiny) {
                                DidExit++;
                                BlockExit[1] = true;
                                Palette[1]=1;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[1][x][y] = iter;
                                break Block1;
                            }

                            // 'Detect' Directive: detect:xval:GetRe(w);yval:GetIm(w);palette:3
                            if (DCD) {
                                detectxval1[iter] = dummyC.GetRe(w);
                                detectyval1[iter] = dummyC.GetIm(w);
                                detectx1[iter] = deTool.GetXFromVal(basinindex, detectxval1[iter]);
                                detecty1[iter] = deTool.GetYFromVal(basinindex, detectyval1[iter]);
                            }

                            // 'Set' Directive: set:w=(A*w)/(1.0+((B*w)+(w*w)))
                            w=(A.Mult(w)).Div(((B.Mult(w)).Add(w.Mult(w))).Add(1.0));

                        } // End of Block 1

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                            // cycle and basin detection code for block 0
                            if (!BlockExit[0]) {
                                switch (CDM) {
                                    case 0: { // pixel level precision
                                        if (deTool.DetectCB(TargetFrameIndex, 0, x, y, basinindex, BDM, detectx0, detecty0, iter)) {
                                            BlockExit[0] = true;
                                            Palette[0]=2;
                                        }
                                    }
                                    break;
                                    case 1: { // Double value level precision
                                        if (deTool.DetectCB(TargetFrameIndex, 0, x, y, basinindex, BDM, detectxval0, detectyval0, iter)) {
                                            BlockExit[0] = true;
                                            Palette[0]=2;
                                        }
                                    }
                                    break;
                                }
                            }
                            // cycle and basin detection code for block 1
                            if (!BlockExit[1]) {
                                switch (CDM) {
                                    case 0: { // pixel level precision
                                        if (deTool.DetectCB(TargetFrameIndex, 1, x, y, basinindex, BDM, detectx1, detecty1, iter)) {
                                            BlockExit[1] = true;
                                            Palette[1]=3;
                                        }
                                    }
                                    break;
                                    case 1: { // Double value level precision
                                        if (deTool.DetectCB(TargetFrameIndex, 1, x, y, basinindex, BDM, detectxval1, detectyval1, iter)) {
                                            BlockExit[1] = true;
                                            Palette[1]=3;
                                        }
                                    }
                                    break;
                                }
                            }
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData QuadRatZeroReplFPBoth_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 36;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double tiny;
      double big;
      Complex A;
      if (InheritParameters) {
          tiny=myMath.getVariable("tiny");
          big=myMath.getVariable("big");
          x_value=myMath.getVariable("A_x");
          y_value=myMath.getVariable("A_y");
          A=new Complex(x_value, y_value);
      } else {
          tiny=0.000001;
          big=100000;
          A=new Complex(0,0);
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex B=new Complex(xval,yval);
      Complex z=new Complex(-1,0);
      Complex w=new Complex(1,0);
      Complex v=dummyC.Sqrt(dummyC.Sub((B.Mult(B)),4.0));
      Complex PreInf1=((B.Add(v)).Mult(-0.5));
      Complex PreInf2=((dummyC.Sub(B,v)).Mult(-0.5));
      double r=0;
      double s=0;
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(z), dummyC.GetIm(z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'Set' Directive: set:r=SqNorm(z)
              r=dummyC.SqNorm(z);

              // 'ExitIf' Directive: exitif:r>big;palette:0
              if (r>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'ExitIf' Directive: exitif:SqNorm(z-PreInf1)<tiny;palette:0
              if (dummyC.SqNorm(dummyC.Sub(z,PreInf1))<tiny) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'ExitIf' Directive: exitif:SqNorm(z-PreInf2)<tiny;palette:0
              if (dummyC.SqNorm(dummyC.Sub(z,PreInf2))<tiny) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:z=(A*z)/(1.0+((B*z)+(z*z)))
              z=(A.Mult(z)).Div(((B.Mult(z)).Add(z.Mult(z))).Add(1.0));

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

  public TraceData QuadRatZeroReplFPBoth_1_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 36;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double tiny;
      double big;
      Complex A;
      if (InheritParameters) {
          tiny=myMath.getVariable("tiny");
          big=myMath.getVariable("big");
          x_value=myMath.getVariable("A_x");
          y_value=myMath.getVariable("A_y");
          A=new Complex(x_value, y_value);
      } else {
          tiny=0.000001;
          big=100000;
          A=new Complex(0,0);
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex B=new Complex(xval,yval);
      Complex z=new Complex(-1,0);
      Complex w=new Complex(1,0);
      Complex v=dummyC.Sqrt(dummyC.Sub((B.Mult(B)),4.0));
      Complex PreInf1=((B.Add(v)).Mult(-0.5));
      Complex PreInf2=((dummyC.Sub(B,v)).Mult(-0.5));
      double r=0;
      double s=0;
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(w), dummyC.GetIm(w));
          // User-defined directives are executed for each iteration in inner loop: 

          Block1: // Start of Block 1
          if (!BlockExit[1]) {

              // 'Set' Directive: set:s=SqNorm(w)
              s=dummyC.SqNorm(w);

              // 'ExitIf' Directive: exitif:s>big;palette:1
              if (s>big) {
                  DidExit++;
                  BlockExit[1] = true;
                  break Block1;
              }

              // 'ExitIf' Directive: exitif:SqNorm(w-PreInf1)<tiny;palette:1
              if (dummyC.SqNorm(dummyC.Sub(w,PreInf1))<tiny) {
                  DidExit++;
                  BlockExit[1] = true;
                  break Block1;
              }

              // 'ExitIf' Directive: exitif:SqNorm(w-PreInf2)<tiny;palette:1
              if (dummyC.SqNorm(dummyC.Sub(w,PreInf2))<tiny) {
                  DidExit++;
                  BlockExit[1] = true;
                  break Block1;
              }

              // 'Set' Directive: set:w=(A*w)/(1.0+((B*w)+(w*w)))
              w=(A.Mult(w)).Div(((B.Mult(w)).Add(w.Mult(w))).Add(1.0));

          } // End of Block 1
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int QuadRatZeroReplFP_Julia(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 37;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;
            int basinindex;
            if (deTool.ImageArray.ParentFrameIndex[TargetFrameIndex] > -1) {
                basinindex = deTool.ImageArray.ParentFrameIndex[TargetFrameIndex];
            } else {
                basinindex = TargetFrameIndex;
            }
            int[] detectx0 = new int[MaxIter];
            int[] detecty0 = new int[MaxIter];
            double[] detectxval0 = new double[MaxIter];
            double[] detectyval0 = new double[MaxIter];

            // initialize parameters
            double tiny;
            double big;
            Complex A;
            Complex B;
            if (InheritParameters) {
                tiny=myMath.getVariable("tiny");
                big=myMath.getVariable("big");
                x_value=myMath.getVariable("A_x");
                y_value=myMath.getVariable("A_y");
                A=new Complex(x_value, y_value);
                x_value=myMath.getVariable("B_x");
                y_value=myMath.getVariable("B_y");
                B=new Complex(x_value, y_value);
            } else {
                tiny=0.000001;
                big=100000;
                A=new Complex(0,0);
                B=new Complex(cX,cY);
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);
            Complex p1=new Complex(1,0);
            Complex p2=new Complex(-1,0);

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex z=new Complex(xval,yval);
                    Complex v=dummyC.Sqrt(dummyC.Sub((B.Mult(B)),4.0));
                    Complex PreInf1=((B.Add(v)).Mult(-0.5));
                    Complex PreInf2=((dummyC.Sub(B,v)).Mult(-0.5));
                    double r=0;

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Set' Directive: set:r=SqNorm(z)
                            r=dummyC.SqNorm(z);

                            // 'ExitIf' Directive: exitif:r>big;palette:0
                            if (r>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'ExitIf' Directive: exitif:SqNorm(z-PreInf1)<tiny;palette:0
                            if (dummyC.SqNorm(dummyC.Sub(z,PreInf1))<tiny) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'ExitIf' Directive: exitif:SqNorm(z-PreInf2)<tiny;palette:0
                            if (dummyC.SqNorm(dummyC.Sub(z,PreInf2))<tiny) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Detect' Directive: detect:xval:GetRe(z);yval:GetIm(z);palette:1
                            if (DCD) {
                                detectxval0[iter] = dummyC.GetRe(z);
                                detectyval0[iter] = dummyC.GetIm(z);
                                detectx0[iter] = deTool.GetXFromVal(basinindex, detectxval0[iter]);
                                detecty0[iter] = deTool.GetYFromVal(basinindex, detectyval0[iter]);
                            }

                            // 'Set' Directive: set:z=(A*z)/(1.0+((B*z)+(z*z)))
                            z=(A.Mult(z)).Div(((B.Mult(z)).Add(z.Mult(z))).Add(1.0));

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                            // cycle and basin detection code for block 0
                            if (!BlockExit[0]) {
                                switch (CDM) {
                                    case 0: { // pixel level precision
                                        if (deTool.DetectCB(TargetFrameIndex, 0, x, y, basinindex, BDM, detectx0, detecty0, iter)) {
                                            BlockExit[0] = true;
                                            Palette[0]=1;
                                        }
                                    }
                                    break;
                                    case 1: { // Double value level precision
                                        if (deTool.DetectCB(TargetFrameIndex, 0, x, y, basinindex, BDM, detectxval0, detectyval0, iter)) {
                                            BlockExit[0] = true;
                                            Palette[0]=1;
                                        }
                                    }
                                    break;
                                }
                            }
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[1]=dummyC.GetRe(p1);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[1]=dummyC.GetIm(p1);
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[2]=dummyC.GetRe(p2);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[2]=dummyC.GetIm(p2);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData QuadRatZeroReplFP_Julia_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 37;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double tiny;
      double big;
      Complex A;
      Complex B;
      if (InheritParameters) {
          tiny=myMath.getVariable("tiny");
          big=myMath.getVariable("big");
          x_value=myMath.getVariable("A_x");
          y_value=myMath.getVariable("A_y");
          A=new Complex(x_value, y_value);
          x_value=myMath.getVariable("B_x");
          y_value=myMath.getVariable("B_y");
          B=new Complex(x_value, y_value);
      } else {
          tiny=0.000001;
          big=100000;
          A=new Complex(0,0);
          B=new Complex(cX,cY);
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex z=new Complex(xval,yval);
      Complex v=dummyC.Sqrt(dummyC.Sub((B.Mult(B)),4.0));
      Complex PreInf1=((B.Add(v)).Mult(-0.5));
      Complex PreInf2=((dummyC.Sub(B,v)).Mult(-0.5));
      double r=0;
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(z), dummyC.GetIm(z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'Set' Directive: set:r=SqNorm(z)
              r=dummyC.SqNorm(z);

              // 'ExitIf' Directive: exitif:r>big;palette:0
              if (r>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'ExitIf' Directive: exitif:SqNorm(z-PreInf1)<tiny;palette:0
              if (dummyC.SqNorm(dummyC.Sub(z,PreInf1))<tiny) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'ExitIf' Directive: exitif:SqNorm(z-PreInf2)<tiny;palette:0
              if (dummyC.SqNorm(dummyC.Sub(z,PreInf2))<tiny) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:z=(A*z)/(1.0+((B*z)+(z*z)))
              z=(A.Mult(z)).Div(((B.Mult(z)).Add(z.Mult(z))).Add(1.0));

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int QuadRatZeroReplFP_Param(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 38;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            double tiny;
            double big;
            if (InheritParameters) {
                tiny=myMath.getVariable("tiny");
                big=myMath.getVariable("big");
            } else {
                tiny=0.000001;
                big=100000;
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex A=new Complex(xval,yval);

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'ExitIf' Directive: exitif:SqNorm(A)>1;palette:1
                            if (dummyC.SqNorm(A)>1) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=1;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'ExitIf' Directive: exitif:1>0;palette:0
                            if (1>0) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

    public int QuadRatZeroReplFPMinusOne(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 39;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;
            int basinindex;
            if (deTool.ImageArray.ParentFrameIndex[TargetFrameIndex] > -1) {
                basinindex = deTool.ImageArray.ParentFrameIndex[TargetFrameIndex];
            } else {
                basinindex = TargetFrameIndex;
            }
            int[] detectx0 = new int[MaxIter];
            int[] detecty0 = new int[MaxIter];
            double[] detectxval0 = new double[MaxIter];
            double[] detectyval0 = new double[MaxIter];

            // initialize parameters
            double tiny;
            double big;
            Complex A;
            if (InheritParameters) {
                tiny=myMath.getVariable("tiny");
                big=myMath.getVariable("big");
                x_value=myMath.getVariable("A_x");
                y_value=myMath.getVariable("A_y");
                A=new Complex(x_value, y_value);
            } else {
                tiny=0.000001;
                big=100000;
                A=new Complex(cX,cY);
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex B=new Complex(xval,yval);
                    Complex z=new Complex(-1,0);
                    Complex v=dummyC.Sqrt(dummyC.Sub((B.Mult(B)),4.0));
                    Complex PreInf1=((B.Add(v)).Mult(-0.5));
                    Complex PreInf2=((dummyC.Sub(B,v)).Mult(-0.5));
                    double s=0;

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Set' Directive: set:s=SqNorm(z)
                            s=dummyC.SqNorm(z);

                            // 'ExitIf' Directive: exitif:s>big;palette:1
                            if (s>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=1;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'ExitIf' Directive: exitif:SqNorm(z-PreInf1)<tiny;palette:1
                            if (dummyC.SqNorm(dummyC.Sub(z,PreInf1))<tiny) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=1;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'ExitIf' Directive: exitif:SqNorm(z-PreInf2)<tiny;palette:1
                            if (dummyC.SqNorm(dummyC.Sub(z,PreInf2))<tiny) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=1;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Detect' Directive: detect:xval:GetRe(z);yval:GetIm(z);palette:3
                            if (DCD) {
                                detectxval0[iter] = dummyC.GetRe(z);
                                detectyval0[iter] = dummyC.GetIm(z);
                                detectx0[iter] = deTool.GetXFromVal(basinindex, detectxval0[iter]);
                                detecty0[iter] = deTool.GetYFromVal(basinindex, detectyval0[iter]);
                            }

                            // 'Set' Directive: set:z=((A*z)/(1.0+((B*z)+(z*z))))
                            z=((A.Mult(z)).Div(((B.Mult(z)).Add(z.Mult(z))).Add(1.0)));

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                            // cycle and basin detection code for block 0
                            if (!BlockExit[0]) {
                                switch (CDM) {
                                    case 0: { // pixel level precision
                                        if (deTool.DetectCB(TargetFrameIndex, 0, x, y, basinindex, BDM, detectx0, detecty0, iter)) {
                                            BlockExit[0] = true;
                                            Palette[0]=3;
                                        }
                                    }
                                    break;
                                    case 1: { // Double value level precision
                                        if (deTool.DetectCB(TargetFrameIndex, 0, x, y, basinindex, BDM, detectxval0, detectyval0, iter)) {
                                            BlockExit[0] = true;
                                            Palette[0]=3;
                                        }
                                    }
                                    break;
                                }
                            }
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData QuadRatZeroReplFPMinusOne_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 39;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double tiny;
      double big;
      Complex A;
      if (InheritParameters) {
          tiny=myMath.getVariable("tiny");
          big=myMath.getVariable("big");
          x_value=myMath.getVariable("A_x");
          y_value=myMath.getVariable("A_y");
          A=new Complex(x_value, y_value);
      } else {
          tiny=0.000001;
          big=100000;
          A=new Complex(cX,cY);
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex B=new Complex(xval,yval);
      Complex z=new Complex(-1,0);
      Complex v=dummyC.Sqrt(dummyC.Sub((B.Mult(B)),4.0));
      Complex PreInf1=((B.Add(v)).Mult(-0.5));
      Complex PreInf2=((dummyC.Sub(B,v)).Mult(-0.5));
      double s=0;
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(z), dummyC.GetIm(z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'Set' Directive: set:s=SqNorm(z)
              s=dummyC.SqNorm(z);

              // 'ExitIf' Directive: exitif:s>big;palette:1
              if (s>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'ExitIf' Directive: exitif:SqNorm(z-PreInf1)<tiny;palette:1
              if (dummyC.SqNorm(dummyC.Sub(z,PreInf1))<tiny) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'ExitIf' Directive: exitif:SqNorm(z-PreInf2)<tiny;palette:1
              if (dummyC.SqNorm(dummyC.Sub(z,PreInf2))<tiny) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:z=((A*z)/(1.0+((B*z)+(z*z))))
              z=((A.Mult(z)).Div(((B.Mult(z)).Add(z.Mult(z))).Add(1.0)));

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int QuadRatZeroReplFPOne(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 40;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;
            int basinindex;
            if (deTool.ImageArray.ParentFrameIndex[TargetFrameIndex] > -1) {
                basinindex = deTool.ImageArray.ParentFrameIndex[TargetFrameIndex];
            } else {
                basinindex = TargetFrameIndex;
            }
            int[] detectx0 = new int[MaxIter];
            int[] detecty0 = new int[MaxIter];
            double[] detectxval0 = new double[MaxIter];
            double[] detectyval0 = new double[MaxIter];

            // initialize parameters
            double tiny;
            double big;
            Complex A;
            if (InheritParameters) {
                tiny=myMath.getVariable("tiny");
                big=myMath.getVariable("big");
                x_value=myMath.getVariable("A_x");
                y_value=myMath.getVariable("A_y");
                A=new Complex(x_value, y_value);
            } else {
                tiny=0.000001;
                big=100000;
                A=new Complex(cX,cY);
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex B=new Complex(xval,yval);
                    Complex w=new Complex(1,0);
                    Complex v=dummyC.Sqrt(dummyC.Sub((B.Mult(B)),4.0));
                    Complex PreInf1=(dummyC.Add(B.Add(v)).Mult(-0.5));
                    Complex PreInf2=(dummyC.Sub(dummyC.Sub(B,v)).Mult(-0.5));
                    double s=0;

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Set' Directive: set:s=SqNorm(w)
                            s=dummyC.SqNorm(w);

                            // 'ExitIf' Directive: exitif:s>big;palette:1
                            if (s>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=1;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'ExitIf' Directive: exitif:SqNorm(w-PreInf1)<tiny;palette:1
                            if (dummyC.SqNorm(dummyC.Sub(w,PreInf1))<tiny) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=1;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'ExitIf' Directive: exitif:SqNorm(w-PreInf2)<tiny;palette:1
                            if (dummyC.SqNorm(dummyC.Sub(w,PreInf2))<tiny) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=1;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Detect' Directive: detect:xval:GetRe(w);yval:GetIm(w);palette:3
                            if (DCD) {
                                detectxval0[iter] = dummyC.GetRe(w);
                                detectyval0[iter] = dummyC.GetIm(w);
                                detectx0[iter] = deTool.GetXFromVal(basinindex, detectxval0[iter]);
                                detecty0[iter] = deTool.GetYFromVal(basinindex, detectyval0[iter]);
                            }

                            // 'Set' Directive: set:w=((A*w)/(1.0+((B*w)+(w*w))))
                            w=((A.Mult(w)).Div(((B.Mult(w)).Add(w.Mult(w))).Add(1.0)));

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                            // cycle and basin detection code for block 0
                            if (!BlockExit[0]) {
                                switch (CDM) {
                                    case 0: { // pixel level precision
                                        if (deTool.DetectCB(TargetFrameIndex, 0, x, y, basinindex, BDM, detectx0, detecty0, iter)) {
                                            BlockExit[0] = true;
                                            Palette[0]=3;
                                        }
                                    }
                                    break;
                                    case 1: { // Double value level precision
                                        if (deTool.DetectCB(TargetFrameIndex, 0, x, y, basinindex, BDM, detectxval0, detectyval0, iter)) {
                                            BlockExit[0] = true;
                                            Palette[0]=3;
                                        }
                                    }
                                    break;
                                }
                            }
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData QuadRatZeroReplFPOne_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 40;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double tiny;
      double big;
      Complex A;
      if (InheritParameters) {
          tiny=myMath.getVariable("tiny");
          big=myMath.getVariable("big");
          x_value=myMath.getVariable("A_x");
          y_value=myMath.getVariable("A_y");
          A=new Complex(x_value, y_value);
      } else {
          tiny=0.000001;
          big=100000;
          A=new Complex(cX,cY);
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex B=new Complex(xval,yval);
      Complex w=new Complex(1,0);
      Complex v=dummyC.Sqrt(dummyC.Sub((B.Mult(B)),4.0));
      Complex PreInf1=(dummyC.Add(B.Add(v)).Mult(-0.5));
      Complex PreInf2=(dummyC.Sub(dummyC.Sub(B,v)).Mult(-0.5));
      double s=0;
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(w), dummyC.GetIm(w));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'Set' Directive: set:s=SqNorm(w)
              s=dummyC.SqNorm(w);

              // 'ExitIf' Directive: exitif:s>big;palette:1
              if (s>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'ExitIf' Directive: exitif:SqNorm(w-PreInf1)<tiny;palette:1
              if (dummyC.SqNorm(dummyC.Sub(w,PreInf1))<tiny) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'ExitIf' Directive: exitif:SqNorm(w-PreInf2)<tiny;palette:1
              if (dummyC.SqNorm(dummyC.Sub(w,PreInf2))<tiny) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:w=((A*w)/(1.0+((B*w)+(w*w))))
              w=((A.Mult(w)).Div(((B.Mult(w)).Add(w.Mult(w))).Add(1.0)));

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int QuadRatZeroReplFPBoth_FP(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 41;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            double tiny;
            double big;
            Complex A;
            if (InheritParameters) {
                tiny=myMath.getVariable("tiny");
                big=myMath.getVariable("big");
                x_value=myMath.getVariable("A_x");
                y_value=myMath.getVariable("A_y");
                A=new Complex(x_value, y_value);
            } else {
                tiny=0.000001;
                big=100000;
                A=new Complex(cX,cY);
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex B=new Complex(xval,yval);
                    Complex z=new Complex(-1,0);
                    Complex w=new Complex(1,0);
                    Complex v=dummyC.Sqrt(dummyC.Sub((B.Mult(B)),4.0));
                    Complex PreInf1=((B.Add(v)).Mult(-0.5));
                    Complex PreInf2=((dummyC.Sub(B,v)).Mult(-0.5));
                    Complex disc=dummyC.Sqrt(dummyC.Sub((B.Mult(B)),((dummyC.Sub(1.0,A)).Mult(4.0))));
                    Complex fp1=((B.Add(disc)).Mult(-0.5));
                    Complex fp2=((B.Add(disc)).Mult(-0.5));
                    double r=0;
                    double s=0;

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Set' Directive: set:r=SqNorm(z)
                            r=dummyC.SqNorm(z);

                            // 'ExitIf' Directive: exitif:r>big;palette:0
                            if (r>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'ExitIf' Directive: exitif:SqNorm((z-PreInf1))<tiny;palette:0
                            if (dummyC.SqNorm(dummyC.Sub(z,PreInf1))<tiny) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'ExitIf' Directive: exitif:SqNorm((z-PreInf2))<tiny;palette:0
                            if (dummyC.SqNorm(dummyC.Sub(z,PreInf2))<tiny) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'ExitIf' Directive: exitif:SqNorm((z-fp1))<tiny;palette:2
                            if (dummyC.SqNorm(dummyC.Sub(z,fp1))<tiny) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=2;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'ExitIf' Directive: exitif:SqNorm((z-fp2))<tiny;palette:2
                            if (dummyC.SqNorm(dummyC.Sub(z,fp2))<tiny) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=2;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Set' Directive: set:z=((A*z)/(1.0+((B*z)+(z*z))))
                            z=((A.Mult(z)).Div(((B.Mult(z)).Add(z.Mult(z))).Add(1.0)));

                        } // End of Block 0

                        Block1: // Start of Block 1
                        if (!BlockExit[1]) {

                            // 'Set' Directive: set:s=SqNorm(w)
                            s=dummyC.SqNorm(w);

                            // 'ExitIf' Directive: exitif:s>big;palette:1
                            if (s>big) {
                                DidExit++;
                                BlockExit[1] = true;
                                Palette[1]=1;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[1][x][y] = iter;
                                break Block1;
                            }

                            // 'ExitIf' Directive: exitif:SqNorm((w-PreInf1))<tiny;palette:1
                            if (dummyC.SqNorm(dummyC.Sub(w,PreInf1))<tiny) {
                                DidExit++;
                                BlockExit[1] = true;
                                Palette[1]=1;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[1][x][y] = iter;
                                break Block1;
                            }

                            // 'ExitIf' Directive: exitif:SqNorm((w-PreInf2))<tiny;palette:1
                            if (dummyC.SqNorm(dummyC.Sub(w,PreInf2))<tiny) {
                                DidExit++;
                                BlockExit[1] = true;
                                Palette[1]=1;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[1][x][y] = iter;
                                break Block1;
                            }

                            // 'ExitIf' Directive: exitif:SqNorm((w-fp1))<tiny;palette:3
                            if (dummyC.SqNorm(dummyC.Sub(w,fp1))<tiny) {
                                DidExit++;
                                BlockExit[1] = true;
                                Palette[1]=3;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[1][x][y] = iter;
                                break Block1;
                            }

                            // 'ExitIf' Directive: exitif:SqNorm((w-fp2))<tiny;palette:3
                            if (dummyC.SqNorm(dummyC.Sub(w,fp2))<tiny) {
                                DidExit++;
                                BlockExit[1] = true;
                                Palette[1]=3;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[1][x][y] = iter;
                                break Block1;
                            }

                            // 'Set' Directive: set:w=((A*w)/(1.0+((B*w)+(w*w))))
                            w=((A.Mult(w)).Div(((B.Mult(w)).Add(w.Mult(w))).Add(1.0)));

                        } // End of Block 1

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData QuadRatZeroReplFPBoth_FP_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 41;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double tiny;
      double big;
      Complex A;
      if (InheritParameters) {
          tiny=myMath.getVariable("tiny");
          big=myMath.getVariable("big");
          x_value=myMath.getVariable("A_x");
          y_value=myMath.getVariable("A_y");
          A=new Complex(x_value, y_value);
      } else {
          tiny=0.000001;
          big=100000;
          A=new Complex(cX,cY);
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex B=new Complex(xval,yval);
      Complex z=new Complex(-1,0);
      Complex w=new Complex(1,0);
      Complex v=dummyC.Sqrt(dummyC.Sub((B.Mult(B)),4.0));
      Complex PreInf1=((B.Add(v)).Mult(-0.5));
      Complex PreInf2=((dummyC.Sub(B,v)).Mult(-0.5));
      Complex disc=dummyC.Sqrt(dummyC.Sub((B.Mult(B)),((dummyC.Sub(1.0,A)).Mult(4.0))));
      Complex fp1=((B.Add(disc)).Mult(-0.5));
      Complex fp2=((B.Add(disc)).Mult(-0.5));
      double r=0;
      double s=0;
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(z), dummyC.GetIm(z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'Set' Directive: set:r=SqNorm(z)
              r=dummyC.SqNorm(z);

              // 'ExitIf' Directive: exitif:r>big;palette:0
              if (r>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'ExitIf' Directive: exitif:SqNorm((z-PreInf1))<tiny;palette:0
              if (dummyC.SqNorm(dummyC.Sub(z,PreInf1))<tiny) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'ExitIf' Directive: exitif:SqNorm((z-PreInf2))<tiny;palette:0
              if (dummyC.SqNorm(dummyC.Sub(z,PreInf2))<tiny) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'ExitIf' Directive: exitif:SqNorm((z-fp1))<tiny;palette:2
              if (dummyC.SqNorm(dummyC.Sub(z,fp1))<tiny) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'ExitIf' Directive: exitif:SqNorm((z-fp2))<tiny;palette:2
              if (dummyC.SqNorm(dummyC.Sub(z,fp2))<tiny) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:z=((A*z)/(1.0+((B*z)+(z*z))))
              z=((A.Mult(z)).Div(((B.Mult(z)).Add(z.Mult(z))).Add(1.0)));

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

  public TraceData QuadRatZeroReplFPBoth_FP_1_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 41;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double tiny;
      double big;
      Complex A;
      if (InheritParameters) {
          tiny=myMath.getVariable("tiny");
          big=myMath.getVariable("big");
          x_value=myMath.getVariable("A_x");
          y_value=myMath.getVariable("A_y");
          A=new Complex(x_value, y_value);
      } else {
          tiny=0.000001;
          big=100000;
          A=new Complex(cX,cY);
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex B=new Complex(xval,yval);
      Complex z=new Complex(-1,0);
      Complex w=new Complex(1,0);
      Complex v=dummyC.Sqrt(dummyC.Sub((B.Mult(B)),4.0));
      Complex PreInf1=((B.Add(v)).Mult(-0.5));
      Complex PreInf2=((dummyC.Sub(B,v)).Mult(-0.5));
      Complex disc=dummyC.Sqrt(dummyC.Sub((B.Mult(B)),((dummyC.Sub(1.0,A)).Mult(4.0))));
      Complex fp1=((B.Add(disc)).Mult(-0.5));
      Complex fp2=((B.Add(disc)).Mult(-0.5));
      double r=0;
      double s=0;
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(w), dummyC.GetIm(w));
          // User-defined directives are executed for each iteration in inner loop: 

          Block1: // Start of Block 1
          if (!BlockExit[1]) {

              // 'Set' Directive: set:s=SqNorm(w)
              s=dummyC.SqNorm(w);

              // 'ExitIf' Directive: exitif:s>big;palette:1
              if (s>big) {
                  DidExit++;
                  BlockExit[1] = true;
                  break Block1;
              }

              // 'ExitIf' Directive: exitif:SqNorm((w-PreInf1))<tiny;palette:1
              if (dummyC.SqNorm(dummyC.Sub(w,PreInf1))<tiny) {
                  DidExit++;
                  BlockExit[1] = true;
                  break Block1;
              }

              // 'ExitIf' Directive: exitif:SqNorm((w-PreInf2))<tiny;palette:1
              if (dummyC.SqNorm(dummyC.Sub(w,PreInf2))<tiny) {
                  DidExit++;
                  BlockExit[1] = true;
                  break Block1;
              }

              // 'ExitIf' Directive: exitif:SqNorm((w-fp1))<tiny;palette:3
              if (dummyC.SqNorm(dummyC.Sub(w,fp1))<tiny) {
                  DidExit++;
                  BlockExit[1] = true;
                  break Block1;
              }

              // 'ExitIf' Directive: exitif:SqNorm((w-fp2))<tiny;palette:3
              if (dummyC.SqNorm(dummyC.Sub(w,fp2))<tiny) {
                  DidExit++;
                  BlockExit[1] = true;
                  break Block1;
              }

              // 'Set' Directive: set:w=((A*w)/(1.0+((B*w)+(w*w))))
              w=((A.Mult(w)).Div(((B.Mult(w)).Add(w.Mult(w))).Add(1.0)));

          } // End of Block 1
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int QuadRat_Per1_A_derivative(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 42;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            double tiny;
            double big;
            Complex A;
            if (InheritParameters) {
                tiny=myMath.getVariable("tiny");
                big=myMath.getVariable("big");
                x_value=myMath.getVariable("A_x");
                y_value=myMath.getVariable("A_y");
                A=new Complex(x_value, y_value);
            } else {
                tiny=0.0000000001;
                big=1000000000;
                A=new Complex(cX,cY);
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex B=new Complex(xval,yval);
                    Complex cplus=new Complex(1,0);
                    Complex cminus=new Complex(-1,0);
                    Complex disc=dummyC.Sqrt(dummyC.Sub(B.Mult(B),4.0));
                    Complex PreInf1=(B.Add(disc)).Mult(-0.5);
                    Complex PreInf2=(dummyC.Sub(B,disc)).Mult(-0.5);
                    Complex Dplus=new Complex(1,0);
                    Complex Dminus=new Complex(1,0);
                    Complex denomplus=new Complex(1,0);
                    Complex denomminus=new Complex(1,0);

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'ExitIf' Directive: exitif:SqNorm(cplus-PreInf1)<tiny;palette:0
                            if (dummyC.SqNorm(dummyC.Sub(cplus,PreInf1))<tiny) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'ExitIf' Directive: exitif:SqNorm(cplus-PreInf2)<tiny;palette:0
                            if (dummyC.SqNorm(dummyC.Sub(cplus,PreInf2))<tiny) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Set' Directive: set:denomplus=1.0+((B+cplus)*cplus)
                            denomplus=((B.Add(cplus)).Mult(cplus)).Add(1.0);

                            // 'Set' Directive: set:cplus=(A*cplus)/(denomplus)
                            cplus=(A.Mult(cplus)).Div(denomplus);

                            // 'Set' Directive: set:Dplus=Dplus*((A*(1.0-cplus*cplus))/(denomplus*denomplus))
                            Dplus=Dplus.Mult((A.Mult(dummyC.Sub(1.0,cplus.Mult(cplus)))).Div(denomplus.Mult(denomplus)));

                            // 'ExitIf' Directive: exitif:SqNorm(Dplus)<tiny;palette:2
                            if (dummyC.SqNorm(Dplus)<tiny) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=2;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'ExitIf' Directive: exitif:SqNorm(cplus)>big;palette:1
                            if (dummyC.SqNorm(cplus)>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=1;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                        } // End of Block 0

                        Block1: // Start of Block 1
                        if (!BlockExit[1]) {

                            // 'Set' Directive: set:denomminus=1.0+((B+cminus)*cminus)
                            denomminus=((B.Add(cminus)).Mult(cminus)).Add(1.0);

                            // 'ExitIf' Directive: exitif:SqNorm(cminus-PreInf1)<tiny;palette:0
                            if (dummyC.SqNorm(dummyC.Sub(cminus,PreInf1))<tiny) {
                                DidExit++;
                                BlockExit[1] = true;
                                Palette[1]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[1][x][y] = iter;
                                break Block1;
                            }

                            // 'ExitIf' Directive: exitif:SqNorm(cminus-PreInf2)<tiny;palette:0
                            if (dummyC.SqNorm(dummyC.Sub(cminus,PreInf2))<tiny) {
                                DidExit++;
                                BlockExit[1] = true;
                                Palette[1]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[1][x][y] = iter;
                                break Block1;
                            }

                            // 'Set' Directive: set:cminus=(A*cminus)/(denomminus)
                            cminus=(A.Mult(cminus)).Div(denomminus);

                            // 'Set' Directive: set:Dminus=Dminus*((A*(1.0-cplus*cplus))/(denomminus*denomminus))
                            Dminus=Dminus.Mult((A.Mult(dummyC.Sub(1.0,cplus.Mult(cplus)))).Div(denomminus.Mult(denomminus)));

                            // 'ExitIf' Directive: exitif:SqNorm(Dminus)<tiny;palette:3
                            if (dummyC.SqNorm(Dminus)<tiny) {
                                DidExit++;
                                BlockExit[1] = true;
                                Palette[1]=3;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[1][x][y] = iter;
                                break Block1;
                            }

                            // 'ExitIf' Directive: exitif:SqNorm(cminus)>big;palette:1
                            if (dummyC.SqNorm(cminus)>big) {
                                DidExit++;
                                BlockExit[1] = true;
                                Palette[1]=1;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[1][x][y] = iter;
                                break Block1;
                            }

                        } // End of Block 1

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

    public int Per_3_zero_cplane(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 43;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            double big;
            double tiny;
            if (InheritParameters) {
                big=myMath.getVariable("big");
                tiny=myMath.getVariable("tiny");
            } else {
                big=100000;
                tiny=0.00001;
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);
            Complex p1=new Complex(-1,0);

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex C=new Complex(xval,yval);
                    Complex a=C.Div(dummyC.Sub(2,C));
                    Complex Z=(a.Mult(2)).Div(a.Add(1));

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'ExitIf' Directive: exitif:SqNorm(Z)>big;palette:0
                            if (dummyC.SqNorm(Z)>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Set' Directive: set:Z=(((Z-a)*(Z-1))/(Z*Z))
                            Z=(((dummyC.Sub(Z,a)).Mult(dummyC.Sub(Z,1))).Div(Z.Mult(Z)));

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[1]=dummyC.GetRe(p1);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[1]=dummyC.GetIm(p1);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData Per_3_zero_cplane_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 43;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double big;
      double tiny;
      if (InheritParameters) {
          big=myMath.getVariable("big");
          tiny=myMath.getVariable("tiny");
      } else {
          big=100000;
          tiny=0.00001;
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex C=new Complex(xval,yval);
      Complex a=C.Div(dummyC.Sub(2,C));
      Complex Z=(a.Mult(2)).Div(a.Add(1));
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(Z), dummyC.GetIm(Z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'ExitIf' Directive: exitif:SqNorm(Z)>big;palette:0
              if (dummyC.SqNorm(Z)>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:Z=(((Z-a)*(Z-1))/(Z*Z))
              Z=(((dummyC.Sub(Z,a)).Mult(dummyC.Sub(Z,1))).Div(Z.Mult(Z)));

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int Per_3_zero_Julia(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 44;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            double big;
            double tiny;
            Complex C;
            if (InheritParameters) {
                big=myMath.getVariable("big");
                tiny=myMath.getVariable("tiny");
                x_value=myMath.getVariable("C_x");
                y_value=myMath.getVariable("C_y");
                C=new Complex(x_value, y_value);
            } else {
                big=100000;
                tiny=0.00001;
                C=new Complex(cX,cY);
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);
            Complex p1=new Complex(1,0);

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex a=C.Div(dummyC.Sub(2,C));
                    Complex Z=new Complex(xval,yval);

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'ExitIf' Directive: exitif:SqNorm(Z)>big;palette:0
                            if (dummyC.SqNorm(Z)>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Set' Directive: set:Z=((Z-a)*(Z-1))/(Z*Z)
                            Z=((dummyC.Sub(Z,a)).Mult(dummyC.Sub(Z,1))).Div(Z.Mult(Z));

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[1]=dummyC.GetRe(p1);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[1]=dummyC.GetIm(p1);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData Per_3_zero_Julia_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 44;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double big;
      double tiny;
      Complex C;
      if (InheritParameters) {
          big=myMath.getVariable("big");
          tiny=myMath.getVariable("tiny");
          x_value=myMath.getVariable("C_x");
          y_value=myMath.getVariable("C_y");
          C=new Complex(x_value, y_value);
      } else {
          big=100000;
          tiny=0.00001;
          C=new Complex(cX,cY);
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex a=C.Div(dummyC.Sub(2,C));
      Complex Z=new Complex(xval,yval);
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(Z), dummyC.GetIm(Z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'ExitIf' Directive: exitif:SqNorm(Z)>big;palette:0
              if (dummyC.SqNorm(Z)>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:Z=((Z-a)*(Z-1))/(Z*Z)
              Z=((dummyC.Sub(Z,a)).Mult(dummyC.Sub(Z,1))).Div(Z.Mult(Z));

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int binomial_random_mandel_complex_simon(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 45;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            double n;
            double errorbound;
            double big;
            double lambda;
            int binom;
            if (InheritParameters) {
                n=myMath.getVariable("n");
                errorbound=myMath.getVariable("errorbound");
                big=myMath.getVariable("big");
                lambda=myMath.getVariable("lambda");
                binom=(int) myMath.getVariable("binom");
            } else {
                n=2;
                errorbound=0.01;
                big=50;
                lambda=1;
                binom=0;
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);
            Complex p1=new Complex(-1,0);

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex Z=new Complex(xval,yval);
                    Complex C=Z;
                    Complex error=new Complex(0,0);

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'ExitIf' Directive: exitif:SqNorm(Z)>big;palette:0;split:GetIm(Z);palette:1,2
                            if (dummyC.SqNorm(Z)>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                SplitResult[0]=dummyC.GetIm(Z);
                                break Block0;
                            }
			if(Math.random()>0.5) {
				binom=1;
			} else {
				binom=0;
			}

                            // 'Set' Directive: set:error=error.SetVals(binom * errorbound, binom * errorbound)
                            error=error.SetVals(binom*errorbound,binom*errorbound);

                            // 'Set' Directive: set:Z=(Z^n)+(C+error)
                            Z=(Z.Pow(n)).Add(C.Add(error));

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[1]=dummyC.GetRe(p1);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[1]=dummyC.GetIm(p1);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData binomial_random_mandel_complex_simon_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 45;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double n;
      double errorbound;
      double big;
      double lambda;
      int binom;
      if (InheritParameters) {
          n=myMath.getVariable("n");
          errorbound=myMath.getVariable("errorbound");
          big=myMath.getVariable("big");
          lambda=myMath.getVariable("lambda");
          binom=(int) myMath.getVariable("binom");
      } else {
          n=2;
          errorbound=0.01;
          big=50;
          lambda=1;
          binom=0;
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex Z=new Complex(xval,yval);
      Complex C=Z;
      Complex error=new Complex(0,0);
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(Z), dummyC.GetIm(Z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'ExitIf' Directive: exitif:SqNorm(Z)>big;palette:0;split:GetIm(Z);palette:1,2
              if (dummyC.SqNorm(Z)>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }
			if(Math.random()>0.5) {
				binom=1;
			} else {
				binom=0;
			}

              // 'Set' Directive: set:error=error.SetVals(binom * errorbound, binom * errorbound)
              error=error.SetVals(binom*errorbound,binom*errorbound);

              // 'Set' Directive: set:Z=(Z^n)+(C+error)
              Z=(Z.Pow(n)).Add(C.Add(error));

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int exponential_random_complex_julia_simon(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 46;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            double n;
            double big;
            double errorbound;
            Complex C;
            double lambda;
            if (InheritParameters) {
                n=myMath.getVariable("n");
                big=myMath.getVariable("big");
                errorbound=myMath.getVariable("errorbound");
                x_value=myMath.getVariable("C_x");
                y_value=myMath.getVariable("C_y");
                C=new Complex(x_value, y_value);
                lambda=myMath.getVariable("lambda");
            } else {
                n=2;
                big=50;
                errorbound=0.0001;
                C=new Complex(cX,cY);
                lambda=1;
            }

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex Z=new Complex(xval,yval);
                    Complex error=new Complex(0,0);
                    double r=0;

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Set' Directive: set:r=SqNorm(Z)
                            r=dummyC.SqNorm(Z);

                            // 'ExitIf' Directive: exitif:r>big;palette:0
                            if (r>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                break Block0;
                            }

                            // 'Set' Directive: set:error=error.SetVals(-1/lambda*log(1-random())*errorbound, -1/lambda*log(1-random())*errorbound)
                            error=error.SetVals(-1/lambda*Math.log(1-Math.random())*errorbound,-1/lambda*Math.log(1-Math.random())*errorbound);

                            // 'Set' Directive: set:Z=(Z^n)+(C+error)
                            Z=(Z.Pow(n)).Add(C.Add(error));

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData exponential_random_complex_julia_simon_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 46;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double n;
      double big;
      double errorbound;
      Complex C;
      double lambda;
      if (InheritParameters) {
          n=myMath.getVariable("n");
          big=myMath.getVariable("big");
          errorbound=myMath.getVariable("errorbound");
          x_value=myMath.getVariable("C_x");
          y_value=myMath.getVariable("C_y");
          C=new Complex(x_value, y_value);
          lambda=myMath.getVariable("lambda");
      } else {
          n=2;
          big=50;
          errorbound=0.0001;
          C=new Complex(cX,cY);
          lambda=1;
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex Z=new Complex(xval,yval);
      Complex error=new Complex(0,0);
      double r=0;
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(Z), dummyC.GetIm(Z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'Set' Directive: set:r=SqNorm(Z)
              r=dummyC.SqNorm(Z);

              // 'ExitIf' Directive: exitif:r>big;palette:0
              if (r>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:error=error.SetVals(-1/lambda*log(1-random())*errorbound, -1/lambda*log(1-random())*errorbound)
              error=error.SetVals(-1/lambda*Math.log(1-Math.random())*errorbound,-1/lambda*Math.log(1-Math.random())*errorbound);

              // 'Set' Directive: set:Z=(Z^n)+(C+error)
              Z=(Z.Pow(n)).Add(C.Add(error));

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int exponential_random_mandel_complex_simon(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 47;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            double n;
            double errorbound;
            double big;
            double lambda;
            if (InheritParameters) {
                n=myMath.getVariable("n");
                errorbound=myMath.getVariable("errorbound");
                big=myMath.getVariable("big");
                lambda=myMath.getVariable("lambda");
            } else {
                n=2;
                errorbound=0.01;
                big=50;
                lambda=1;
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);
            Complex p1=new Complex(-1,0);

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex Z=new Complex(xval,yval);
                    Complex C=Z;
                    Complex error=new Complex(0,0);

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'ExitIf' Directive: exitif:SqNorm(Z)>big;palette:0;split:GetIm(Z);palette:1,2
                            if (dummyC.SqNorm(Z)>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                SplitResult[0]=dummyC.GetIm(Z);
                                break Block0;
                            }

                            // 'Set' Directive: set:error=error.SetVals(-1/lambda*log(1-random())*errorbound, -1/lambda*log(1-random())*errorbound)
                            error=error.SetVals(-1/lambda*Math.log(1-Math.random())*errorbound,-1/lambda*Math.log(1-Math.random())*errorbound);

                            // 'Set' Directive: set:Z=(Z^n)+(C+error)
                            Z=(Z.Pow(n)).Add(C.Add(error));

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[1]=dummyC.GetRe(p1);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[1]=dummyC.GetIm(p1);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData exponential_random_mandel_complex_simon_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 47;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double n;
      double errorbound;
      double big;
      double lambda;
      if (InheritParameters) {
          n=myMath.getVariable("n");
          errorbound=myMath.getVariable("errorbound");
          big=myMath.getVariable("big");
          lambda=myMath.getVariable("lambda");
      } else {
          n=2;
          errorbound=0.01;
          big=50;
          lambda=1;
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex Z=new Complex(xval,yval);
      Complex C=Z;
      Complex error=new Complex(0,0);
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(Z), dummyC.GetIm(Z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'ExitIf' Directive: exitif:SqNorm(Z)>big;palette:0;split:GetIm(Z);palette:1,2
              if (dummyC.SqNorm(Z)>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:error=error.SetVals(-1/lambda*log(1-random())*errorbound, -1/lambda*log(1-random())*errorbound)
              error=error.SetVals(-1/lambda*Math.log(1-Math.random())*errorbound,-1/lambda*Math.log(1-Math.random())*errorbound);

              // 'Set' Directive: set:Z=(Z^n)+(C+error)
              Z=(Z.Pow(n)).Add(C.Add(error));

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int normal_random_mandel_complex_simon(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 48;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            double n;
            double errorbound;
            double big;
            double lambda;
            if (InheritParameters) {
                n=myMath.getVariable("n");
                errorbound=myMath.getVariable("errorbound");
                big=myMath.getVariable("big");
                lambda=myMath.getVariable("lambda");
            } else {
                n=2;
                errorbound=0.0001;
                big=50;
                lambda=1;
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);
            Complex p1=new Complex(-1,0);

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex Z=new Complex(xval,yval);
                    Complex C=Z;
                    Complex error=new Complex(0,0);

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'ExitIf' Directive: exitif:SqNorm(Z)>big;palette:0;split:GetIm(Z);palette:1,2
                            if (dummyC.SqNorm(Z)>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                SplitResult[0]=dummyC.GetIm(Z);
                                break Block0;
                            }

                            // 'Set' Directive: set:error=error.SetVals(NormalDist.inverseF(0,1,random())*errorbound,NormalDist.inverseF(0,1,random())*errorbound)
                            error=error.SetVals(NormalDist.inverseF(0,1,Math.random())*errorbound,NormalDist.inverseF(0,1,Math.random())*errorbound);

                            // 'Set' Directive: set:Z=(Z^n)+(C+error)
                            Z=(Z.Pow(n)).Add(C.Add(error));

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[1]=dummyC.GetRe(p1);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[1]=dummyC.GetIm(p1);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData normal_random_mandel_complex_simon_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 48;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double n;
      double errorbound;
      double big;
      double lambda;
      if (InheritParameters) {
          n=myMath.getVariable("n");
          errorbound=myMath.getVariable("errorbound");
          big=myMath.getVariable("big");
          lambda=myMath.getVariable("lambda");
      } else {
          n=2;
          errorbound=0.0001;
          big=50;
          lambda=1;
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex Z=new Complex(xval,yval);
      Complex C=Z;
      Complex error=new Complex(0,0);
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(Z), dummyC.GetIm(Z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'ExitIf' Directive: exitif:SqNorm(Z)>big;palette:0;split:GetIm(Z);palette:1,2
              if (dummyC.SqNorm(Z)>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:error=error.SetVals(NormalDist.inverseF(0,1,random())*errorbound,NormalDist.inverseF(0,1,random())*errorbound)
              error=error.SetVals(NormalDist.inverseF(0,1,Math.random())*errorbound,NormalDist.inverseF(0,1,Math.random())*errorbound);

              // 'Set' Directive: set:Z=(Z^n)+(C+error)
              Z=(Z.Pow(n)).Add(C.Add(error));

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int binomial_random_complex_julia_simon(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 49;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            double n;
            double r;
            double errorbound;
            double big;
            double tiny;
            int binom;
            Complex C;
            if (InheritParameters) {
                n=myMath.getVariable("n");
                r=myMath.getVariable("r");
                errorbound=myMath.getVariable("errorbound");
                big=myMath.getVariable("big");
                tiny=myMath.getVariable("tiny");
                binom=(int) myMath.getVariable("binom");
                x_value=myMath.getVariable("C_x");
                y_value=myMath.getVariable("C_y");
                C=new Complex(x_value, y_value);
            } else {
                n=2;
                r=0;
                errorbound=0.01;
                big=50;
                tiny=0.001;
                binom=0;
                C=new Complex(cX,cY);
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex Z=new Complex(xval,yval);
                    Complex error=new Complex(0,0);

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Set' Directive: set:r=SqNorm(Z)
                            r=dummyC.SqNorm(Z);

                            // 'ExitIf' Directive: exitif:r>big;palette:0;split:GetIm(Z);palette:2,3
                            if (r>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                SplitResult[0]=dummyC.GetIm(Z);
                                break Block0;
                            }

                            // 'ExitIf' Directive: exitif:r<tiny;palette:1
                            if (r<tiny) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=1;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                SplitResult[0]=dummyC.GetIm(Z);
                                break Block0;
                            }
			if(Math.random()>0.5) {
				binom=1;
			} else {
				binom=0;
			}

                            // 'Set' Directive: set:error=error.SetVals(binom * errorbound, binom * errorbound)
                            error=error.SetVals(binom*errorbound,binom*errorbound);

                            // 'Set' Directive: set:Z=(Z^n)+(C+error)
                            Z=(Z.Pow(n)).Add(C.Add(error));

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData binomial_random_complex_julia_simon_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 49;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double n;
      double r;
      double errorbound;
      double big;
      double tiny;
      int binom;
      Complex C;
      if (InheritParameters) {
          n=myMath.getVariable("n");
          r=myMath.getVariable("r");
          errorbound=myMath.getVariable("errorbound");
          big=myMath.getVariable("big");
          tiny=myMath.getVariable("tiny");
          binom=(int) myMath.getVariable("binom");
          x_value=myMath.getVariable("C_x");
          y_value=myMath.getVariable("C_y");
          C=new Complex(x_value, y_value);
      } else {
          n=2;
          r=0;
          errorbound=0.01;
          big=50;
          tiny=0.001;
          binom=0;
          C=new Complex(cX,cY);
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex Z=new Complex(xval,yval);
      Complex error=new Complex(0,0);
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(Z), dummyC.GetIm(Z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'Set' Directive: set:r=SqNorm(Z)
              r=dummyC.SqNorm(Z);

              // 'ExitIf' Directive: exitif:r>big;palette:0;split:GetIm(Z);palette:2,3
              if (r>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'ExitIf' Directive: exitif:r<tiny;palette:1
              if (r<tiny) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }
			if(Math.random()>0.5) {
				binom=1;
			} else {
				binom=0;
			}

              // 'Set' Directive: set:error=error.SetVals(binom * errorbound, binom * errorbound)
              error=error.SetVals(binom*errorbound,binom*errorbound);

              // 'Set' Directive: set:Z=(Z^n)+(C+error)
              Z=(Z.Pow(n)).Add(C.Add(error));

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int mandel_complex_array_input(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 50;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;
            int basinindex;
            if (deTool.ImageArray.ParentFrameIndex[TargetFrameIndex] > -1) {
                basinindex = deTool.ImageArray.ParentFrameIndex[TargetFrameIndex];
            } else {
                basinindex = TargetFrameIndex;
            }
            int[] detectx0 = new int[MaxIter];
            int[] detecty0 = new int[MaxIter];
            double[] detectxval0 = new double[MaxIter];
            double[] detectyval0 = new double[MaxIter];

            // initialize parameters
            double n;
            double big;
            if (InheritParameters) {
                n=myMath.getVariable("n");
                big=myMath.getVariable("big");
            } else {
                n=2;
                big=BIG;
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);
            Complex p1=new Complex(0,-1);

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex Z=new Complex(xval,yval);
                    Complex C=Z;
                    double test=0;

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'ExitIf' Directive: exitif:SqNorm(Z)>big;palette:0;split:GetIm(Z);palette:1,2
                            if (dummyC.SqNorm(Z)>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                SplitResult[0]=dummyC.GetIm(Z);
                                break Block0;
                            }

                            // 'Set' Directive: set:Z=Z^n+C
                            Z=Z.Pow(n).Add(C);

                            // 'Detect' Directive: detect:xval:GetRe(Z);yval:GetIm(Z);palette:3
                            if (DCD) {
                                detectxval0[iter] = dummyC.GetRe(Z);
                                detectyval0[iter] = dummyC.GetIm(Z);
                                detectx0[iter] = deTool.GetXFromVal(basinindex, detectxval0[iter]);
                                detecty0[iter] = deTool.GetYFromVal(basinindex, detectyval0[iter]);
                            }

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                            // cycle and basin detection code for block 0
                            if (!BlockExit[0]) {
                                switch (CDM) {
                                    case 0: { // pixel level precision
                                        if (deTool.DetectCB(TargetFrameIndex, 0, x, y, basinindex, BDM, detectx0, detecty0, iter)) {
                                            BlockExit[0] = true;
                                            Palette[0]=3;
                                            SplitResult[0]=dummyC.GetIm(Z);
                                        }
                                    }
                                    break;
                                    case 1: { // Double value level precision
                                        if (deTool.DetectCB(TargetFrameIndex, 0, x, y, basinindex, BDM, detectxval0, detectyval0, iter)) {
                                            BlockExit[0] = true;
                                            Palette[0]=3;
                                            SplitResult[0]=dummyC.GetIm(Z);
                                        }
                                    }
                                    break;
                                }
                            }
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[1]=dummyC.GetRe(p1);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[1]=dummyC.GetIm(p1);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData mandel_complex_array_input_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 50;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double n;
      double big;
      if (InheritParameters) {
          n=myMath.getVariable("n");
          big=myMath.getVariable("big");
      } else {
          n=2;
          big=BIG;
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex Z=new Complex(xval,yval);
      Complex C=Z;
      double test=0;
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(Z), dummyC.GetIm(Z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'ExitIf' Directive: exitif:SqNorm(Z)>big;palette:0;split:GetIm(Z);palette:1,2
              if (dummyC.SqNorm(Z)>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:Z=Z^n+C
              Z=Z.Pow(n).Add(C);

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int mandel_complex_julia_array_input(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 51;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;
            int basinindex;
            if (deTool.ImageArray.ParentFrameIndex[TargetFrameIndex] > -1) {
                basinindex = deTool.ImageArray.ParentFrameIndex[TargetFrameIndex];
            } else {
                basinindex = TargetFrameIndex;
            }
            int[] detectx0 = new int[MaxIter];
            int[] detecty0 = new int[MaxIter];
            double[] detectxval0 = new double[MaxIter];
            double[] detectyval0 = new double[MaxIter];

            // initialize parameters
            double n;
            double big;
            Complex C;
            if (InheritParameters) {
                n=myMath.getVariable("n");
                big=myMath.getVariable("big");
                x_value=myMath.getVariable("C_x");
                y_value=myMath.getVariable("C_y");
                C=new Complex(x_value, y_value);
            } else {
                n=2;
                big=100;
                C=new Complex(cX,cY);
            }

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex Z=new Complex(xval,yval);
                    Complex S=new Complex();

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'Detect' Directive: detect:xval:GetRe(Z);yval:GetIm(Z);palette:3
                            if (DCD) {
                                detectxval0[iter] = dummyC.GetRe(Z);
                                detectyval0[iter] = dummyC.GetIm(Z);
                                detectx0[iter] = deTool.GetXFromVal(basinindex, detectxval0[iter]);
                                detecty0[iter] = deTool.GetYFromVal(basinindex, detectyval0[iter]);
                            }

                            // 'ExitIf' Directive: exitif:SqNorm(Z)>big;palette:0;split:GetIm(Z);palette:1,2
                            if (dummyC.SqNorm(Z)>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                SplitResult[0]=dummyC.GetIm(Z);
                                break Block0;
                            }

                            // 'Set' Directive: set:S=S.SetRe(GetData(ProfileIndex,0,iter,0))
                            S=S.SetRe(GetData1(ProfileIndex,0,iter,0));

                            // 'Set' Directive: set:S=S.SetIm(GetData(ProfileIndex,0,iter,1))
                            S=S.SetIm(GetData1(ProfileIndex,0,iter,1));

                            // 'Set' Directive: set:Z=(Z^n)+C+S
                            Z=(Z.Pow(n)).Add(C).Add(S);

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                            // cycle and basin detection code for block 0
                            if (!BlockExit[0]) {
                                switch (CDM) {
                                    case 0: { // pixel level precision
                                        if (deTool.DetectCB(TargetFrameIndex, 0, x, y, basinindex, BDM, detectx0, detecty0, iter)) {
                                            BlockExit[0] = true;
                                            Palette[0]=3;
                                            SplitResult[0]=dummyC.GetIm(Z);
                                        }
                                    }
                                    break;
                                    case 1: { // Double value level precision
                                        if (deTool.DetectCB(TargetFrameIndex, 0, x, y, basinindex, BDM, detectxval0, detectyval0, iter)) {
                                            BlockExit[0] = true;
                                            Palette[0]=3;
                                            SplitResult[0]=dummyC.GetIm(Z);
                                        }
                                    }
                                    break;
                                }
                            }
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData mandel_complex_julia_array_input_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 51;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double n;
      double big;
      Complex C;
      if (InheritParameters) {
          n=myMath.getVariable("n");
          big=myMath.getVariable("big");
          x_value=myMath.getVariable("C_x");
          y_value=myMath.getVariable("C_y");
          C=new Complex(x_value, y_value);
      } else {
          n=2;
          big=100;
          C=new Complex(cX,cY);
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex Z=new Complex(xval,yval);
      Complex S=new Complex();
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(Z), dummyC.GetIm(Z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'ExitIf' Directive: exitif:SqNorm(Z)>big;palette:0;split:GetIm(Z);palette:1,2
              if (dummyC.SqNorm(Z)>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }

              // 'Set' Directive: set:S=S.SetRe(GetData(ProfileIndex,0,iter,0))
              S=S.SetRe(GetData1(ProfileIndex,0,iter,0));

              // 'Set' Directive: set:S=S.SetIm(GetData(ProfileIndex,0,iter,1))
              S=S.SetIm(GetData1(ProfileIndex,0,iter,1));

              // 'Set' Directive: set:Z=(Z^n)+C+S
              Z=(Z.Pow(n)).Add(C).Add(S);

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

    public int bernoulli_mandel_complex(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 52;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            double n;
            double big;
            double ratio;
            Complex C0;
            if (InheritParameters) {
                n=myMath.getVariable("n");
                big=myMath.getVariable("big");
                ratio=myMath.getVariable("ratio");
                x_value=myMath.getVariable("C0_x");
                y_value=myMath.getVariable("C0_y");
                C0=new Complex(x_value, y_value);
            } else {
                n=2;
                big=50;
                ratio=0.5;
                C0=new Complex(0,0);
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex Z=new Complex(xval,yval);
                    Complex C1=Z;
                    Complex C=new Complex(0,0);

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'ExitIf' Directive: exitif:SqNorm(Z)>big;palette:0;split:GetIm(Z);palette:1,2
                            if (dummyC.SqNorm(Z)>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                SplitResult[0]=dummyC.GetIm(Z);
                                break Block0;
                            }
			if(Math.random()>ratio) {
				C=C1;
			} else {
				C=C0;
			}

                            // 'Set' Directive: set:Z=(Z^n)+C
                            Z=(Z.Pow(n)).Add(C);

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

    public int bernoulli_complex_julia(int TargetFrameIndex, int ChunkNumber, int StartX, int EndX, int StartY, int EndY, int MaxIter, boolean DCD, int CDM, int BDM) { 
        try{
            int ProfileIndex = 53;
            boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
            int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
            boolean[] BlockSplit = deTool.ProfileArray.BlockSplit[ProfileIndex];
            int[] SplitPaletteA = deTool.ProfileArray.SplitPaletteA[ProfileIndex];
            int[] SplitPaletteB = deTool.ProfileArray.SplitPaletteB[ProfileIndex];
            MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
            double cX = myMath.getVariable("cX");
            double cY = myMath.getVariable("cY");
            double TINY = myMath.getVariable("TINY");
            double BIG = myMath.getVariable("BIG");
            double myProgress = 0.0;
            int Width = EndX - StartX;
            int Height = EndY - StartY;
            double NumPixels = Width * Height;
            int x, y, iter;
            double xval = 0, yval = 0;
            boolean[] BlockExit = new boolean[BlockCount];
            double[] SplitResult = new double[BlockCount];
            int[] Palette = new int[BlockCount];
            int myCounter;
            int DidExit;
            Complex dummyC = new Complex(0,0);
            double x_value, y_value;

            // initialize parameters
            double n;
            double big;
            double ratio;
            Complex C0;
            Complex C1;
            if (InheritParameters) {
                n=myMath.getVariable("n");
                big=myMath.getVariable("big");
                ratio=myMath.getVariable("ratio");
                x_value=myMath.getVariable("C0_x");
                y_value=myMath.getVariable("C0_y");
                C0=new Complex(x_value, y_value);
                x_value=myMath.getVariable("C1_x");
                y_value=myMath.getVariable("C1_y");
                C1=new Complex(x_value, y_value);
            } else {
                n=2;
                big=50;
                ratio=0.5;
                C0=new Complex(cX,cY);
                C1=new Complex(0,0);
            }

            // User-defined distinguished points are set before loop: 
            Complex p0=new Complex(0,0);

            OuterLoop:

            for (x = StartX; x < EndX; x++) {
                xval = deTool.GetValFromX(TargetFrameIndex, x);

                for (y = StartY; y < EndY; y++) {
                    yval = deTool.GetValFromY(TargetFrameIndex, y);

                    // System variables: 
                    DidExit = 0;
                    for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                        SplitResult[myCounter] = 0;
                        Palette[myCounter] = -1;
                        BlockExit[myCounter] = false;
                    }

                    // User-defined variables: 
                    Complex Z=new Complex(xval,yval);
                    Complex C=new Complex(0,0);

                    InnerLoop:

                    for (iter = 0; iter < MaxIter; iter++) {

                        Block0: // Start of Block 0
                        if (!BlockExit[0]) {

                            // 'ExitIf' Directive: exitif:SqNorm(Z)>big;palette:0;split:GetIm(Z);palette:1,2
                            if (dummyC.SqNorm(Z)>big) {
                                DidExit++;
                                BlockExit[0] = true;
                                Palette[0]=0;
                                deTool.ImageArray.PixelArray[TargetFrameIndex].iter[0][x][y] = iter;
                                SplitResult[0]=dummyC.GetIm(Z);
                                break Block0;
                            }
			if(Math.random()>ratio) {
				C=C1;
			} else {
				C=C0;
			}

                            // 'Set' Directive: set:Z=(Z^n)+C
                            Z=(Z.Pow(n)).Add(C);

                        } // End of Block 0

                        if(DidExit == BlockCount) {
                            break InnerLoop;
                        }
                    }

                    if (deTool.ImageArray.ImageStatus[TargetFrameIndex] == 2) {
                        if (DCD) {
                        }
                        // split detection code
                        for (myCounter = 0; myCounter < BlockCount; myCounter++) {
                            if (BlockExit[myCounter] && BlockSplit[myCounter]) {
                                if (SplitResult[myCounter] < 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteA[myCounter];
                                } else if (SplitResult[myCounter] > 0) {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = SplitPaletteB[myCounter];
                                } else {
                                    deTool.ImageArray.PixelArray[TargetFrameIndex].SplitPalette[myCounter][x][y] = Palette[myCounter];
                                }
                            }
                        }

                        myProgress++;
                        for (myCounter = 0; myCounter < Palette.length; myCounter++) {
                            deTool.ImageArray.PixelArray[TargetFrameIndex].Palette[myCounter][x][y] = Palette[myCounter];
                        }
                        deTool.ImageArray.ChunkArray[TargetFrameIndex].Progress[ChunkNumber] = myProgress/NumPixels;
                    } else {
                        break OuterLoop;
                    }
                } // end of Y loop
            } // end of X loop
            // Set values for distinguished points:
            deTool.ImageArray.DPointArray[TargetFrameIndex].XVal[0]=dummyC.GetRe(p0);
            deTool.ImageArray.DPointArray[TargetFrameIndex].YVal[0]=dummyC.GetIm(p0);
           } catch (Exception ex) {
              Logger.getLogger(Profiles.class.getName()).log(Level.SEVERE, null, ex);
              deTool.Log(deTool.ExToString(ex), TargetFrameIndex);
              return 0;
          }
      return 1;
  }

  public TraceData bernoulli_complex_julia_0_trace(int TargetFrameIndex, int traceIndex, int MaxIter) { 
      int ProfileIndex = 53;
      boolean InheritParameters = deTool.ImageArray.InheritParameters[TargetFrameIndex];
      int BlockCount = deTool.ImageArray.PixelArray[TargetFrameIndex].length;
      int[] Palette = new int[BlockCount];
      boolean[] BlockExit = new boolean[BlockCount];
      MathEval myMath = deTool.ImageArray.myMath[TargetFrameIndex];
      double cX = myMath.getVariable("cX");
      double cY = myMath.getVariable("cY");
      double TINY = myMath.getVariable("TINY");
      double BIG = myMath.getVariable("BIG");
      double xval = 0, yval = 0;
      int myCounter;
      Complex dummyC = new Complex(0,0);
      double x_value, y_value;

      // initialize parameters
      double n;
      double big;
      double ratio;
      Complex C0;
      Complex C1;
      if (InheritParameters) {
          n=myMath.getVariable("n");
          big=myMath.getVariable("big");
          ratio=myMath.getVariable("ratio");
          x_value=myMath.getVariable("C0_x");
          y_value=myMath.getVariable("C0_y");
          C0=new Complex(x_value, y_value);
          x_value=myMath.getVariable("C1_x");
          y_value=myMath.getVariable("C1_y");
          C1=new Complex(x_value, y_value);
      } else {
          n=2;
          big=50;
          ratio=0.5;
          C0=new Complex(cX,cY);
          C1=new Complex(0,0);
      }

      // setup trace base values
      xval = deTool.ImageArray.TraceArray[TargetFrameIndex].xval[traceIndex];
      yval = deTool.ImageArray.TraceArray[TargetFrameIndex].yval[traceIndex];
      int x = deTool.GetXFromVal(TargetFrameIndex, xval);
      int y = deTool.GetYFromVal(TargetFrameIndex, yval);

      // System variables: 
      int DidExit = 0;
      for (myCounter = 0; myCounter < BlockCount; myCounter++) {
          Palette[myCounter] = -1;
          BlockExit[myCounter] = false;
      }

      // User-defined variables: 
      Complex Z=new Complex(xval,yval);
      Complex C=new Complex(0,0);
      TraceData traceReturn = new TraceData();

      for (int iter = 0; iter < MaxIter; iter++) {
          traceReturn.add(TargetFrameIndex, dummyC.GetRe(Z), dummyC.GetIm(Z));
          // User-defined directives are executed for each iteration in inner loop: 

          Block0: // Start of Block 0
          if (!BlockExit[0]) {

              // 'ExitIf' Directive: exitif:SqNorm(Z)>big;palette:0;split:GetIm(Z);palette:1,2
              if (dummyC.SqNorm(Z)>big) {
                  DidExit++;
                  BlockExit[0] = true;
                  break Block0;
              }
			if(Math.random()>ratio) {
				C=C1;
			} else {
				C=C0;
			}

              // 'Set' Directive: set:Z=(Z^n)+C
              Z=(Z.Pow(n)).Add(C);

          } // End of Block 0
            if (DidExit == BlockCount) {
                break;
            }
      }

      return traceReturn;
    }

}